// Version mqe_java/source/examples/adapters/MQeDiskFieldsAdapter.java, MQeExamples, la200, la200-L030324.1 1.8

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2000                                                 */

/* -----------------------------------------------------------------------------*/
package examples.adapters;


import  com.ibm.mqe.*;
import  com.ibm.mqe.adapters.*;
import  java.io.*;


/********************************************************************************
 *
 * The MQeDiskFieldsAdapter is an adapter that provides support for
 * reading and writing MQeFields information to a local disk. A single
 * instance of this class refers to a single file on disk.
 * This file extends the base adapter class (com.ibm.mqe.MQeAdapter).
 *
 * @see com.ibm.mqe.MQeAdapter
 *
 ********************************************************************************/
public class MQeDiskFieldsAdapter extends MQeAdapter implements FilenameFilter {

    public static short  version[] = { 2, 0, 0, 10 }; 

    /**
     * File type filter.
     * Presumably used to describe the file extension.
     * Used by the accept() method.
     */
    protected String  filter = "";

    /**
     * The disk file name that this instance is working with.
     */
    protected String  fileName = "";

    /**
     * The drive and directory path of the file.
     */
    protected String  filePath = "";

    /**
     * True if the file is opened for reading.
     */
    protected boolean reading = false;

    /**
     * True if the file is opened for writing.
     */
    protected boolean writing = false;

    /**
     * Checks the extension of the given string to see if it ends
     * with the currently defined file filter.
     * @param dir File (not used)
     * @param name String, the name of the file to be checked.
     * @return true If the named parameter ends with the current
     * filter String.
     */
    public boolean accept(File dir, String name) {
        return (name.endsWith(filter));
    }

    /**
     * The activate() method initialises a new MQeDiskFieldsAdapter object.
     * Note that this call also sets the QOS Size parameter to be the size of
     * the file or MQeFields object.
     *
     * @param fileDesc String, the directory path to be opened.
     * Note: If you want to open a file the name of the file
     * will be specified on the open. Just the path to the file here.
     *
     * @param param Object, that sets the adapter's parameter value.
     * Can supply an MQeAttribute object here that will cause the data
     * to be encoded on disk. See the writeObject/readObject methods.
     *
     * @param options Object, String that specifies the open options:
     * <pre>
     *     MQe_Adapter_READ   - read  the file
     *     MQe_Adapter_WRITE  - write the file
     *     MQe_Adapter_UPDATE - read and write the file
     * </pre>
     * @param lrecl Not used.
     * @param norec Not used.
     * @return void
     * @exception java.lang.Exception
     */
    public void   activate(String  fileDesc,
        Object  param,
        Object  options,
        int     lrecl,
        int     noRec) throws Exception {

        // Calling our parent's activate method will give us the fileId.
        // From this we will get our file information.
        super.activate(fileDesc, param, options, lrecl, noRec);
        filePath = fileId.substring(fileId.indexOf(':') + 1);

        // This is code to get round inconsistencies in various JVM's.
        // Start by copying the path data into a temporary area. If the
        // path data ends with a file separator then remove it.
        String Temp = filePath;

        if (filePath.endsWith(File.separator))
            Temp = Temp.substring(0, Temp.length() - File.separator.length());

        // However if the path did not end with a file separator update our
        // permanent copy of the variable so that it *does* include one.
        // This means that Temp will now never end with a file separator,
        // but filePath will always end with a file separator.
        else
            filePath = filePath + File.separator;

        // Initialise a File object that represents the folder we will be working with.
        File diskFile = new File(Temp);

        // Make sure the path exists. If not, create it.
        if (!diskFile.isDirectory()) {
            if (!diskFile.mkdirs()) {

                // On some JVMs mkdirs() fails for the root directory. If it
                // failed then do some checking to see whether the failure
                // was valid.
                diskFile = new File(filePath);
                if (!diskFile.isDirectory())
                    if (!diskFile.mkdirs())
                        throw new MQeException(MQe.Except_NotAllowed,
                                "mkdirs '" + filePath + "' failed");
            }
        }

        // The user may have specified a relative path. Convert it to an
        // absolute path, because the current directory *may* be changed during
        // the lifetime of this adapter!
        filePath = diskFile.getAbsolutePath() + File.separator;

        // Add the file size to the QOS object.
        qos.putInt(MQe.QoS_Size, (int) diskFile.length());

        // The following call sets the read/write/update state.
        this.open(null);
    }

    /**
     * Closes the current file.
     * @param opt Object (not used)
     * @return void
     * @exception java.lang.Exception
     */
    public void close(Object opt) throws Exception {
        reading = false;
        writing = false;
    }

    /**
     * Allows the user to set the file filter or to
     * list the files in the current file's directory.
     *
     * @param opt String options. One of:
     * <pre>
     *   MQe_Adapter_FILTER - set the filter to the value of the
     *                        second parameter.
     *   MQe_Adapter_LIST   - to list the files.
     * </pre>
     * @param ctrlObj Object String that states the filter
     * to use (if MQe_Adapter_FILTER was specified). Not used
     * for MQe_Adapter_LIST.
     *
     * @return null for MQe_Adapter_FILTER, or the list
     * of files in the case of MQe_Adapter_LIST.
     * @exception java.lang.Exception
     */
    public Object control(Object opt, Object ctrlObj) throws Exception {

        // LIST
        if (checkOption(opt, MQeAdapter.MQe_Adapter_LIST))
            return (new File(filePath).list(this));

        // FILTER
        else if (checkOption(opt, MQeAdapter.MQe_Adapter_FILTER)) {
            filter = (String) ctrlObj;
            return (null);
        } // EXISTS
        else if (checkOption(opt, MQeAdapter.MQe_Adapter_EXISTS)) {
            // check to see if the File exists
            return new Boolean(new File((String) ctrlObj).exists());
        } // RENAME (need to have filter set to name of New File)
        else if (checkOption(opt, MQeAdapter.MQe_Adapter_RENAME)) {
            File oldF = new File((String) ctrlObj);

            oldF.renameTo(new File(filter));
            return null;
        } // UNKNOWN (pass to parent to sort out)
        else
            return (super.control(opt, ctrlObj));
    }

    /**
     * Erases a file from disk.
     * @param opt Object String that contains the file name to be deleted.
     *
     * @return void
     * @exception java.lang.Exception
     * <p><b>Note:</b> Before passing to the OS's delete routine, the file name is
     * appended to the current FileDesc value.
     * <p>Does not work for directories.
     */
    public void erase(Object opt) throws Exception {
        if (opt instanceof String) {
            String FN = (String) opt;

            // Don't allow directories to be deleted.
            if (FN.indexOf(File.separator) > -1)
                throw new MQeException(MQe.Except_Syntax, "Not allowed");

            MQeTrace.trace(this, (short) 0, MQeTrace.GROUP_INFO, filePath + FN); // "Deleting this file..."

            // Try to delete the file.
            if (!new File(filePath + FN).delete())
                throw new MQeException(MQe.Except_NotAllowed, "Erase failed");
        } else
            throw new MQeException(MQe.Except_NotSupported, "Not supported");
    }

    /**
     * Opens a file for use, after having first closed any open file.
     * @param opt Object String that specifies the filename to be
     * appended to the FileDesc.
     * @return void
     * @exception java.lang.Exception
     */
    public void open(Object opt) throws Exception {

        // Close the file if it was currently open, and clear the reference to it.
        this.close(null);
        fileName = null;

        // Open the file object.
        if (opt instanceof String) {
            fileName = (String) opt;

            MQeTrace.trace(this, (short) 0, MQeTrace.GROUP_INFO, filePath + fileName); // "Opening this file..."

            // Allocate a new file object and update the QOS object with the
            // file size.
            File diskFile = new File(filePath + fileName);

            qos.putInt(MQe.QoS_Size, (int) diskFile.length());
        }

        // Get the correct values of the read and write states.
        reading = checkOption(MQeAdapter.MQe_Adapter_READ) ||
                checkOption(MQeAdapter.MQe_Adapter_UPDATE);
        writing = checkOption(MQeAdapter.MQe_Adapter_WRITE) ||
                checkOption(MQeAdapter.MQe_Adapter_UPDATE);
    }

    /**
     * Reads an MQeFields object from the current file.
     *
     * @param opt Object MQeFields object that can be used to
     * provide a filter within the directory object. Otherwise null.
     *
     * @return Object The MQeFields object that was read.
     * @exception java.lang.Exception
     */
    public synchronized Object readObject(Object opt) throws Exception {

        // First check that the file or directory is opened for reading.
        if (reading) {

            // Does the user want us to match a supplied MQeFields?
            // If so, we need to
            // 1. list all files in the directory
            // 2. Read each file in turn and restore as a Fields object
            // 3. Check the filter - if equal then return that object
            if (opt instanceof MQeFields) {

                String List[] = new File(filePath).list(this);
                MQeFields Fields = null;

                // Go through the list of files.
                for (int i = 0; i < List.length; i = i + 1)
                    try {

                        // Open each file and read the MQeFields object.
                        fileName = List[i];
                        open(fileName);
                        Fields = (MQeFields) readObject(null);

                        // Compare the MQeFields just read with
                        // the one the user specified.
                        if (Fields.includes((MQeFields) opt)) {

                            MQeTrace.trace(this, (short) 0, MQeTrace.GROUP_INFO, fileName);
                            return (Fields);
                        }
                    } catch (Exception e) {// Trace the exception
                    }
                throw new MQeException(Except_NotFound, "No match");
            }

            // If we get here then the user didn't specify a match.
            // Just open the file and read the bytes from disk.
            File diskFile = new File(filePath + fileName);

            // Read the data bytes into an array.
            byte data[] = new byte[(int) diskFile.length()];
            FileInputStream InputFile = new FileInputStream(diskFile);

            InputFile.read(data);

            // Close the file and update the bytes read field in the QOS object.
            InputFile.close();
            qosIncrement(MQe.QoS_BytesRead, data.length);

            // Feed the MQeFields through the attribute decoder
            data = attributeDecode(data);

            // Get the MQeFields from the bytes and return it.
            MQeFields FieldsObject = MQeFields.reMake(data, null);

            MQeTrace.trace(this, (short) 0, MQeTrace.GROUP_INFO, FieldsObject.toString());
            return (FieldsObject);
        } // The file is not open for reading.
        else
            throw new MQeException(MQe.Except_NotSupported, "Not supported");
    }

    /**
     * Provides information of the currently active settings.
     *
     * @param opt Object String, one of the following :
     * <pre>
     * MQe_Adapter_FILTER - to get the name of the filter.
     * MQe_Adapter_FILENAME - to get the name of the file.
     * </pre>
     * @return String either the name of the filter or the name of the file.
     * @exception java.lang.Exception
     */
    public String status(Object  opt) throws Exception {

        // FILTER
        if (checkOption(opt, MQeAdapter.MQe_Adapter_FILTER))
            return (filter);

        // FILENAME
        if (checkOption(opt, MQeAdapter.MQe_Adapter_FILENAME))
            return (fileName);

        // UNKNOWN (ask parent)
        return (super.status(opt));
    }

    /**
     * Writes an MQeFields object to disk.
     * @param opt Object not used.
     * @param data Object MQeFields object to dump to disk.
     * @return void
     * @exception java.lang.Exception
     */
    public synchronized void writeObject(Object  opt,
        Object  data) throws Exception {

        // Only allow the operation if the user supplied an MQeFields
        // object and the current file is available for writing.
        if (writing && (data instanceof MQeFields)) {

            // Convert the MQeFields object into a byte array
            byte dump[] = ((MQeFields) data).dump();

            // Encode the data using any active attributes.
            dump = attributeEncode(dump);

            // Update the QOS size attribute with the file size
            // before opening the file and dumping the bytes out to it.
            qos.putInt(MQe.QoS_Size, dump.length);

            File diskFile = new File(filePath + fileName);
            FileOutputStream OutputFile = new FileOutputStream(diskFile);

            OutputFile.write(dump);

            // Synchronise the update with any cache stuff that may be around.
            OutputFile.getFD().sync();
            OutputFile.close();

            // Update the QOS object with the number of bytes written.
            qosIncrement(MQe.QoS_BytesWritten, dump.length);

            MQeTrace.trace(this, (short) 0, MQeTrace.GROUP_INFO, data.toString());
        } else {
            // Thrown if the file is not available for writing or if
            // an MQeFields object was not supplied.
            throw new MQeException(MQe.Except_NotSupported, "Not supported");
        }
    }

}
