package examples.adapters;


//Version mqe_java/source/examples/adapters/MQeTcpipLengthGUIAdapter.java, MQeExamples, la200, la200-L030324.1 1.3
/* -----------------------------------------------------------------------------*/

/*                   WebSphere MQ Everyplace for Multiplatforms                 */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/* -----------------------------------------------------------------------------*/

import com.ibm.mqe.MQePropertyProvider;
import com.ibm.mqe.adapters.MQeCommunicationsAdapter;
import java.net.*;
import java.io.*;
import com.ibm.mqe.*;


/**
 * The MQeTcpipLengthGUIAdapter is a simple example of how to create a communications
 * adapter to be used with MQe.  There is no rigorous error checking, neither does
 * the adapter correctly deal with the situation where the data length to be written
 * is greater than the length of buffer to use.
 *
 */
public class MQeTcpipLengthGUIAdapter extends MQeCommunicationsAdapter {

    public static short  version[] = { 2, 0, 0, 10 };  // MQe version

    // network information
    private int port; // port number
    private String address; // address of server
    private BufferedInputStream input = null; // input stream for reading
    private BufferedOutputStream output = null; // output stream for writing
    private Socket mySocket = null; // client socket
    private ServerSocket serverSocket = null; // server socket
    private int nonBlockingTimeout = 1000; // socket timeout to prevent blocking

    // synchronization objects
    private Object readLock = null;
    private Object writeLock = null;

    // other
    private String greeting;
    private static String GREETING = "examples.adapters.MQeTcpipLengthGUIAdapter";

    // configurable values
    private int defaultPacketSize = 4096;
    private int defaultTimeout = 5000;
    private int adapterTimeout;
    private int adapterPacketSize;

    /**
     * @see com.ibm.mqe.adapters.MQeCommunicationsAdapter#activate(MQePropertyProvider)
     */
    public void activate(MQePropertyProvider info) throws Exception {

        // This method is called by WebSphere MQ Everyplace just once in order
        // to activate the adapter.  The MQeCommunicationsAdapter will have set
        // listeningAdapter to true if we have to listen and the persistAdapter
        // variable if the network connection is to persist over calls to the adapter.

        if (!listeningAdapter) {
            // if we are not a listening adapter we need the address of the server
            address = info.getProperty(MQeCommunicationsAdapter.COMMS_ADAPTER_ADDRESS);
        }

        // In all instances we need the port number
        String temp = info.getProperty(MQeCommunicationsAdapter.COMMS_ADAPTER_PORT);

        if (null == temp) {
            port = 8080;
        } else {
            port = new Integer(temp).intValue();
        }
        greeting = info.getProperty(GREETING);
        if (null == greeting) {
            greeting = GREETING;
        }

        // see if we have a packet size set, otherwise we will use the default

        temp = info.getProperty(MQeCommunicationsAdapter.COMMS_ADAPTER_PKTSIZE);
        if (null == temp) {
            adapterPacketSize = defaultPacketSize;
        } else {
            adapterPacketSize = new Integer(temp).intValue();
        }

        // see if the adapter timeout has been set, if not use the default
        temp = info.getProperty(MQeCommunicationsAdapter.COMMS_ADAPTER_TIMEOUT);
        if (null == temp) {
            adapterTimeout = defaultTimeout;
        } else {
            adapterTimeout = new Integer(temp).intValue();
        }

        // create the objects required to synchronize the read and write
        readLock = new Object();
        writeLock = new Object();

    }

    /**
     * @see com.ibm.mqe.adapters.MQeCommunicationsAdapter#close()
     */
    public void close() throws Exception {
        try {
            // close the server socket if applicable
            if (null != serverSocket) {
                serverSocket.close();
                serverSocket = null;
            }

            // close the client socket if applicable
            if (null != mySocket) {
                mySocket.close();
                mySocket = null;
            }

            // close the input stream
            if (null != input) {
                input.close();
                input = null;
            }

            // close the output stream
            if (null != output) {
                output.close();
                output = null;
            }
        } catch (Exception e) {
            // not worried about an exception, but make sure we know about it
            e.printStackTrace();
        }
    }

    /**
     * @see com.ibm.mqe.adapters.MQeCommunicationsAdapter#open()
     */
    public void open() throws Exception {

        // if the socket is a null then create the required socket
        // the responder will have it's socket set by the listening
        // adapter
        if (listeningAdapter && null == serverSocket) {
            serverSocket = new ServerSocket(port);
            serverSocket.setSoTimeout(adapterTimeout);
        } else if (!responderAdapter && null == mySocket) {
            mySocket = new Socket(InetAddress.getByName(address), port);
            mySocket.setSoTimeout(adapterTimeout);
        }

    }

    /**
     * @see com.ibm.mqe.adapters.MQeCommunicationsAdapter#read()
     */
    public byte[] read() throws Exception {

        // create the buffered input stream
        input = new BufferedInputStream(mySocket.getInputStream());

        // create the byte array in order to read the first four bytes
        // of data which will contain the length of data to read
        byte[] dataLength = new byte[4];
        int bytesRead = 0;

        // we are about to read so synchronize
        synchronized (readLock) {
            do {
                try {
                    bytesRead = input.read(dataLength);
                } catch (InterruptedIOException iioe) {
                    if (MQeThread.getDemandStop()) {
                        throw iioe;
                    }
                }
            }
            while (bytesRead == 0);

            // turn the four bytes read into an integer and create the
            // array to read the rest of the data
            int length = MQe.byteToInt(dataLength, 0);
            byte[] data = new byte[length];

            // read the data from the buffered intput stream
            length = input.read(data);
            System.out.println(length + " bytes read");

            // return the data
            return data;
        }

    }

    /**
     * @see com.ibm.mqe.adapters.MQeCommunicationsAdapter#temporaryClose()
     */
    public void temporaryClose() throws Exception {

        // only call the close method if we are a non-persistent adapter
        if (!persistentAdapter) {
            close();
        }
    }

    /**
     * @see com.ibm.mqe.adapters.MQeCommunicationsAdapter#waitForContact(MQePropertyProvider)
     */
    public MQeCommunicationsAdapter waitForContact(MQePropertyProvider info) throws Exception {
        // create the client socket then wait for an incoming connection request
        Socket clientSocket = null;

        do {
            try {
                clientSocket = serverSocket.accept();
            } catch (InterruptedIOException iioe) {
                if (MQeThread.getDemandStop()) {
                    throw iioe;
                }
            }
        }
        while (null == clientSocket);

        // request received so create a new adapter
        MQeTcpipLengthGUIAdapter clientAdapter =
            (MQeTcpipLengthGUIAdapter) MQeCommunicationsAdapter.createNewAdapter(info);

        // set the boolean variables so the adapter knows it is a responder. the listening
        // variable will have been set to true as the MQePropertyProvider has the relevant
        // information to create this listening adapter.  We must therefore reset the
        // listeningAdapter variable to false and the responderAdapter variable to true.
        clientAdapter.responderAdapter = true;
        clientAdapter.listeningAdapter = false;

        // Assign the new socket to this new adapter
        clientAdapter.setSocket(clientSocket);
        return clientAdapter;
    }

    /**
     * @see com.ibm.mqe.adapters.MQeCommunicationsAdapter#writeData(byte[])
     */
    public void writeData(byte[] data) throws Exception {

        // create a buffered output stream and get the length of data to write
        output = new BufferedOutputStream(mySocket.getOutputStream());
        synchronized (writeLock) {
            int length = data.length;

            if (length > adapterPacketSize) {
                // For the purposes of the example we truncate data that is longer than
                // the packet size - default of 4096
                System.out.println("data longer than packet, data being truncated");
            }
            System.out.println("Length of data being written is: " + length);

            // create a byte array to hold the length of the data
            byte[] outputData = new byte[4 + length];

            // copy the length and the data to a new byte array.
            System.arraycopy(MQe.intToByte(length), 0, outputData, 0, 4);
            System.arraycopy(data, 0, outputData, 4, length);

            // write the byte array to the buffered output stream and flush the stream
            // so it is all written to the network
            output.write(outputData);
            output.flush();
        }
    }

    /**
     * @see com.ibm.mqe.adapters.MQeCommunicationsAdapter#writeResponse(byte[])
     */
    public void writeResponse(byte[] data) throws Exception {
        // we do not do anything special on writing a response, so just route the call
        // to write data
        this.writeData(data);
    }

    protected void setSocket(Socket newSocket) throws Exception {
        // set the socket
        mySocket = newSocket;
        mySocket.setSoTimeout(adapterTimeout);
    }

}
