//Version mqe_java/source/examples/administration/commandline/ConnectionCreator.java, MQe, la200, la200-L030414.1 1.8

/*------------------------------------------------------------------------------
 *                    WebSphere MQ Everyplace for Multiplatforms                
 *                                                                              
 * Licensed Materials - Property of IBM                                         
 *                                                                              
 * 5765-E63                                                                     
 *                                                                              
 * (C) Copyright IBM Corp. 2001                                                 
 *                                                                              
 *------------------------------------------------------------------------------*/

package examples.administration.commandline; 


import com.ibm.mqe.*; 
import com.ibm.mqe.administration.*;  

import examples.queuemanager.*;


/**
 * A command-line tool to allow someone to add a connection to an MQe queue manager without programming
 * anything in Java.
 * 
 * <b><p>Syntax</b></p>
 * <P><TT>java examples.administration.commandline.ConnectionCreator
 * &lt;connectionName&gt; &lt;qMgr&gt; &lt;adapter&gt; &lt;channel&gt; &lt;ini_file&gt;
 * </TT></P>
 * 
 * <dl>
 * 
 * <dt>connectionName</dt>
 * <dd>The name of connection which is to be created.</dd>
 * 
 * <dt>qMgr</dt>
 * <dd>The MQe queue manager to which the connection will be added.
 * Normally this would be the same as the queue manager that this example runs.</dd>
 * 
 * <dt>adapter</dt>
 * <dd>The adapter definition, describing how the connection
 * is to be made. This will be of the form Network:hostName:portNumber
 * normally. If you wish the adapter to be a "null" adapter, then use a
 * value of "null" here.</p></dd>
 * 
 * <dt>channel</dt>
 * <DD>The name of the channel class (or a class alias) which will be used
 * to establish the connection. If you wish to have a null channel, then use the
 * special value of "null" here.</DD>
 * 
 * <DT>iniFile</DT>
 * <DD>The fully qualified path of the .ini file used by this tool to instantiate
 * a device queue manager, send the admin message, and wait for the reply.
 * </DD>
 * 
 * </dl>
 * 
 * <B><P>Examples</B></P>
 * <P>This tool would normally be used to create a straight connection from the device
 * queue manager to a server queue manager for example:
 * ...ConnectionCreator MyServerQM MyDeviceQM Network:MyServer:MyServersPort DefaultChannel C:\MQe\...\MyDeviceQM.ini
 * </P>
 * 
 * <P>It can also be used to create a "via" routing... where the MyServer2QM is sent messages via MyServerQM...
 * ...ConnectionCreator MyServer2QM MyDeviceQM MyServerQM DefaultChannel C:\MQe\...\MyDeviceQM.ini<P>
 * 
 * <P>It can also be used to create a peer channel listener on the device queue manager, say for port 8085...
 * ...ConnectionCreator MyDeviceQM MyDeviceQM Network::8085 com.ibm.mqe.MQePeerChannel C:\MQe\...\MyDeviceQM.ini<P>
 */
public class ConnectionCreator extends examples.administration.commandline.MQeAbstractCmdLineAdminTool { 

    public static short[] version = { 2, 0, 0, 10 }; 

    /**
     * Entry point from the command line.
     * 
     * @param args   Command-line arguments being passed in.
     * @return void
     */
    public static void main(String args[]) {
        int rc = 0;
        ConnectionCreator me = new ConnectionCreator();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    } 

    /**
     * Does all the work.
     * 
     * @param args   Command-line arguments to be used.
     * @return void
     * @exception Exception
     */
    public void activate(String args[]) throws Exception {

        String connection = null;
        String qmgr = null;
        String iniFile = null;
        String channel = null;
        String adapter = null;

        // Get the parameters from the system properties...
        if (args.length < 5) {
            throw new Exception("Error: Not enough parameters. "
                    + "Syntax :ConnectionCreator <connection_name> <qmgr> <adapter> <channel> <ini_file>"
                );
        }
        connection = args[0].trim();
        qmgr = args[1].trim();
        adapter = args[2].trim();
        channel = args[3].trim();
        iniFile = args[4].trim();

        System.out.println("i: Attempting to define a connection " + connection + " using adapter "
            + adapter + " and channel " + channel + " on qmgr " + qmgr + " using the " + iniFile + " ini file.");

        // Access the local MQe queue manager...
        MQeClient client = new MQeClient(iniFile, true // Suppress stdout.
            );

        // Access the queue manager created by the MQeClient class...
        MQeQueueManager myQMgr = MQeClient.queueManager;

        // Create the queue...
        createConnection(connection, adapter, channel, qmgr, myQMgr);

        // Close down the MQeClient
        client.close();
    } 

    /**
     * Creates an admin message which will cause a connection definition to be created
     * then it sends it to a queue manager.
     * 
     * @param connection The name of the connection definition to be created.
     * 
     * @param adapter The adapter definition, describing how the connection
     *                is to be made. This will be of the form Network:hostName:portNumber
     *                normally. If you wish the adapter to be a "null" adapter, then use a
     *                value of "null" here.
     * 
     * @param channel The name of the channel class (or a class alias) which will be used
     *                to establish the connection. If you wish to have a null channel, then use the
     *                special value of "null" here.
     * 
     * @param targetQMgr The name of the queue manager to which the admin message will be sent.
     * 
     * @param myQMgr The reference to an active MQe queue manager object, which will be
     *               used to send the admin message, and receive the reply.
     * @return void
     * @exception Exception
     */
    static public void createConnection(String connection, String adapter, String channel, String targetQMgr, MQeQueueManager myQMgr
    ) throws Exception {

        System.out.println("Building the admin message");

        MQeConnectionAdminMsg msg = new MQeConnectionAdminMsg(connection);

        MQeFields params = new MQeFields();

        // Fill the params with any other information we need for our queue definition.
        if (channel.equals("null")) {
            params.putAscii(MQeConnectionAdminMsg.Con_Channel, null);
        } else {
            params.putAscii(MQeConnectionAdminMsg.Con_Channel, channel);
        }

        if (adapter.equals("null")) {// Do nothing...
        } else {
            MQeFields adapters[] = new MQeFields[1];

            adapters[0] = new MQeFields();
            adapters[0].putAscii(MQeConnectionAdminMsg.Con_Adapter, adapter);
            params.putFieldsArray(MQeConnectionAdminMsg.Con_Adapters, adapters);
        }

        // Say this admin message should create the connection...
        msg.create(params);

        // Set the name of the connection.
        msg.setName(connection);

        System.out.println("Built the admin message.");

        // Send the message.
        MQeFields results = sendWaitMessage(myQMgr, targetQMgr, msg);
        
        System.out.println("Connection created successfully.");        
    } 
} 
