// Version mqe_java/source/examples/administration/commandline/IniFileCreator.java, MQe, la200, la200-L030624 1.20

/*------------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms                *
 *                                                                              *
 * Licensed Materials - Property of IBM                                         *
 *                                                                              *
 * 5765-E63                                                                     *
 *                                                                              *
 * (C) Copyright IBM Corp. 2001                                                 *
 *                                                                              *
 *------------------------------------------------------------------------------*/
package examples.administration.commandline;


import com.ibm.mqe.*;
import examples.queuemanager.*; 
import java.util.*;


/**
 * A command-line tool to create the simplest MQe queue manager .ini files possible.
 * 
 * <P><B>Syntax</P></B>
 * <P><TT>java examples.administration.commandline.IniFileCreator
 * &lt;iniFilePath&gt;
 * &lt;qMgrName&gt;
 * &lt;registryDir&gt;
 * &lt;qMgrType&gt;
 * &lt;port&gt;
 * &lt;pin&gt;
 * &lt;keyRingPassword&gt;
 * &lt;msgLogAlias&gt;
 * &lt;registryType&gt;
 * &lt;listeningAdapter&gt;
 * &lt;registryAdapter&gt;
 * &lt;adminQStore&gt; 
 * &lt;adminReplyQStore&gt;
 * &lt;deadLetterQStore&gt;
 * &lt;systemDefaultLocalQStore&gt;
 * &lt;propertyList&gt;
 * </TT></P>
 * <B><P>Parameters</p></B>
 * <DL>
 * <DT>iniFilePath</DT>
 * <DD>The fully qualified path name of the .ini file we are creating.</DD>
 * 
 * <DT>qMgrName</DT>
 * <DD>The name of the queue manager in the ini file.</DD>
 * 
 * <DT>registryDir</DT>
 * <DD>The name of the directory which will be used to store the queue manager registry information.
 * Note: This must have a trailing slash character to be valid.</DD>
 * 
 * <DT>qMgrType</DT>
 * <DD>The type of queue manager to be created.
 * Can be either "Device", "Server" or "Gateway"
 * <DL>
 * <DT>Device</DT>
 * <DD>A queue manager which does not have a channel manager.
 * Such queue managers are incapable of creating the server-end of a client-server channel.</DD>
 * <DT>Server</DD>
 * <DD>A queue manager which is not bridge-enabled, but does have a channel manager configured.
 * Such queue managers (unlike the 'device' style of queue manager) is capable of creating the
 * server-end of a client-server channel.</DD>
 * <DT>Gateway</DT>
 * <DD>Essentially a 'server'-style queue manager (as above) which is enabled with a bridge to MQSeries.
 * The MQSeries classes are assumed to be available for such MQe queue managers to operate.
 * Specialist administration console which can administer bridge objects are
 * detailed in gateway ini files. Same as "Server" from v2.0 onwards.</DD>
 * </DL>
 * </DD>
 * 
 * <DT>port</DT>
 * <DD>The port the queue manager will listen on, or 0 if the QM is a device queue manager.</DD>
 * 
 * <DT>pin</DT>
 * <DD>The Pin used to open the registry.</DD>
 * 
 * <DT>keyRingPassword</DT>
 * <DD>The password used to access secure keys within the registry.</DD>
 * 
 * <DT>msgLogAlias</DT>
 * <DD>What the "MsgLog" class alias should map to for this queue manager.
 * Use "default" to set it to the usual value.
 * </DD>
 *
 * <DT>registryType</DT>
 * <DD>The type of the registry to be used to hold the queue manager configuration.
 * This must be either "PrivateRegistry" or "FileRegistry". FileRegistry should be thought of as the default.
 * <DL>
 * <DT>PrivateRegistry</DT>
 * <DD>Used when you want an encrypted registry</DD>
 * <DT>FileRegistry</DT>
 * <DD>Used when you don't want to use encryption, or if you want to use anything other than the
 * default type of storage adapter under-pinning the configuration data registry</DD>
 * </DL>
 *
 * <DT>listeningAdapter</DT>
 * <DD>The MQe communications adapter which is used to create the server-side of all client-server
 * connections made to this queue manager. This parameter will not be used for device queue managers,
 * as they don't create a listener section in their .ini files.
 * This can also be the alias name of a communications adapter.
 * For example: "Network" or "FastNetwork"
 * </DD>
 * 

 * </DL>
 * 
 * <P><B>Example</B></P>
 * <P><TT>Java ...IniFileCreator C:\sMQe\QMgrs\MyQM\MyQM.ini MyQM C:\MQe\QMgrs\MyQM\Registry\
 * Server 8085 12345678 my_secret default FileRegistry FastNetwork 
 * </TT></P>
 * <P>Creates an .ini file for a device queue manager "MyQM".</P>
 */
public class IniFileCreator extends examples.administration.commandline.MQeAbstractCmdLineAdminTool {

    public static short version[] = { 2, 0, 0, 10 };  

    public static final String syntax =
        "\nSyntax: \n"
        + "IniFileCreator <iniFileName> <qMgrName> <registryDir> <qMgrType>\n"
        + "               <port> <pin> <keyRingPassword> <msgLogAlias>\n"
        + "               <registryType> <listenerAdapter> <registryAdapter>\n"
        + "               [<propertyName>=<propertyValue>]*\n";

    /**
     * Allows the tool to be called from the command-line.
     */
    public static void main(String args[]) {
        try {
            IniFileCreator me = new IniFileCreator();

            me.activate(args);
        } catch (Exception e) {
            System.out.println("Failed to create the ini file !. " + e.toString());
            System.err.println("\nStack trace : \n");
            e.printStackTrace();
        }
    }

    /**
     * Uses the command-line parameters to call the method doing the work.
     * 
     * @param args   Array of string arguments from the command-line.
     * @return void
     * @exception Exception
     */
    public void activate(String args[]) throws Exception {

        // Name of the ini file we will create.
        String iniFileName = null;

        // Name of the queue manager we will create.
        String qMgrName = null;

        // Directory in which we put registry information.
        String registryDir = null;

        // The type of the queue manager. Device or Gateway.
        String qMgrType = null;

        // The port to be used, or 0 if the QM is a device queue manager.
        String portString = null;

        // The password used to access secure keys within the registry.
        String keyRingPassword = null;

        // The Pin used to open the registry.
        String pin = null;

        String msgLogAlias = null;

        // Whether it's an encrypted registry or a normal one.
        String registryType = null;

        // The type of adapter the listener uses. 
        String listenerAdapter = null;

        if (args.length < 10) {
            throw new Exception("Not enough parameters" + syntax);
        }

        iniFileName = args[0].trim();
        qMgrName = args[1].trim();
        registryDir = args[2].trim();
        qMgrType = args[3].trim();
        portString = args[4].trim();
        pin = args[5].trim();
        keyRingPassword = args[6].trim();
        msgLogAlias = args[7].trim();
        registryType = args[8].trim();
        listenerAdapter = args[9].trim();
        
        String registryAdapter = null;

        if (args.length > 10) {
            registryAdapter = args[10].trim();
        }
		
        // All the remaining parameters are system properties...
        Hashtable properties = new Hashtable();

        if (args.length > 11) {
            int index = 11;		
            int numberOfProperties = args.length - index;	

            while (index < args.length) {

                String parameter = args[index].trim();
                int equalsIndex = parameter.indexOf('=');

                if (equalsIndex >= 0) {
                    // The argument is of the form propert-name=propery-value.
                    String propertyName = parameter.substring(0, equalsIndex);
                    String propertyValue = parameter.substring(equalsIndex + 1, parameter.length());

                    properties.put(propertyName, propertyValue);
                }								
                index += 1;
            }
        }					

        // Check that the port string is an integer...
        int port = Integer.parseInt(portString);

        if (qMgrType.equals("Device")) {
            createDeviceQMIniFile(
                iniFileName,
                qMgrName,
                registryDir,
                pin,
                keyRingPassword,
                msgLogAlias,
                registryType,
                registryAdapter,
                properties				
            );
        } else {

            createServerQMIniFile(
                iniFileName,
                qMgrName,
                registryDir,
                portString,
                pin,	
                keyRingPassword,
                msgLogAlias,
                registryType,
                registryAdapter,
                listenerAdapter,
                properties
            );
        }		
    }

    /**
     * Creates an .ini file for the device queue manager.
     * 
     * @param iniFileName
     *                 A string containing the fully-qualified or relative- path name of the .ini file
     *                 which is going to be created.
     * @param qMgrName A string containing the name of the MQe queue manager name for which the .ini file
     *                 is being produced.
     * @param registryDir
     *                 A string containing the fully-qualified path name of the directory in which registry
     *                 information will be placed.
     *                 This parameter (rather unusually) requires a trailing directory-separator character.
     * @param pin    The PIN number to encrypt the registry with.
     * @param keyRingPassword
     *                 The keyring password to use when accessing keys within the registry.
     * @param msgLogAlias
     *                 What the "MsgLog" class alias should map to for this queue manager.
     *                 Use "default" to set it to the usual value.
     *                 
     *                 
     * @param registryType
     *                 The name of the type of registry session we will use. "FileRegistry" or "PrivateRegistry" are good values.
     * @return void
     * @exception Exception
     */
    public void createDeviceQMIniFile(
        String iniFileName,
        String qMgrName,
        String registryDir,
        String pin,
        String keyRingPassword,
        String msgLogAlias,
        String registryType,
        String registryAdapter,
        Hashtable properties		
    ) throws Exception {

        MQeFields iniFile = new MQeFields();

        MQeFields aliasSection = new MQeFields();

        // aliasSection.putAscii("EventLog", "examples.log.LogToDiskFile");

        // Only put the alias out if it's not the same as the default.
        if ((!msgLogAlias.equals("default")) && (!msgLogAlias.equals("null"))) {
            aliasSection.putAscii("MsgLog", msgLogAlias);
        }

        iniFile.putFields("Alias", aliasSection);

        MQeFields qMgrSection = new MQeFields();

        qMgrSection.putAscii("Name", qMgrName);
        iniFile.putFields("QueueManager", qMgrSection);

        MQeFields registrySection = new MQeFields();

        registrySection.putAscii("LocalRegType", registryType);
        registrySection.putAscii("DirName", registryDir);
        registrySection.putAscii("PIN", pin);
        registrySection.putAscii("KeyRingPassword", keyRingPassword);
        if (registryAdapter != null) {
            registrySection.putAscii("Adapter", registryAdapter);
        } else {
            registrySection.putAscii("Adapter", "MsgLog");
        }
        iniFile.putFields("Registry", registrySection);

        MQeFields propertiesSection = new MQeFields();

        propertiesSection.putAscii("example.property", "example property value");
        Enumeration propKeys = properties.keys();

        while (propKeys.hasMoreElements()) {
            String propName = (String) propKeys.nextElement();
            String propValue = (String) properties.get(propName);

            propertiesSection.putAscii(propName, propValue);
        }
        iniFile.putFields("SystemProperties", propertiesSection);

        // Generate the .ini file
        MQeQueueManagerUtils.saveConfigFile(iniFileName, iniFile);
    }

    /**
     * Creates an .ini file for the server queue manager (with no bridge function).
     * 
     * @param iniFileName
     *                   A string containing the fully-qualified or relative- path name of the .ini file
     *                   which is going to be created.
     * @param qMgrName A string containing the name of the MQe queue manager name for which the .ini file
     *                 is being produced.
     * @param registryDir
     *                   A string containing the fully-qualified path name of the directory in which registry
     *                   information will be placed.
     *                   This parameter (rather unusually) requires a trailing directory-separator character.
     * @param portString A string containing an integer number.
     *                   Indicates which port the server queue manager will set it's client-server listener and channel manager up on.
     *                   For example, 8082
     * @param pin    The PIN number to encrypt the registry with.
     * @param keyRingPassword
     *                   The keyring password to use when accessing keys within the registry.
     * @param msgLogAlias
     *                   What the "MsgLog" class alias should map to for this queue manager.
     *                   Use "default" to set it to the usual value.
     * @param registryType
     *                   The name of the type of registry session we will use. "FileRegistry" or "PrivateRegistry" are good values.
     * @param listenerAdapter
     *                   A String holding the class name (or alias of a class name) of an MQe communications adapter.
     *                   Examples are: "Network" , "FastNetwork" , "com.ibm.mqe.adpaters.MQeHTTPTcpipAdapter"
     *                   Note that the .class extension is not specified.
     * @return void
     * @exception Exception
     */
    public void createServerQMIniFile(
        String iniFileName,
        String qMgrName,
        String registryDir,
        String portString,
        String pin,
        String keyRingPassword,
        String msgLogAlias,
        String registryType,
        String registryAdapter,
        String listenerAdapter,
        Hashtable properties
    ) throws Exception {

        MQeFields iniFile = new MQeFields();

        MQeFields aliasSection = new MQeFields();

        // aliasSection.putAscii("EventLog", "examples.log.LogToDiskFile");

        if ((!msgLogAlias.equals("default")) 
            && (!msgLogAlias.equals("null"))) {
            aliasSection.putAscii("MsgLog", msgLogAlias);
        }
        iniFile.putFields("Alias", aliasSection);

        MQeFields qMgrSection = new MQeFields();

        qMgrSection.putAscii("Name", qMgrName);
        iniFile.putFields("QueueManager", qMgrSection);

        MQeFields channelMgrSection = new MQeFields();

        channelMgrSection.putInt("MaxChannels", 0);
        iniFile.putFields("ChannelManager", channelMgrSection);

        MQeFields registrySection = new MQeFields();

        registrySection.putAscii("LocalRegType", registryType);
        registrySection.putAscii("DirName", registryDir);
        registrySection.putAscii("PIN", pin);
        registrySection.putAscii("KeyRingPassword", keyRingPassword);
        if (registryAdapter != null) {
            registrySection.putAscii("Adapter", registryAdapter);
        } else {
            registrySection.putAscii("Adapter", "MsgLog");
        }
        iniFile.putFields("Registry", registrySection);

        MQeFields listenerSection = new MQeFields();

        listenerSection.putInt("TimeInterval", 300);
        listenerSection.putAscii("Listen", listenerAdapter + "::" + portString);
        listenerSection.putAscii("Network", listenerAdapter + ":");
        iniFile.putFields("Listener", listenerSection);

        MQeFields propertiesSection = new MQeFields();

        propertiesSection.putAscii("example.property", "example property value");
        Enumeration propKeys = properties.keys();

        while (propKeys.hasMoreElements()) {
            String propName = (String) propKeys.nextElement();
            String propValue = (String) properties.get(propName);

            propertiesSection.putAscii(propName, propValue);
        }
        iniFile.putFields("SystemProperties", propertiesSection);

        // Generate the .ini file
        MQeQueueManagerUtils.saveConfigFile(iniFileName, iniFile);
    }
}
