package examples.administration.commandline;


// Version mqe_java/source/examples/administration/commandline/ListenerDelete.java, MQe, la200, la200-L030324.1 1.6
/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/*------------------------------------------------------------------------------*/

import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;


/**
 * A command-line tool to allow someone to delete a listener from an MQe queue manager without programming
 * anything in Java.  This version uses admin messages to send the command to the destination
 * queue manager, and so can be used to remove a listener from a remote queue manager.
 *
 * <b><p>Syntax</b></p>
 * <P><TT>java examples.administration.commandline.ListenerDelete
 * &lt;listenerName&gt; &lt;qMgr&gt; &lt;ini_file&gt;
 * </TT></P>
 *
 * <dl>
 *
 * <dt>listenerName</dt>
 * <dd>The name of listener which is to be deleted.</dd>
 *
 * <dt>qMgr</dt>
 * <dd>The MQe queue manager on which the listener will be deleted.
 * Normally this would be the same as the queue manager that this example runs.</dd>
 *
 * <DT>iniFile</DT>
 * <DD>The fully qualified path of the .ini file used by this tool to instantiate
 * a device queue manager, send the admin message, and wait for the reply. Works with
 * MQe 1.x style .ini files.
 * </DD>

 * </dl>
 *
 * <P><B>Examples</B></P>
 * <P>This tool would normally be used to delete a listener from the local queue manager
 * for example:
 * ...ListenerDelete ListenerName MyDeviceQM C:\MQe\...\MyDeviceQM.ini
 * </P>
 *
 */
public class ListenerDelete extends MQeAbstractCmdLineAdminTool {

    public static short[] version = { 2, 0, 0, 10 }; 

    /**
     * Entry point from the command line.
     *
     * @param args   Command-line arguments being passed in.
     */
    public static void main(String args[]) {
        int rc = 0;
        ListenerDelete me = new ListenerDelete();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Gathers command-line parameters into variables, and calls the method which creates the queue.
     *
     * @param args
     * @return void
     * @exception Exception
     */
    public void activate(String args[]) throws Exception {

        // Get the parameters from the system properties...
        if (args.length < 3) {
            throw new Exception(
                    "Error: Not enough parameters. "
                    + "Syntax :ListenerDelete <listenerName> <qMgr> <ini_file>");
        }
        String listenerName = args[0].trim();
        String qmgr = args[1].trim();
        String iniFile = args[2].trim();

        System.out.println(
            "i: Attempting to delete a listener "
            + listenerName
            + " on qmgr "
            + qmgr
            + " using the "
            + iniFile
            + " ini file");

        // Access the local MQe queue manager...
        MQeClient client = new MQeClient(iniFile, true/* Suppress stdout*/);

        // Access the queue manager created by the MQeClient class...
        MQeQueueManager myQMgr = client.queueManager;

        // Delete the listener...
        deleteListener(listenerName, qmgr, myQMgr);

        // Close down the MQeClient
        client.close();
    }

    /**
     * Creates an admin message which will cause a listener to be deleted
     * then it sends it to a queue manager.
     *
     * @param listenerName (java.lang.String) The name of listener which is to be created.
     *
     * @param targetQMgr The name of the queue manager to which the admin message will be sent.
     *
     * @param myQMgr The reference to an active MQe queue manager object, which will be
     *               used to send the admin message, and receive the reply.
     *
     * @exception Exception
     */
    public static void deleteListener(String listenerName, String targetQMgr, MQeQueueManager myQMgr) throws Exception {

        System.out.println("Build an admin message to stop a listener");
        MQeCommunicationsListenerAdminMsg msg = new MQeCommunicationsListenerAdminMsg();

        msg.setName(listenerName);
        msg.stop();

        MQeFields results = sendWaitMessage(myQMgr, targetQMgr, msg);

        System.out.println("Listener stopped.");
        System.out.println("now build an admin message to delete a listener");
        msg = new MQeCommunicationsListenerAdminMsg();

        msg.setName(listenerName);
        msg.delete((MQeFields) null);

        results = sendWaitMessage(myQMgr, targetQMgr, msg);

        System.out.println("Listener deleted successfully.");
        
    }
}
