package examples.administration.commandline;


// Version mqe_java/source/examples/administration/commandline/ListenerStop.java, MQe, la200, la200-L030324.1 1.4
/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/*------------------------------------------------------------------------------*/

import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;


/**
 * A command-line tool to allow someone to Stop a listener from an MQe queue
 * manager without programming anything in Java.  This version uses admin
 * messages to send the command to the destination queue manager.
 *
 * <b><p>Syntax</b></p>
 * <P><TT>java examples.administration.commandline.ListenerStop
 * &lt;listenerName&gt; &lt;qMgr&gt; &lt;ini_file&gt;
 * </TT></P>
 *
 * <dl>
 *
 * <dt>listenerName</dt>
 * <dd>The name of listener which is to be Stoped.</dd>
 *
 * <dt>qMgr</dt>
 * <dd>The MQe queue manager on which the listener will be Stoped.</dd>
 *
 * <DT>iniFile</DT>
 * <DD>The fully qualified path of the .ini file used by this tool to instantiate
 * a device queue manager, send the admin message, and wait for the reply. Works with
 * MQe 1.x style .ini files.
 * </DD>
 *
 * </dl>
 *
 * <B><P>Examples</B></P>
 * <P>This tool would normally be used to Stop a listener on the local queue manager
 * for example:
 * ...ListenerStop ListenerName MyDeviceQM C:\MQe\...\MyDeviceQM.ini
 * </P>
 *
 */
public class ListenerStop extends MQeAbstractCmdLineAdminTool {

    public static short[] version = { 2, 0, 0, 10 }; 

    /**
     * Entry point from the command line.
     *
     * @param args   Command-line arguments being passed in.
     */
    public static void main(String args[]) {
        int rc = 0;
        ListenerStop me = new ListenerStop();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Gathers command-line parameters into variables, and calls the method which creates the queue.
     *
     * @param args
     * @return void
     * @exception Exception
     */
    public void activate(String args[]) throws Exception {
        // Get the parameters from the system properties...
        if (args.length < 3) {
            throw new Exception("Error: Not enough parameters. " + "Syntax :ListenerStop <listenerName> <qMgr> <ini_file>");
        }

        String listenerName = args[0].trim();
        String qmgr = args[1].trim();
        String iniFile = args[2].trim();

        System.out.println("i: Attempting to Stop a listener " + listenerName + " on qmgr " + qmgr + " using the " + iniFile + " ini file");

        // Access the local MQe queue manager...
        MQeClient client = new MQeClient(iniFile, true/* Suppress stdout*/);

        // Access the queue manager created by the MQeClient class...
        MQeQueueManager myQMgr = client.queueManager;

        // Delete the listener...
        stopListener(listenerName, qmgr, myQMgr);

        // Close down the MQeClient
        client.close();
    }

    /**
     * Creates an admin message which will cause a listener to be stopped
     * then it sends it to a queue manager.
     *
     * @param listenerName (java.lang.String) The name of listener.
     *
     * @param targetQMgr The name of the queue manager to which the admin message will be sent.
     *
     * @param myQMgr The reference to an active MQe queue manager object, which will be
     *               used to send the admin message, and receive the reply.
     *
     * @exception Exception
     */
    public static void stopListener(String listenerName, String targetQMgr, MQeQueueManager myQMgr) throws Exception {
        
        System.out.println("Build an admin message to stop a listener");
        MQeCommunicationsListenerAdminMsg msg = new MQeCommunicationsListenerAdminMsg();

        msg.setName(listenerName);
        msg.stop();

        MQeFields results = sendWaitMessage(myQMgr, targetQMgr, msg);

        System.out.println("Listen stopped successfully.");
        
    }

}
