// Version mqe_java/source/examples/administration/commandline/MQeAbstractCmdLineAdminTool.java, MQe, la200, la200-L030324.1 1.5

/*------------------------------------------------------------------------------*/

/*                   WebSphere MQ Everyplace for Multiplatforms                 */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2001                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/

package examples.administration.commandline;


import java.util.Enumeration;

import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;


/**
 * A concrete super-class of all the MQe command-line admin tools.
 * <P>Some common methods and utility functions are implemented in this class</P>
 * <P>Command-line MQe administration tools inherit from this class</P>
 */
public class MQeAbstractCmdLineAdminTool {

    public static short[] version = { 2, 0, 0, 10 }; 

    /**
     * The queue to which the admin message is sent.
     */
    public static String ADMIN_Q_NAME = MQe.Admin_Queue_Name;

    /**
     * The queue to which the admin reply should be sent.
     */
    public static String REPLY_TO_Q = MQe.Admin_Reply_Queue_Name;

    /**
     * The tool will wait for an admin reply message to appear on the reply queue for a set
     * timeout period.
     */
    public static int    TIMEOUT_MILLISECS = 10000;

    /**
     * To put the specified message to the Admin Q on the specified MQe queue manager
     * and get a response.
     *
     * @param myQMgr A reference to an active MQe queue manager, which will be used send and receive
     *               MQe administration messages.
     *
     * @param targetQMgrName
     *               The name of the queue manager to which the message will be sent.
     *
     * @param msg    The message we will send to the target queue manager, and await a response.
     *
     * @return The MQeFields response that the reply message contains
     * @exception Exception fails it the reply indicates the Admin message
     *                      caused a failure, or if there was a failure posting the message.
     */
    public static MQeFields sendWaitMessage(MQeQueueManager myQMgr, String targetQMgrName, MQeAdminMsg     msg
    ) throws Exception {
        System.out.println("..Sending the admin message to " + targetQMgrName + " using queue manager " + myQMgr.getName());

        // Prime the admin message, for example set the reply to queue...
        MQeFields msgFilter = primeAdminMsg(targetQMgrName, msg);

        // Put the message and wait for a reply
        MQeAdminMsg respMsg = processAdminMsg(myQMgr, targetQMgrName, msg, msgFilter);

        // Check if the request worked if not an exception is thrown
        checkAdminReply(respMsg);

        // If no exception was thrown then admin request was successful.
        System.out.println("..Admin request was successful.");

        return respMsg.getOutputFields();
    }

    /**
     * processAdminMsg()
     *
     *  Send the admin message, and wait for a repy. An exception is thrown
     *  if there is an error
     * @param myQMgr A reference to an active MQe queue manager, which will be
     *                used to send and receive administration messages.
     *
     * @param onQMgr   - the name of the queue manager where the message will be sent.
     * @param msg      - the admin message to send
     * @param filter   - an mqefields object that allows request and reply
     *                    messages to be correlated.
     * @return replyMsg  returns the message received in response to the
     *                    admin request or null if none was returned.
     */
    public static MQeAdminMsg processAdminMsg(MQeQueueManager myQMgr, String onQMgr, MQeAdminMsg msg, MQeFields filter)
        throws Exception {

        /*
         *  Put the admin message to the admin queue
         */
        System.out.println(
            "..Put admin message to QM/queue: "
            + onQMgr
            + "/"
            + ADMIN_Q_NAME);
        myQMgr.putMessage(onQMgr, ADMIN_Q_NAME, msg, null, 0);

        /*
         * Wait a while for the response message
         */
        System.out.println("..Waiting for a response to the admin message to arrive");
        MQeAdminMsg respMsg = (MQeAdminMsg) myQMgr.waitForMessage(
                onQMgr                  // The queue manager we're getting the response from.
                , REPLY_TO_Q            // The queue we're expecting the response to appear on.
                , filter                // Filter on which message we want to wait for.
                , null                  // Attributes.
                , (long) 0              // We're not going to confirm our get.
                , TIMEOUT_MILLISECS     // time before we give up waiting for the message.
            );

        return respMsg;
    }

    /**
     * checkAdminReply()
     *
     * Check to see if a good reply was
     * received. If not detail the error(s) that occured and then throw
     * an exception.
     *
     * @return boolean  true if good
     * @param replyMsg MQeAdminMsg - reply message to check
     */
    public static boolean checkAdminReply(MQeAdminMsg replyMsg) throws Exception {
        // Was a reply received ?
        if (replyMsg == null) {
            System.out.println(
                "..No response received to the request");
            throw new MQeException("No response message received");
        }

        // If the reply was bad then check why
        if (replyMsg.getRC() != MQeAdminMsg.RC_Success) {
            System.out.println(
                "..Action Failed: " + replyMsg.getReason());

            // If mixed then list each error that occured
            if (replyMsg.getRC() == MQeAdminMsg.RC_Mixed) {
                MQeFields errors = replyMsg.getErrorFields();
                Enumeration en = errors.fields();

                //            System.out.println("ERRORS: "+errors.dumpToString("#0 \t #1\t #2\r\n"));

                // process each error
                while (en.hasMoreElements()) {
                    String value[] = {};
                    String name = (String) en.nextElement();

                    // Field in error may be an array
                    if (errors.dataType(name) == MQeField.TypeArrayElements)
                        value = errors.getAsciiArray(name);
                    else { // in the case when it is not an array
                        // Ignore individual dynamic array elements, the array is handled
                        // as a whole.
                        if (name.indexOf(MQeFields.ArraySeparator) < 0) {
                            value = new String[] { errors.getAscii(name)};
                        }
                    }

                    for (int j = 0; j < value.length; j++)
                        System.out.println(
                            "Field in error: " + name + "  " + value[j]);
                }
            }
            // Failed - throw exception
            throw new MQeException(replyMsg.getReason());
        }
        return true; // All is OK
    }

    /**
     * primeAdminMsg()
     *
     *     Prime an admin message. Sets the admin message with items such as
     *     who to reply to along with a unique identifier that can be used to
     *     check for a matching response.
     *
     * @param onQMgr   - the name of the queue manager where the message will be sent.
     * @param msg      - the admin message to prime
     * @return MQeFields - a unique identifier for this request
     */
    public static MQeFields primeAdminMsg(String onQMgr, MQeAdminMsg msg)
        throws Exception {

        /*
         * Set the target queue manager that will process this message
         */
        msg.setTargetQMgr(onQMgr);

        /*
         * Ask for a reply message
         */
        msg.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);
        msg.putAscii(MQe.Msg_ReplyToQ, REPLY_TO_Q);
        msg.putAscii(MQe.Msg_ReplyToQMgr, onQMgr);

        /*
         *  Setup the correl id so we can match the reply to the request
         */
        msg.putArrayOfByte(
            MQe.Msg_CorrelID,
            Long.toHexString(MQe.uniqueValue()).getBytes());

        /*
         * Ensure matching response message is retrieved
         */
        MQeFields msgTest = new MQeFields();

        msgTest.putArrayOfByte(
            MQe.Msg_CorrelID,
            msg.getArrayOfByte(MQe.Msg_CorrelID));

        /*
         * Return the unique filter for this message
         */
        return msgTest;
    }

}
