package examples.application;


// Version mqe_java/source/examples/application/Example4.java, MQeExamples, la200, la200-L030324.1 1.10
/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
import com.ibm.mqe.*;


/***
 * <p>Extends ExampleBase, Application example base.
 *
 * <p>Wait for message example.
 *
 * <p>Example showing how to:
 * <pre>
 *  . start a queue manager
 *  . spawn a new thread which will wait for a new message and
 *    - display message if it arrives
 *  . put 2 messages to the system default queue
 *  . browse the queue to locate any remaining messages
 *  . remove any remaining messages.
 * </pre>
 * <p>The result should be that the WaitForMessage should find one message.
 *
 */
public class Example4 extends ExampleBase implements Runnable {
    public static short[] version = { 2, 0, 0, 10 }; 
    String msgData1 = example + ": Msg1, Little Bo Peep";
    String msgData2 = example + ": Msg2, has lost her sheep";

    int     waitFor = 10000;

    Thread  msgWaitThread = null;

    private volatile boolean toldToStop = false;

    /**
     * Constructor that uses the default queue manager.
     * @exception java.lang.Exception
     */
    public Example4() throws Exception {
        super();
    }

    /**
     * Constructor that uses the queue manager passed in on the command line.
     * @param queueManagerName java.lang.String
     * @exception java.lang.Exception
     */
    public Example4(String queueManagerName) throws Exception {
        super(queueManagerName);
    }

    /**
     * Constructor that starts a queue manager with paramters
     * from an ini file. This queue manager is then used as the
     * access point for interacting with the same or different
     * queue managers.
     * @param qMgrName java.lang.String, queue manager name
     *        The name of the queue manager to interact with.
     *        The queue manager can be a local or a remote queue
     *        manager.
     * @param localQMgrIniFile java.lang.String, ini file which
     *        contains startup parameters for the local queue
     *        manager.
     * @exception java.lang.Exception
     */
    public Example4(String qMgrName, String localQMgrIniFile) throws Exception {
        super(qMgrName, localQMgrIniFile);
    }

    /**
     * Create a separate thread that will issue the WaitForMsg method
     * against the queue manager.
     * @return void
     */
    public void createMsgWaitThread() {
        msgWaitThread = new Thread(this);
        toldToStop = false;
        msgWaitThread.start();
    }

    /**
     * All work is performed in this method.
     * @return void
     * @exception java.lang.Exception
     */
    public void doIt() throws Exception {
        try {

            /*
             * Create a thread to wait for new messages
             */
            createMsgWaitThread();

            /*
             * Wait a little while msg wait thread gets up to speed
             */
            System.out.println("..Wait a few secs for wait for thread to initialise");
            Thread.sleep(1000);

            /*
             * Create a msg with some data
             */
            System.out.println("..Create a message with data:" + msgData1);
            MQeMsgObject msgObj1 = new MQeMsgObject();

            msgObj1.putAscii(msgField, msgData1);

            /*
             * Put the msg to a queue
             */
            System.out.println("..Put the message to QM/queue: " + qMgrName + "/" + queueName);
            myQM.putMessage(qMgrName, queueName, msgObj1, null, 0);

            /*
             * Create a msg with some data
             */
            System.out.println("..Create a message with data:" + msgData2);
            MQeMsgObject msgObj2 = new MQeMsgObject();

            msgObj2.putAscii(msgField, msgData2);

            /*
             * Put the msg to a queue
             */
            System.out.println("..Put the message to QM/queue: " + qMgrName + "/" + queueName);
            myQM.putMessage(qMgrName, queueName, msgObj2, null, 0);

            /*
             * Giving the waiting thread a bit of time before we clean up.
             */
            Thread.sleep(1000);

            /*
             * Tidy up by removing the 2 remaining messages
             */
            emptyQueue();

            System.out.println(example + " Finished");
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(example + " Failed!");
            throw e;
        } finally {

            /*
             * Clean up
             */
            stop();
            close();
        }
    }

    /**
     * Starts the application.
     * @param param1  Queue Manager Name to interact with.
     *                      If queue manager name matches the local QM name then
     *                      all interactions are local otherwise the local QM acts
     *                      as a stub QM sending all messages to a remote QM.
     * @param param2  Local queue manager startup parameters ini file.
     * @return void
     * @exception java.lang.Exception
     */
    public static void main(String args[]) throws Exception {
        short rc = 0;

        try {
            if (args.length == 0)
                new Example4().doIt();
            else if (args.length == 1)
                new Example4(args[0]).doIt();
            else
                new Example4(args[0], args[1]).doIt();
        } catch (Exception ex) {
            System.out.println("Example failed: " + ex);
            ex.printStackTrace();
            rc = 99;
        }
        System.exit(rc);
    }

    /**
     * Thread that will sit around waiting for messages
     */
    public void run() {
        try {
            System.out.println("==Wait for a Message to arrive for " + waitFor);
            System.out.println("== on QM/Queue " + qMgrName + "/" + queueName);
            MQeMsgObject msgObj = myQM.waitForMessage(qMgrName, queueName, null, null, 0, waitFor);

            System.out.println("==WaitForMessage: message arrived with data " + msgObj.getAscii(msgField));
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * Over-ride default parameter settings.
     */
    public void setParms() {
        example = "Example4";
    }

    /*----------------------------------------------------------------------------*/

    /* stop procedure                                                             */

    /*----------------------------------------------------------------------------*/
    public void stop() {
        toldToStop = true;
        System.out.println("==Stop msg wait thread");
        // stop the thread
        if (msgWaitThread != null) {
            Thread tmp = msgWaitThread;

            try {
                tmp.join(waitFor);
                msgWaitThread = null;
            } catch (InterruptedException e) {}
        }
    }
}
