// Version mqe_java/source/examples/application/Example7.java, MQeExamples, la200, la200-L030324.1 1.11

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2000                                                 */

/* -----------------------------------------------------------------------------*/
package examples.application;


import com.ibm.mqe.*;
import com.ibm.mqe.adapters.*;


/**
 * <p>Extends, ExampleBase, the simple application that puts and gets a message to
 * a queue.
 *
 * <p>This example shows how to set the userid and password information
 *  before sending data through the WES Authentication Proxy.
 *
 * <p>Usage:  java examples.application.Example7 <server> <client.ini>
 *
 * <p>where <server> is the name of the remote queue manager
 *  and
 * <p><client.ini> is the .ini file that defines the client.
 *
 * <p>Both the client and the server should be set up with
 *  com.ibm.mqe.adapters.MQeWESTcpipHttpAdapter as the network adapter, and
 *  a connection should already exist that enables the client to communicate
 *  with the server. In addition, the client should be routing HTTP requests
 *  through the Websphere Everyplace Authentication Proxy.
 *
 * <p>This example will use the existing client queue manager in order to
 *  programmatically set the user-id and password fields that are required
 *  by the proxy. It will also check for invalid responses from the
 *  server.
 *
 * <p>The example relies on the queue containing no messages at the time
 *  when the example is run.
 *
 * <p>This example is a modified version of Example1 with added exception checks.
 */
public class Example7 extends ExampleBase {
    public static short[] version = { 2, 0, 0, 10 }; 

    /**
     * The UserId to use. The bit after the @ sign is
     * the realm.
     */
    private static final String USERID = "homer@springfield";

    /**
     * The password to use
     */
    private static final String PASSWORD = "MrSparkle";

    /**
     * Constructor that starts a queue manager with parameters
     * from an ini file. This queue manager is then used as the
     * access point for interacting with the same or different
     * queue managers.
     * @param qMgrName java.lang.String - queue manager name
     *        The name of the queue manager to interact with.
     *        The queue manager should be a remote queue
     *        manager for the proxy to be invoked.
     * @param localQMIniFile java.lang.String - ini file which
     *        contains startup parameters for the local queue
     *        manager
     * @exception java.lang.Exception
     */
    public Example7(String qMgrName, String localQMgrIniFile) throws Exception {
        super(qMgrName, localQMgrIniFile);
    }

    /**
     * All work is performed in this method.
     * @return void
     * @exception java.lang.Exception
     */
    public void doIt() throws Exception {

        // TODO -start
        // Set the proxy name (TP ONLY)
        System.getProperties().put("http.proxyHost", "wes.hursley.ibm.com");
        System.getProperties().put("http.proxyPort", "80");
        System.getProperties().put("http.proxySet", "true");

        System.out.println("..PROXY SET");

        // TODO -end

        // Set the username and password.
        MQeWESAuthenticationAdapter.setBasicAuthorization(USERID, PASSWORD);
        System.out.println("..UserId and password set to " + USERID + ":" + PASSWORD);

        // Do the real MQe bits.
        // The code in the try block is lifted from Example1.
        try {

            // Create a msg with some data
            System.out.println("..Create a message and add data:" + msgData);
            MQeMsgObject msgObj1 = new MQeMsgObject();

            msgObj1.putAscii(msgField, msgData);

            // Put msg to queue
            System.out.println("..Put the message to QM/queue: " + qMgrName + "/" + queueName);
            myQM.putMessage(qMgrName, queueName, msgObj1, null, 0);

            // Retrieve the first available message on the queue (and only msg!)
            System.out.println("..Get a message from QM/queue: " + qMgrName + "/" + queueName);
            MQeMsgObject msgObj2 = myQM.getMessage(qMgrName, queueName, null, null, 0);

            System.out.println("..Message retrieved contains data " + msgObj2.getAscii(msgField));

            // Check if it contains the expected data
            if (msgObj2.getAscii(msgField).equals(msgData))
                System.out.println(example + " Finished");
            else
                System.out.println(example + " Failed: Message data does not match.");
        } // Catch a "bad response"-type exception from the proxy.
        // This is thrown from either the putMessage() or the getMessage(),
        // although will usually only be thrown on the first put when the
        // first exchange with the proxy takes place.
        catch (MQeException mqeEx) {

            mqeEx.printStackTrace();

            // Thrown for 401 (Unauthorized) and 403 (Forbidden)
            if (mqeEx.code() == MQe.Except_Authenticate) {
                System.err.println("Failed: Could not be authenticated with the proxy.\n" +
                    "Realm = " + MQeWESAuthenticationAdapter.getRealm());
            } // Thrown if the server responded with 404 (Not found)
            else if (mqeEx.code() == MQe.Except_NotFound) {
                System.err.println("Failed: Requested URL could not be found (404).\n" +
                    "Amend the URL inside the client's connection definition of " + qMgrName + ".");
            } // Thrown for all other 40x and 50x server responses
            else if (mqeEx.code() == MQe.Except_NotSupported) {
                System.err.println("Failed: Server rejected the request.\n" +
                    "Response = " + mqeEx.getMessage());
            } // Something not related to the adapter went wrong.
            else {
                throw mqeEx;
            }
        } // Catch other exceptions
        catch (Exception e) {
            e.printStackTrace();
            System.out.println(example + " Failed! " + e);
            throw e;
        } // Clean up.
        finally {
            close();
        }
    }

    /**
     * Override default parameter settings.
     */
    public void setParms() {
        example = "Example7";
        msgData = example + ":The land of wind and ghosts";
    }

    /**
     * Starts the application.
     * @param String  Queue Manager Name to interact with.
     *                 If queue manager name matches the local QM name then
     *                 all interactions are local otherwise the local QM acts
     *                 as a stub QM sending all messages to a remote QM.
     * @param String  Local queue manager startup parameters ini file.
     * @return void
     * @exception java.lang.Exception
     */
    public static void main(String args[]) throws Exception {
        short rc = 0;

        try {
            if (args.length < 2) {
                System.err.println("java examples.application.Example7 <server> <client.ini>");
                System.err.println("where <server>     is the name of the remote queue manager");
                System.err.println("  and <client.ini> is the .ini file that defines the client");
            } else {
                new Example7(args[0], args[1]).doIt();
            }

        } catch (Exception ex) {
            rc = 99;
            System.out.println("Example failed: " + ex);
            ex.printStackTrace();
        }
        System.exit(rc);
    }

}
