// Version mqe_java/source/examples/certificates/ListWTLSCertificates.java, MQe, la200, la200-L030324.1 1.6

/*------------------------------------------------------------------------------*/

/*                   WebSphere MQ Everyplace for Multiplatforms                 */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2001                                                 */

/*------------------------------------------------------------------------------*/
package examples.certificates;


import java.util.*;
import com.ibm.mqe.*;
import com.ibm.mqe.attributes.*;
import examples.queuemanager.*;


/****
 * <p>List certificates in a registry, at different levels of detail.
 * This uses the com.ibm.mqe.attributes.MQeListCertificate class to
 * read the certificates from the registry and get the required
 * information from the certificates.
 *
 * <p>The example program takes these command-line parameters:
 *
 * <p>ListWTLSCertificates [reg Name] [ini file] [level] [cert names]
 *
 * <p>where:
 * <pre>
 *
 *        reg Name -   is the name of the registry whose certificates are to
 *                     be listed. This can be a queue manager's registry, a
 *                     queue's registry, a public registry, or (for the
 *                     administrator) the mini-certificate server's registry.
 *
 *        ini file -   is the name of the configuration file. This would
 *                     typically be the same file that is used for the
 *                     queue manager or mini-certificate server. This can be
 *                     omitted for public registries.
 *
 *        level    -   is the level of detail for the listing. This can be:
 *                     -b or -brief    this is the default. It prints the
 *                                     names of the certificates, one per
 *                                     line.
 *                     -f or -full     prints the names of the certificates,
 *                                     their type, and some of their contents
 *
 *        cert names - the flag "-cn" followed by names of specific
 *                     certificates, eg:
 *                        -cn ExampleQM putQM
 *                     If certificate names are specified, only these
 *                     certificates will be listed. If no names are specified,
 *                     all the certificates in the registry will be listed.
 * </pre>
 *
 ****/
public class ListWTLSCertificates {
    public static short  version[] = { 2, 0, 0, 10 }; 

    // constants
    public final static String  MiniCertSvrRegistry = "MiniCertSvrRegistry";
    public final static String  Registry = "Registry";
    public final static String  MiniCertificateServer = "MiniCertificateServer";

    // labels for the listing
    public final static String subject = "Subject";
    public final static String issuer = "Issuer";
    public final static String notBefore = "Not Valid Before";
    public final static String notAfter = "Not Valid After";

    // values from the command line parameters
    String                   regName = null;
    String                   iniFile = null;
    Vector                   certNames = new Vector();

    final int                ListBrief = 1;
    final int                ListFull = 2;
    int                      listMode = ListBrief;

    // the ListCertificates object
    MQeListCertificates listCerts = null;

    /**
     * Empty constructor. The activate() method must be called after this constructor.
     **/
    public ListWTLSCertificates() {}

    /**
     * Process the command line arguments:
     * <p>ListWTLSCertificates <reg Name> [<ini file>] [<level>] [cert names]
     *
     * @param  args    The command line arguments.
     * @exception java.lang.exception  Thrown on error.
     **/
    void processParameters(String[] args) throws Exception {
        // any parameters ?
        if (args.length < 1) {
            System.out.println("*** invalid parameters, should be:");
            System.out.println("  ListWTLSCertificates <reg Name> [<ini file>] [-b|-n|-f] [-cn certificate ...]");
            System.exit(1);
        }

        regName = args[0];
        int consumed = 1;

        // is there an ini file ?
        if (args.length > 1 && args[1].charAt(0) != '-') {
            iniFile = args[1];
            consumed = 2;
        }

        // iterate through the parameters picking out the values
        while (args.length > consumed) {
            String thisFlag = args[consumed];

            if (thisFlag.equals("-b") || thisFlag.equals("-brief")) { // List mode
                listMode = ListBrief;
                ++consumed;
            } else if (thisFlag.equals("-f") || thisFlag.equals("-full")) { // List mode
                listMode = ListFull;
                ++consumed;
            } else if (thisFlag.equals("-cn")) { // certificate names on the command line
                if (args.length < consumed + 2)
                    throw new Exception("invalid number of parameters");
                String certName = args[consumed + 1];

                // carry on until the end or until we find the next flag
                while (certName.length() == 0 || certName.charAt(0) != '-') {
                    if (certName.length() > 0) {
                        // add each name into the certNames vector
                        certNames.addElement(certName);
                    }
                    ++consumed;
                    if (args.length <= consumed + 1)
                        break;
                    certName = args[consumed + 1];
                }
                ++consumed;
            } else
                throw new Exception("unrecognised flag: " + thisFlag);
        }
    }

    /**
     * OpenList, ie initialialize everything.
     * @return void
     * @exception java.lang.exception  Thrown on error.
     **/
    public void openList() throws Exception {
        MQeFields regParams = null;

        if (iniFile != null) {
            // use the example queue manager utilities to read the ini file
            MQeFields configParams = MQeQueueManagerUtils.loadConfigFile(iniFile);

            // use the example queue manager utilities to process any aliases
            MQeQueueManagerUtils.processAlias(configParams);
            // find the registry section
            if (configParams.contains(Registry))
                regParams = configParams.getFields(Registry);
            else if (configParams.contains(MiniCertSvrRegistry)) { // this is a mini-certificate server registry, so make sure
                // the name is correct
                regName = MiniCertificateServer;
                regParams = configParams.getFields(MiniCertSvrRegistry);
            }
        }
        listCerts = new MQeListCertificates(regName, regParams);
    }

    /**
     * List the certificates in the registry.
     * @return void
     * @exception  com.ibm.mqe.MQeException
     **/
    public void list() throws Exception {
        // if no certificates are specified, list them all
        if (certNames.size() == 0) {
            MQeFields certs = listCerts.readAllEntries();

            // readAllEntries() returns a MQeFields object, each entry of which
            // is itself a MQeFields object containing the registry entry for
            // a certificate.
            // We want to iterate through these entries, which are keyed on the
            // name of the certificate, and print out each one.

            Enumeration enum = certs.fields();

            if (!enum.hasMoreElements())
                System.out.println("--- no certificates found");
            else {
                while (enum.hasMoreElements()) {
                    // get the name of the certificate
                    String entity = (String) enum.nextElement();
                    // get the certificate's registry entry
                    MQeFields cert = certs.getFields(entity);

                    // print it
                    printCertificate(entity, cert);
                }
            }
        } else {
            // only print the named certificates
            for (int i = 0; i < certNames.size(); ++i) {
                String entity = (String) certNames.elementAt(i);
                MQeFields cert = listCerts.readEntry(entity);

                if (cert == null)
                    System.out.println("Cannot read certificate for <" + entity + ">");
                else
                    printCertificate(entity, cert);
            }
        }
    }

    /**
     * Print the certificate to standard out.
     *
     * @param  entity     The name of the certificate
     * @param  certEntry  The registry entry for the cetificate
     * @return void
     * @exception  java.lang.Exception  Thrown if there is an error processing the
     *                                  certificate.
     **/
    void printCertificate(String entity, MQeFields cert) throws Exception {
        // print the name of the certificate
        System.out.println(entity);

        // if listMode is full, print some of the certificate contents
        if (listMode == ListFull) {
            // extract the certificate object from the registry entry
            Object certificate = listCerts.getWTLSCertificate(cert);

            if (certificate == null)
                System.out.println("*** invalid certificate registry entry for " + entity);
            else {
                System.out.println(subject + ": \t\t" + listCerts.getSubject(certificate));
                System.out.println(issuer + ": \t\t" + listCerts.getIssuer(certificate));
                System.out.println(notBefore + ": \t" + new Date(listCerts.getNotBefore(certificate) * 1000));
                System.out.println(notAfter + ": \t" + new Date(listCerts.getNotAfter(certificate) * 1000));
                System.out.println("");
            }
        }

    }

    /**
     * Close - free any resources.
     * @return void
     **/
    public void close() {
        listCerts.close();
    }

    /**
     * Main program
     *
     * @param args[0]  The command line parameters.
     **/
    public static void main(String[] args) {
        try {
            ListWTLSCertificates lst = new ListWTLSCertificates();

            lst.processParameters(args);
            lst.openList();
            lst.list();
            lst.close();
        } catch (Exception e) {
            System.err.println(e);
        }
    }

}

