// Version mqe_java/source/examples/certificates/RenewWTLSCertificates.java, MQe, la200, la200-L030324.1 1.4

/*------------------------------------------------------------------------------*/

/*                   WebSphere MQ Everyplace for Multiplatforms                 */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/*------------------------------------------------------------------------------*/
package examples.certificates;


import java.util.*;
import com.ibm.mqe.registry.*;
import com.ibm.mqe.*;
import examples.queuemanager.*;


/*****
 * <p>Renews a WTLS certificate in a registry using the renewCertificates()   
 * method in MQePrivateRegistryConfigure. The certificate for the          
 * mini-certifiate server itself is obtained at the same time.             
 *                                                                              
 * <p>The example program takes four command-line parameters:                 
 *                                                                              
 * <p>RenewWTLSCertificates [entity] [ini file] [MCS addr] [MCS Pin]       
 *                                                                              
 * <p> where:                                                                  
 * <pre>                                                                            
 *        entity   -  is the name of the entity for which a renewed             
 *                    certificate is required. This would typically be          
 *                    either a queue manager (eg ExampleQM) or a queue          
 *                    (eg ExampleQM+secureQueue)                                
 *                                                                               
 *        ini file -  is the name of the configuration file. This would         
 *                    typically be the same file that is used for the           
 *                    queue manager and it must contain a [Registry] section.   
 *                                                                              
 *        MCS addr -  is the name and port address of the mini-certificate      
 *                    server (eg myServer:8085)                                 
 *                                                                              
 *        MCS Pin  -  is the one-time use PIN issued by the mini-certificate    
 *                    server administrator to authorize this renewal request.   
 *                                                                              
 *  </pre>                                                                            
 * <p> Restrictions:                                                                                                                                            
 * <p>This example should only be used on a private registry, typically       
 * owned by a queue manager or a queue.   
 *                                  
 *                                                                   
 */

public class RenewWTLSCertificates {
    public static short  version[] = { 2, 0, 0, 10 }; 

    // the private registry PIN
    String prPIN = null;

    // the RegistryConfigure object
    MQePrivateRegistryConfigure regConf = null;

    /**
     * Empty constructor. The activate() method must be called after this constructor.
     **/
    public RenewWTLSCertificates() {}

    /**
     * Constructor. This calls activate(), which does all the initialisation work.
     *
     * @param  regName  The name of the registry
     * @param  iniFile  The name of the ini file which identifies the registry.
     *                  This can be either a queue manager's or a queue's registry.
     * @exception java.lang.exception  Thrown on error.
     **/
    public RenewWTLSCertificates(String regName, String iniFile) throws Exception {
        // use the example queue manager utilities to read the ini file
        activate(regName, MQeQueueManagerUtils.loadConfigFile(iniFile));
    }

    /**
     * Activate, ie initialialize everything.
     *
     * @param  regName       The name of the registry
     * @param  configParams  The configuration parameters for the registry.
     *                       This can be either a queue manager's or a queue's registry.
     * @return void
     * @exception java.lang.exception  Thrown on error.
     **/
    public void activate(String regName, MQeFields configParams) throws Exception {
        // use the example queue manager utilities to process any aliases
        MQeQueueManagerUtils.processAlias(configParams);
        // find the "Registry" section
        MQeFields regParams = configParams.getFields("Registry");

        // save the private registry pin, we need it later
        prPIN = regParams.getAscii(MQeRegistry.PIN);
        // instantiate a Private Registry Configure object
        regConf = new MQePrivateRegistryConfigure(regName, regParams, prPIN);
    }

    /**
     * Renew the certificates in the registry
     *
     * @param  miniCertServer The network address and port number of the Mini-Certificate
     *                        Server.
     * @param  miniCertPIN    The one-time PIN required by the Mini-Certificate
     *                        Server.
     * @return void
     * @exception  com.ibm.mqe.MQeException  Propagated from renewCertificates()
     **/
    public void renew(String miniCertServer, String miniCertPIN) throws Exception {
        // use a timestamp as the rename prefix
        String prefix = Long.toString(new Date().getTime()) + "_";

        regConf.renewCertificates(prPIN, miniCertServer, miniCertPIN, prefix);

        System.out.println("successfully renewed certificates");
    }

    /**
     * Close - free any resources.
     * @return void
     **/
    public void close() {
        regConf.close();
    }

    /**
     * Main program
     *
     * @param args[0]  The command line parameters.
     **/
    public static void main(String[] args) {
        if (args.length < 4) {
            System.out.println("*** invalid parameters, should be:");
            System.out.println("  RenewWTLSCertificates <entity> <ini file> <MCS addr> <MCS Pin>");
        } else {
            String regName = args[0];
            String iniFile = args[1];
            String MCSaddr = args[2];
            String MCSpin = args[3];

            try {
                RenewWTLSCertificates rn = new RenewWTLSCertificates(regName, iniFile);

                rn.renew(MCSaddr, MCSpin);
                rn.close();
            } catch (Exception e) {
                System.err.println(e);
            }
        }
    }

}
