// Version 1.3 mqe_java/source/examples/config/CreateQueueManager.java, MQeExamples, la200, la200-L030324.1

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/

package examples.config;


import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeQueueManager;
import com.ibm.mqe.MQeQueueManagerConfigure;
import com.ibm.mqe.registry.MQeRegistry;

import java.io.File;


/**
 * This class provides simple code examples for creating and starting queue managers
 */
public class CreateQueueManager {

    public static short  version[] = { 2, 0, 0, 10 };  // MQe version

    public static final void main(String[] args) {
        try {
            String queueManagerName = "SimpleQM";
            String registryLocation = ".\\ExampleQM";
            String queueStore = "MsgLog:" + registryLocation + File.separator + "queues";

            createQueueManagerDefinition(queueManagerName, registryLocation, queueStore);
            MQeQueueManager queueManager = startQueueManager(queueManagerName, registryLocation);

            stopQueueManager(queueManager);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Create a queue manager representation on disc.  The queue manager will
     * have the standard queues.
     *
     * @param queueManagerName - name of queue manager to create
     * @param registryLocation - location where registry is to be created
     * @param queueStore - location where queues will store their messages.
     */
    public static final void createQueueManagerDefinition(
        String queueManagerName,
        String registryLocation,
        String queueStore)
        throws Exception {

        MQeFields parms = new MQeFields();
        MQeFields queueManagerParameters = new MQeFields();

        queueManagerParameters.putAscii(MQeQueueManager.Name, queueManagerName);
        parms.putFields(MQeQueueManager.QueueManager, queueManagerParameters);

        MQeFields registryParameters = new MQeFields();

        parms.putFields(MQeQueueManager.Registry, registryParameters);
        registryParameters.putAscii(MQeRegistry.DirName, registryLocation);

        MQeQueueManagerConfigure qmConfig = new MQeQueueManagerConfigure(parms, queueStore);

        qmConfig.defineQueueManager();
        qmConfig.defineDefaultSystemQueue();
        qmConfig.defineDefaultDeadLetterQueue();
        qmConfig.defineDefaultAdminReplyQueue();
        qmConfig.defineDefaultAdminQueue();
        qmConfig.close();
    }

    /**
     * Create a queue manager representation on disc.  The queue manager will
     * have the standard queues and a private registry.
     *
     * @param queueManagerName - name of queue manager to create
     * @param registryLocation - location where registry is to be created
     * @param queueStore - location where queues will store their messages.
     * @param pin - string with secret pin number
     */
    public static final void createQueueManagerDefinitionPrivate(
        String queueManagerName,
        String registryLocation,
        String queueStore,
        String pin)
        throws Exception {

        MQeFields parms = new MQeFields();
        MQeFields queueManagerParameters = new MQeFields();

        queueManagerParameters.putAscii(MQeQueueManager.Name, queueManagerName);
        parms.putFields(MQeQueueManager.QueueManager, queueManagerParameters);

        MQeFields registryParameters = new MQeFields();

        parms.putFields(MQeQueueManager.Registry, registryParameters);
        registryParameters.putAscii(MQeRegistry.DirName, registryLocation);

        // the following two lines are required for a private registry
        registryParameters.putAscii(MQeRegistry.LocalRegType, "PrivateRegistry");
        registryParameters.putAscii(MQeRegistry.PIN, pin);

        MQeQueueManagerConfigure qmConfig = new MQeQueueManagerConfigure(parms, queueStore);

        qmConfig.defineQueueManager();
        qmConfig.defineDefaultSystemQueue();
        qmConfig.defineDefaultDeadLetterQueue();
        qmConfig.defineDefaultAdminReplyQueue();
        qmConfig.defineDefaultAdminQueue();
        qmConfig.close();
    }

    /**
     * Create a queue manager representation on disc.  The queue manager will
     * have the standard queues and a private registry that uses certificates.
     *
     * @param queueManagerName - name of queue manager to create
     * @param registryLocation - location where registry is to be created
     * @param queueStore - location where queues will store their messages.
     * @param pin - string with secret pin number
     * @param key ring password - user defined secret string
     * @param certificateServerAddress - network address of certificate server
     * @param certificateServerPin - pin registered with the certificate server
     *         against the certificate required.
     */
    public static final void createQueueManagerDefinitionPrivateCertificate(
        String queueManagerName,
        String registryLocation,
        String queueStore,
        String pin,
        String keyRingPassword,
        String certificateServerAddress,
        String certificateServerPin)
        throws Exception {

        MQeFields parms = new MQeFields();
        MQeFields queueManagerParameters = new MQeFields();

        queueManagerParameters.putAscii(MQeQueueManager.Name, queueManagerName);
        parms.putFields(MQeQueueManager.QueueManager, queueManagerParameters);

        MQeFields registryParameters = new MQeFields();

        parms.putFields(MQeQueueManager.Registry, registryParameters);
        registryParameters.putAscii(MQeRegistry.DirName, registryLocation);

        // the following two lines are required for a private registry
        registryParameters.putAscii(MQeRegistry.LocalRegType, "PrivateRegistry");
        registryParameters.putAscii(MQeRegistry.PIN, pin);

        // the following three line stimulate the registry to request a certificate
        // when it is opened.  When this has been done succesfully one, it need
        // not be done again
        registryParameters.putAscii(MQeRegistry.CAIPAddrPort, certificateServerAddress);
        registryParameters.putAscii(MQeRegistry.CertReqPIN, certificateServerPin);
        registryParameters.putAscii(MQeRegistry.KeyRingPassword, keyRingPassword);

        MQeQueueManagerConfigure qmConfig = new MQeQueueManagerConfigure(parms, queueStore);

        qmConfig.defineQueueManager();
        qmConfig.defineDefaultSystemQueue();
        qmConfig.defineDefaultDeadLetterQueue();
        qmConfig.defineDefaultAdminReplyQueue();
        qmConfig.defineDefaultAdminQueue();
        qmConfig.close();
    }

    /**
     * Starts the specified queue manager
     * @param queueManagerName - name of queue manager to create
     * @param registryLocation - location where registry is to be created
     */
    public static final MQeQueueManager startQueueManager(String queueManagerName, String registryLocation)
        throws Exception {
        return new MQeQueueManager(queueManagerName, registryLocation);
    }

    /**
     * Starts the specified queue manager using a private registry
     * @param queueManagerName - name of queue manager to create
     * @param registryLocation - location where registry is to be created
     * @param pin - string with secret pin number
     */
    public static final MQeQueueManager startQueueManagerPrivate(
        String queueManagerName,
        String registryLocation,
        String pin)
        throws Exception {

        MQeFields parms = new MQeFields();
        MQeFields queueManagerParameters = new MQeFields();

        queueManagerParameters.putAscii(MQeQueueManager.Name, queueManagerName);
        parms.putFields(MQeQueueManager.QueueManager, queueManagerParameters);

        MQeFields registryParameters = new MQeFields();

        parms.putFields(MQeQueueManager.Registry, registryParameters);
        registryParameters.putAscii(MQeRegistry.DirName, registryLocation);

        // the following two lines are required for a private registry
        registryParameters.putAscii(MQeRegistry.LocalRegType, "PrivateRegistry");
        registryParameters.putAscii(MQeRegistry.PIN, pin);

        MQeQueueManager queueManager = new MQeQueueManager();

        queueManager.activate(parms);
        return queueManager;
    }

    /**
     * Start the specified queue manager
     *
     * @param queueManager  the queue manager to start
     */
    public static final void stopQueueManager(MQeQueueManager queueManager) throws Exception {
        queueManager.close();
    }
}
