// Version 1.3 mqe_java/source/examples/config/LocalQueueAdmin.java, MQeExamples, la200, la200-L030324.1

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
package examples.config;


import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeAdminMsg;
import com.ibm.mqe.MQeException;
import com.ibm.mqe.MQeExceptionCodes;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeMsgObject;
import com.ibm.mqe.MQeQueueManager;
import com.ibm.mqe.administration.MQeQueueAdminMsg;
import com.ibm.mqe.administration.MQeQueueManagerAdminMsg;

import java.io.File;


/**
 * This class illustrates queue administrtion using admin messages.
 *
 * <p>It creates a local queue  a local queue manager.
 * This example will create a queue manager, open it, and create
 * a local queue called 'NewQueue' on the local queue manager.  Because the example is simple
 * it assumes that the queue manager does not alreadfy exist.  If the queue manager already
 * exists before you run the example, then you will see an exception.  Please delete the existing
 * queue manager and try again....
 *
 * <p>The example will
 * <ul>
 * <li>create a local queue,
 * <li>add an alias,
 * <li>remove the alias,
 * <li>delete the queue.
 * </ul>
 *
 * <p>It uses: {@link examples.config.CreateQueueManager} to create and start a queue manager
 *  as an entry point to MQe, and as a subject of a queue manager query.
 * <p> It uses {@link examples.config.BasicAdministration} to handle the admin message
 */
public class LocalQueueAdmin {

    public static short  version[] = { 2, 0, 0, 10 };  // MQe version

    public static final void main(String[] args) {
        try {
            String queueManagerName = "SimpleQM";
            String registryLocation = ".\\ExampleQM";
            String queueStore = "MsgLog:" + registryLocation + File.separator + "queues";
            String queueName = "NewQueue";
            String aliasName = "NewQueueAlias";

            CreateQueueManager.createQueueManagerDefinition(queueManagerName, registryLocation, queueStore);
            MQeQueueManager queueManager = CreateQueueManager.startQueueManager(queueManagerName, registryLocation);

            // create local queue
            createLocalQueue(queueManager, queueManagerName, queueName);
            // create local queue again - show an error occurs
            createLocalQueue(queueManager, queueManagerName, queueName);

            // add an alias to the queue
            addLocalQueueAlias(queueManager, queueManagerName, queueName, aliasName);
            // add an alias to the queue again to get an error
            addLocalQueueAlias(queueManager, queueManagerName, queueName, aliasName);

            // remove alias from the queue
            removeLocalQueueAlias(queueManager, queueManagerName, queueName, aliasName);
            // remove alias from the queue to show an error
            removeLocalQueueAlias(queueManager, queueManagerName, queueName, aliasName);

            // delete the local queue
            deleteLocalQueue(queueManager, queueManagerName, queueName);
            // delete the local queue again to show an error
            deleteLocalQueue(queueManager, queueManagerName, queueName);

            CreateQueueManager.stopQueueManager(queueManager);
        } catch (MQeException e) {
            if (e.code() == MQeExceptionCodes.Except_QMgr_AlreadyExists) {
                System.out.println("Queue Manager already defined - please delete it and try again");
            } else {
                e.printStackTrace();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Create a local queue using a queue manager to access the MQe Network.  This routine will also
     * create a local queue on a remote queue manager, but there maust be a path for the admin message
     * to travel to the destination queue manager
     *
     * @param localQueueManager: the queue manager to use as an entry point
     * @param targetQueueManagerName: the name of the queue manager that will own the local queue
     * @param queueName: the name of the local queue being created
     */
    public static final void createLocalQueue(MQeQueueManager localQueueManager, String targetQueueManagerName, String queueName)
        throws Exception {
        // create a queue admin message, specifying the name of the queue and queue manager.
        MQeQueueAdminMsg msg = new MQeQueueAdminMsg(targetQueueManagerName, queueName);

        // set up for reply etc
        String uniqueTag = BasicAdministration.decorateAdminMsg(msg, targetQueueManagerName);

        // request a create
        msg.setAction(MQeAdminMsg.Action_Create);

        // put the message to the right admin queue
        localQueueManager.putMessage(targetQueueManagerName, MQe.Admin_Queue_Name, msg, null, 0L);

        // wait for the reply message
        MQeAdminMsg reply = BasicAdministration.waitForRemoteAdminReply(localQueueManager, targetQueueManagerName, uniqueTag);

        // get the return code from the reply, and check whether we succeeded
        final int returnCode = reply.getRC();

        if (MQeAdminMsg.RC_Success == returnCode) {
            System.out.println("Queue creation succeeded");
        } else {
            // if we fail we can examine the reason
            System.out.println("Queue creation failed: " + reply.getReason());
        }
    }

    /**
     * Delete a local queue using a queue manager to access the MQe Network.  This routine will also
     * delete a local queue on a remote queue manager, but there maust be a path for the admin message
     * to travel to the destination queue manager
     *
     * @param localQueueManager: the queue manager to use as an entry point
     * @param targetQueueManagerName: the name of the queue manager that owns the local queue
     * @param queueName: the name of the local queue being deleted
     */
    public static final void deleteLocalQueue(MQeQueueManager localQueueManager, String targetQueueManagerName, String queueName)
        throws Exception {
        // create a queue admin message, specifying the name of the queue and queue manager.
        MQeQueueAdminMsg msg = new MQeQueueAdminMsg(targetQueueManagerName, queueName);

        // set up for reply etc
        String uniqueTag = BasicAdministration.decorateAdminMsg(msg, targetQueueManagerName);

        // request a create
        msg.setAction(MQeAdminMsg.Action_Delete);

        // put the message to the right admin queue
        localQueueManager.putMessage(targetQueueManagerName, MQe.Admin_Queue_Name, msg, null, 0L);

        // wait for the reply message
        MQeAdminMsg reply = BasicAdministration.waitForRemoteAdminReply(localQueueManager, targetQueueManagerName, uniqueTag);

        // get the return code from the reply, and check whether we succeeded
        final int returnCode = reply.getRC();

        if (MQeAdminMsg.RC_Success == returnCode) {
            System.out.println("Queue deletion succeeded");
        } else {
            // if we fail we can examine the reason
            System.out.println("Queue deletion failed: " + reply.getReason());
        }
    }

    /**
     * Add an alias to a local queue using a queue manager to access the MQe Network.  This routine will also
     * add an alias to a local queue on a remote queue manager, but there maust be a path for the admin message
     * to travel to the destination queue manager
     *
     * @param localQueueManager: the queue manager to use as an entry point
     * @param targetQueueManagerName: the name of the queue manager that owns the local queue
     * @param queueName: the name of the local queue to which alias is being added
     */
    public static final void addLocalQueueAlias(MQeQueueManager localQueueManager, String targetQueueManagerName, String queueName, String aliasName)
        throws Exception {
        // create a queue admin message, specifying the name of the queue and queue manager.
        MQeQueueAdminMsg msg = new MQeQueueAdminMsg(targetQueueManagerName, queueName);

        // set up for reply etc
        String uniqueTag = BasicAdministration.decorateAdminMsg(msg, targetQueueManagerName);

        // request a create
        msg.addAlias(aliasName);

        // put the message to the right admin queue
        localQueueManager.putMessage(targetQueueManagerName, MQe.Admin_Queue_Name, msg, null, 0L);

        // wait for the reply message
        MQeAdminMsg reply = BasicAdministration.waitForRemoteAdminReply(localQueueManager, targetQueueManagerName, uniqueTag);

        // get the return code from the reply, and check whether we succeeded
        final int returnCode = reply.getRC();

        if (MQeAdminMsg.RC_Success == returnCode) {
            System.out.println("Alias addition succeeded");
        } else if (MQeAdminMsg.RC_Fail == returnCode) {
            // if we fail we can examine the reason
            System.out.println("Alias addition failed: " + reply.getReason());
        } else {
            System.out.println("Partial failure:\n" + reply.getErrorFields().toString());
        }
    }

    /**
     * Remove an alias from a local queue using a queue manager to access the MQe Network.  This routine will also
     * remove an alias from a local queue on a remote queue manager, but there maust be a path for the admin message
     * to travel to the destination queue manager
     *
     * @param localQueueManager: the queue manager to use as an entry point
     * @param targetQueueManagerName: the name of the queue manager that owns the local queue
     * @param queueName: the name of the local queue from which alias is being removed
     */
    public static final void removeLocalQueueAlias(MQeQueueManager localQueueManager, String targetQueueManagerName, String queueName, String aliasName)
        throws Exception {
        // create a queue admin message, specifying the name of the queue and queue manager.
        MQeQueueAdminMsg msg = new MQeQueueAdminMsg(targetQueueManagerName, queueName);

        // set up for reply etc
        String uniqueTag = BasicAdministration.decorateAdminMsg(msg, targetQueueManagerName);

        // request a create
        msg.removeAlias(aliasName);

        // put the message to the right admin queue
        localQueueManager.putMessage(targetQueueManagerName, MQe.Admin_Queue_Name, msg, null, 0L);

        // wait for the reply message
        MQeAdminMsg reply = BasicAdministration.waitForRemoteAdminReply(localQueueManager, targetQueueManagerName, uniqueTag);

        // get the return code from the reply, and check whether we succeeded
        final int returnCode = reply.getRC();

        if (MQeAdminMsg.RC_Success == returnCode) {
            System.out.println("Alias removal succeeded");
        } else {
            // if we fail we can examine the reason
            System.out.println("Alias removal failed: " + reply.getReason());
        }
    }

}
