// Version mqe_java/source/examples/install/ConfigUtils.java, MQeExamples, la200, la200-L030324.1 1.7

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.install;


import com.ibm.mqe.*;

import examples.queuemanager.MQeQueueManagerUtils;

import java.io.*;
import java.util.ResourceBundle;


/*
 *  <p>A collection of utility methods used throughout the configuration       
 *  classes. They are collected here so that they can be easily re-used     
 *  and hence avoid duplication.                                                                                
 **/

public class ConfigUtils {
    public       static short version[] = { 2, 0, 0, 10 }; 

    // Resources
    ResourceBundle resources;

    // Other Strings, for NLS
    String noPortStr;
    String negativePortStr;
    String nonNumPortStr;
    String noDirStr;

    /**
     * ConfigUtils constructor.
     **/
    public ConfigUtils() {
        // initialise all the NLS strings
        initStrings();
    }

    /**
     * Check for a valid port number.
     *
     * @param  port   The String value to check
     * @return        An error message if the port number is invalid,
     *                or null if the port number is valid.
     **/
    public String checkPortNumber(String port) {
        if (port == null || port.equals(""))
            return noPortStr;

        // check if it is positive and numeric
        if (port.charAt(0) == '-')
            return negativePortStr;

        try {
            Integer.parseInt(port);
        } catch (NumberFormatException e) {
            return nonNumPortStr;
        }

        return null;
    }

    /**
     * Get the ResourceBundle for NLS.
     *
     * This method is provided so that we don't have to load
     * the bundle twice (once for ConfigUtils and again for the
     * class which uses ConfigUtils).
     *
     * The bundle will always be loaded (by the constructor) before
     * this method can be called.
     * @return ResourceBundle 
     **/
    public ResourceBundle getResourceBundle() {
        return resources;
    }

    /**
     * Ensure that a directory exists - create it if necessary.
     *
     * @param  dirName    The name of the directory.
     * @return            'true' if the directory had to be created
     *                    'false' if it already exists
     * @exception  java.lang.Exception  If the directory cannot be created
     **/
    public boolean makeDirectory(String dirName) throws Exception {
        boolean result = false;

        // check if it exists
        File dirFile = new File(dirName);

        if (!dirFile.exists()) {
            dirFile.mkdirs();
            result = true;
        }

        // check that it does exist and is a directory
        if (!dirFile.exists() || !dirFile.isDirectory())
            throw new Exception(noDirStr + ": '" + dirName + "'");

        // it exists
        return result;
    }

    /**
     * Verify that a file is a valid MQe configuration file.
     *
     * @param  filename   The name of the file.
     * @return            If it is a valid MQe configuration file,
     *                    this returns a MQeFields object containing
     *                    the configuration parameters.
     *                    If it is not valid, this returns null;
     **/
    public MQeFields verifyIniFile(String filename) {
        MQeFields result = null;
        String qMgrName = null;

        try {
            // load the file
            result = MQeQueueManagerUtils.loadConfigFile(filename);

            // see if it is a valid configuration file by looking
            // for a queue manager name
            if (result.contains(MQeQueueManager.QueueManager)) {
                MQeFields qMgrFields = result.getFields(MQeQueueManager.QueueManager);

                if (qMgrFields.contains(MQeQueueManager.Name))
                    qMgrName = qMgrFields.getAscii(MQeQueueManager.Name);
            }
        } catch (Exception e) {}

        // if we didn't find a queue manager name, it is not
        // a valid configuration file
        if (qMgrName == null)
            result = null;

        return result;
    }

    /**
     * Initialise all the strings using a ResourceBundle.
     * @return void
     **/
    private void initStrings() {
        // Resources
        resources = ResourceBundle.getBundle("examples.install.ConfigResource");

        noPortStr = resources.getString("noPort");
        negativePortStr = resources.getString("negativePort");
        nonNumPortStr = resources.getString("nonNumPort");
        noDirStr = resources.getString("noDir");

    }

}

