/*------------------------------------------------------------------------------*/

/*                 WebSphere MQ Everyplace for Multiplatforms                   */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5724-C77                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999, 2002                                           */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.jms;


import javax.naming.*;
import javax.naming.directory.*;
import java.util.Properties;
import com.ibm.mqe.jms.*;


/*
 *  Example to create JNDI entries for WebSphere MQ Everyplace support for Java
 *  Message Service.
 *
 *  This program creates entries in a JNDI context. It can create one entry for
 *  a QueueConnectionFactory and one entry for a Queue.
 *
 *  Usage:
 *
 *      java CreateJNDIEntry -url <providerURL> [ -icf <initialContextFactory> ]
 *                        [ -ldap ] [ -q <entry name> <MQe queue name> ]
 *                        [ -qcf <entry name> <MQe queue manager ini file> ]
 *
 *  or  java CreateJNDIEntry -h
 *
 *  where:
 *
 *      -url <providerURL>         - the URL of the JNDI initial context. This
 *                                   parameter is obligatory.
 *
 *      -icf <initialContextFactory> - the initialContextFactory for JNDI. This
 *                                   defaults to the file system:
 *                                   com.sun.jndi.fscontext.RefFSContextFactory
 *
 *      -ldap                      - the JNDI provider is an LDAP directory
 *
 *      -q <entry name> <MQe queue name>
 *                                 - the name of a JNDI entry to be created for
 *                                   a JMS Queue and the name of the MQe queue
 *                                   to be used to configure it. The MQe queue
 *                                   can be specified as <qmgr>+<queue>. If
 *                                   <qmgr> is not specified, the local queue
 *                                   manager is assumed.
 *
 *      -qcf <entry name> <MQe queue manager ini file>
 *                                 - the name of a JNDI entry to be created for
 *                                   a JMS QueueConnectionFactory and the name
 *                                   of an initialisation (ini) file for a MQe
 *                                   queue manager to be used to configure it
 *
 *      -h                         - displays a help message
 *
 */




class CreateJNDIEntry {
    public static short  version[] = { 2, 0, 0, 10 }; 

    Properties         env = new Properties();
    Context            ctx = null;
    boolean            ldap = false;

    // the Initial Context Factory defaults to Sun's File System context. Other values
    // include Sun's LDAP context: "com.sun.jndi.ldap.LdapCtxFactory"
    // or IBM's LDAP context:      "com.ibm.jndi.LDAPCtxFactory"
    String             icf = "com.sun.jndi.fscontext.RefFSContextFactory";
    String             url = null;
    String             qEntry = null;
    String             mqeQueue = null;
    String             qcfEntry = null;
    String             iniFileName = null;

    /**
     * the main routine
     **/
    public static void main(String[] args) {
        CreateJNDIEntry jndi = new CreateJNDIEntry();

        jndi.processArguments(args);
        jndi.createContext();
        jndi.addQCF();
        jndi.addQ();
    }

    /**
     * process the command line arguments
     * This is not very sophisticated.
     */
    void processArguments(String[] args) {
        boolean exit = false;

        // Process the command-line arguments
        for (int i = 0; i < args.length; i++) {
            String arg = args[i].toLowerCase();

            if (arg.equals("-url")) {
                // next argument should be the url
                if (i + 1 < args.length)
                    url = args[++i];
                else
                    System.err.println("too few parameters");
            } else if (arg.equals("-icf")) {
                // next argument should be the initial context factory
                if (i + 1 < args.length)
                    icf = args[++i];
                else
                    System.err.println("too few parameters");
            } else if (arg.equals("-ldap")) {
                ldap = true;
            } else if (arg.equals("-q")) {
                // next 2 arguments should be the JNDI entry name for a
                // JMS Queue and the MQe Queue name
                if (i + 2 < args.length) {
                    qEntry = args[++i];
                    mqeQueue = args[++i];
                } else
                    System.err.println("too few parameters");
            } else if (arg.equals("-qcf")) {
                // next 2 arguments should be the JNDI entry name for a
                // JMS QueueConnectionFactory and the ini file for the
                // MQe Queue Manager
                if (i + 2 < args.length) {
                    qcfEntry = args[++i];
                    iniFileName = args[++i];
                } else
                    System.err.println("too few parameters");
            } else if (arg.equals("-h") || arg.equals("-?")) {
                exit = true;
            } else {
                System.err.println("unrecognised flag: " + arg);
                exit = true;
            }
        }

        // perform basic checks on the arguments
        if (url == null) {
            System.err.println("url must be specified");
            exit = true;
        }

        if (qEntry == null && qcfEntry == null) {
            System.err.println("no entries to add");
            exit = true;
        }

        // if exit == true, display usage and exit
        if (exit) {
            displayUsage();
            System.exit(-1);
        }
    }

    /**
     * create the initial JNDI context
     */
    void createContext() {
        env.put(Context.INITIAL_CONTEXT_FACTORY, icf);
        env.put(Context.PROVIDER_URL, url);
        //May also need to add security details here
        //env.put(Context.SECURITY_AUTHENTICATION, "simple");
        //env.put(Context.SECURITY_PRINCIPAL, "cn=Administrator...");
        //env.put(Context.SECURITY_CREDENTIALS, "password");

        try {
            ctx = new InitialContext(env);
            System.out.println("JNDI context created");
        } catch (NamingException e) {
            System.out.println("exception creating context: " + e);
        }
    }

    /**
     * add a Queue Connection Factory, if requested
     */
    void addQCF() {
        if (ctx != null && qcfEntry != null && qcfEntry.length() != 0) {
            String entryName = qcfEntry;

            // if using LDAP, add a "cn=" prefix to the name
            if (ldap)
                entryName = "cn=" + entryName;

            try {
                MQeJNDIQueueConnectionFactory factory = new MQeJNDIQueueConnectionFactory();

                factory.setIniFileName(iniFileName);
                ctx.bind(entryName, factory);
                System.out.println("added JNDI Queue Connection Factory entry: " + qcfEntry);
            } catch (Exception e) {
                System.out.println("exception adding Queue Connection Factory: " + e);
            }
        }
    }

    /**
     * add a Queue, if requested
     */
    void addQ() {
        if (ctx != null && qEntry != null && qEntry.length() != 0) {
            String entryName = qEntry;

            // if using LDAP, add a "cn=" prefix to the name
            if (ldap)
                entryName = "cn=" + entryName;

            // does the queue name include a queue manager component
            // <qmgr>+<queue>
            String mqeQMgr = null;
            int i = mqeQueue.indexOf('+');

            if (i > -1) {
                mqeQMgr = mqeQueue.substring(0, i);
                mqeQueue = mqeQueue.substring(i + 1);
            }

            try {
                MQeJMSJNDIQueue queue = new MQeJMSJNDIQueue(mqeQMgr, mqeQueue);

                ctx.bind(entryName, queue);
                System.out.println("added JNDI Queue entry: " + qEntry);
            } catch (Exception e) {
                System.out.println("exception adding Queue entry: " + e);
            }
        }
    }

    /**
     * displayUsage - displays usage information to stdout
     */
    private static void displayUsage() {
        System.out.println("usage: java CreateJNDIEntry -url <providerURL> [ -icf <initialContextFactory> ]" +
            " [ -ldap ] [ -q <entry name> <MQe queue name> ]" +
            " [ -qcf <entry name> <MQe queue manager ini file> ]");
    }

}

