/*------------------------------------------------------------------------------*/

/*                 WebSphere MQ Everyplace for Multiplatforms                   */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5724-C77                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999, 2002                                           */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.jms;


import javax.jms.*;

// we need MQeQueueConnectionFactory because we are not using JNDI
import com.ibm.mqe.jms.MQeQueueConnectionFactory;

// Used for Trace
import com.ibm.mqe.MQeTrace;
import com.ibm.mqe.MQeTraceHandler;
import com.ibm.mqe.trace.*;


/*
 *  IVT example for WebSphere MQ Everyplace support for Java Message Service
 *  without JNDI.
 *
 *  This program connects to a queue manager on the local machine. It sends
 *  a messge to the queue SYSTEM.DEFAULT.LOCAL.QUEUE and reads it back again.
 *
 *  This version does not use JNDI. It configures the QueueConnectionFactory
 *  and Queue directly in the code. The connection factory is configured
 *  using the name of an initialisation file which is passed in as a command
 *  line parameter.
 *
 *  Usage:
 *
 *      java MQeJMSIVT  -i <ini file name>
 *  or  java MQeJMSIVT -h
 *
 *  where:
 *
 *      -t                         - enables tracing
 *
 *      -i <ini file name>         - the name of the initialisation (.ini) file
 *                                   for a local MQe queue manager. The queue
 *                                   manager must have a queue called
 *                                   SYSTEM.DEFAULT.LOCAL.QUEUE
 *
 *      -h                         - displays a help message
 *
 */



public class MQeJMSIVT {
    public static short  version[] = { 2, 0, 0, 10 }; 

    public static String iniFileName = null;
    public static final String queueName = "SYSTEM.DEFAULT.LOCAL.QUEUE";

    public static void main(String[] args) {
        String                 outString = "A simple text message from the MQeJMSIVT program";
        Queue                  ioQueue = null;
        QueueSession           session = null;
        QueueConnection        connection = null;
        QueueConnectionFactory factory = null;

        boolean                enableTrace = false;

        // Process the command-line arguments
        for (int i = 0; i < args.length; i++) {
            String arg = args[i].toLowerCase();

            if (arg.equals("-t")) {
                enableTrace = true;
            } else if (arg.equals("-i")) {
                // Obtain the name of the .ini file for the MQe queue manager
                if (i + 1 < args.length)
                    iniFileName = args[++i];
                else {
                    System.err.println("too few parameters");
                }
            } else if (arg.equals("-h") || arg.equals("-?")) {
                displayUsage();
                System.exit(-1);
            }
        }

        // If the user has asked trace to be enabled, then enable it
        if (enableTrace) {
            System.out.println("turning trace on");
            MQeTraceHandler trace = (MQeTraceHandler) new MQeTraceToReadable();

            MQeTrace.setHandler(trace);
            MQeTrace.setFilter(MQeTrace.GROUP_JMS);
        }

        // Validate the options on the command line
        if (iniFileName == null || iniFileName.length() == 0) {
            System.err.println("an ini file name must be specified");
            displayUsage();
            System.exit(-1);
        } else {
            System.out.println("using ini file '" + iniFileName + "' to configure the connection");
        }

        // check the classpath
        System.out.println("checking classpath");

        // Sun's JMS interfaces
        // This test is superfluous because the program will not load correctly without these classes
        try {
            Class.forName("javax.jms.Message");
            System.out.println("found JMS interface classes");
        } catch (ClassNotFoundException e) {
            System.err.println("Cannot find javax.jms.Message, is jms.jar on the classpath?");
            System.exit(-1);
        }

        // the WebSphere MQ Everyplace JMS classes
        // This test is superfluous because the program will not load correctly without these classes
        try {
            Class.forName("com.ibm.mqe.jms.MQeMessage");
            System.out.println("found MQe JMS classes");
        } catch (ClassNotFoundException e) {
            System.err.println("Cannot find com.ibm.mqe.jms.MQeMessage, is MQeJMS.jar on the classpath?");
            System.exit(-1);
        }

        // the WebSphere MQ Everyplace base classes
        try {
            Class.forName("com.ibm.mqe.MQe");
            System.out.println("found MQe base classes");
        } catch (ClassNotFoundException e) {
            System.err.println("Cannot find com.ibm.mqe.MQe, is MQeBase.jar on the classpath?");
            System.exit(-1);
        }

        // create the JMS classes and send and receive the message
        try {
            // Create a QueueConnectionFactory ourselves
            System.out.println("Creating and configuring QueueConnectionFactory");
            factory = new MQeQueueConnectionFactory();
            ((MQeQueueConnectionFactory) factory).setIniFileName(iniFileName);

            System.out.println("Creating connection");
            connection = factory.createQueueConnection();
            // Receive calls will block if connection is not started
            connection.start();

            // Create a connectionMetaData to get provider details
            ConnectionMetaData CMD = connection.getMetaData();
            String pn = CMD.getJMSProviderName();
            String pv = CMD.getProviderVersion();

            System.out.println("From the connection data, JMS provider is " + pn + " Version " + pv);

            // Create session
            boolean transacted = false;

            System.out.println("Creating session");
            session = connection.createQueueSession(transacted, Session.AUTO_ACKNOWLEDGE);

            // Create queue
            System.out.println("Creating queue");
            ioQueue = session.createQueue(queueName);

            // Create sender
            System.out.println("Creating sender");
            QueueSender queueSender = session.createSender(ioQueue);

            // Create receiver
            System.out.println("Creating receiver");
            QueueReceiver queueReceiver = session.createReceiver(ioQueue);

            // Create a message
            System.out.println("Creating message");
            TextMessage outMessage = session.createTextMessage(outString);

            // Send it
            System.out.println("Sending message");
            queueSender.send(outMessage);

            // Get it back again. Time out after 1s if unable to find it
            System.out.println("Receiving message");
            Message inMessage = queueReceiver.receive(1000);

            if (inMessage == null) {
                System.err.println("Failed to receive the message");
                // Throwing an exception indicates that the test failed and
                // takes us into the clean routine
                throw new JMSException("Failed to receive the message");
            }

            // print the message to the screen
            System.out.println("");
            System.out.println(inMessage.toString());

            // Check that the reply matches what we sent
            if (inMessage instanceof TextMessage) {
                System.out.println("");
                System.out.println("Retrieved message is a TextMessage; now checking " +
                    "for equality with the sent message");

                String replyString = ((TextMessage) inMessage).getText();

                if (replyString.equals(outString)) {
                    System.out.println("Messages are equal. Great!");
                } else {
                    System.err.println("ERROR! Messages differ!");
                    System.err.println("Original=" + outString);
                    System.err.println("Reply=" + replyString);
                }
            } else {
                System.err.println("The retrieved message was not a TextMessage");
                // Throwing an exception indicates that the test failed and
                // takes us into the clean routine
                throw new JMSException("The retrieved message was not a TextMessage");
            }

            // Close everything down.
            System.out.println("Closing connection");
            connection.close();
            connection = null;

            // If we get here, hopefully everything worked.
            System.out.println("connection closed");
        } catch (Exception e) {
            System.err.println("Exception caught: " + e);

            // check for embedded exceptions in JMSException
            if (e instanceof JMSException) {
                Exception le = ((JMSException) e).getLinkedException();

                if (le != null) {
                    System.err.println("linked exception: " + le);
                }
            }
        }

        System.out.println("IVT finished");
    }

    //------------------------------------------------------------------------
    // displayUsage - displays usage information to stdout
    //------------------------------------------------------------------------
    private static void displayUsage() {
        System.out.println("usage: java MQeJMSIVT [-t] -i <ini file name>");
    }

}

