/*------------------------------------------------------------------------------*/

/*                 WebSphere MQ Everyplace for Multiplatforms                   */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5724-C77                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999, 2002                                           */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.jms;


/*
 *  IVT example for WebSphere MQ Everyplace support for Java Message Service
 *  with JNDI
 *
 *  This program connects to a queue manager on the local machine. It sends
 *  a messge to the queue SYSTEM.DEFAULT.LOCAL.QUEUE and reads it back again.
 *
 *  This version uses JNDI. It requires that there are two administered objects
 *  set up in the JNDI directory namespace:
 *
 *   (i) an MQeQueueConnectionFactory bound under the name "ivtQCF", configured
 *       with an initiaisation (ini) file name for an MQe queue manager.
 *
 *  (ii) an MQeJMSQueue bound under the name "ivtQ". This should refer to a
 *       local queue that exists on the queue manager that the connection
 *       factory is configured to work with, for example SYSTEM.DEFAULT.LOCAL.QUEUE.
 *
 *  This test will fail if these objects cannot be retrieved from a JNDI
 *  service provider. They can be created using the example program ??????????
 *
 *
 *  Usage:
 *
 *      java MQeJMSIVT_JNDI -url providerURL [ -icf initialContextFactory ] [ -t ]
 *  or  java MQeJMSIVT_JNDI -h
 *
 *  where:
 *
 *      -t                         - enables tracing
 *
 *      -url <providerURL>         - the URL of the JNDI initial context. This
 *                                   parameter is obligatory.
 *
 *      -icf <initialContextFactory> - the initialContextFactory for JNDI. This
 *                                   defaults to the file system:
 *                                   com.sun.jndi.fscontext.RefFSContextFactory
 *
 *      -h                         - displays a help message
 *
 */


import javax.jms.*;
import javax.naming.*;
import javax.naming.directory.*;

// Used for Trace
import com.ibm.mqe.MQeTrace;
import com.ibm.mqe.MQeTraceHandler;
import com.ibm.mqe.trace.*;


public class MQeJMSIVT_JNDI {
    public static short  version[] = { 2, 0, 0, 10 }; 

    public static final String qcfLookup = "ivtQCF";
    public static final String qLookup = "ivtQ";

    public static void main(String[] args) {
        String                 outString = "A simple text message from the MQeJMSIVT_JNDI program";
        Queue                  ioQueue = null;
        QueueSession           session = null;
        QueueConnection        connection = null;
        QueueConnectionFactory factory = null;
        Context                ctx = null;

        boolean                enableTrace = false;
        String             icf = "com.sun.jndi.fscontext.RefFSContextFactory";
        String             url = null;

        // Process the command-line arguments
        for (int i = 0; i < args.length; i++) {
            String arg = args[i].toLowerCase();

            if (arg.equals("-t")) {
                enableTrace = true;
            } else if (arg.equals("-url")) {
                if (i + 1 < args.length)
                    url = args[++i];
                else {
                    System.err.println("too few parameters");
                }
            } else if (arg.equals("-icf")) {
                if (i + 1 < args.length)
                    icf = args[++i];
                else {
                    System.err.println("too few parameters");
                }
            } else if (arg.equals("-h") || arg.equals("-?")) {
                displayUsage();
                System.exit(-1);
            }
        }

        // If the user has asked trace to be enabled, then enable it
        if (enableTrace) {
            System.out.println("turning trace on");
            MQeTraceHandler trace = (MQeTraceHandler) new MQeTraceToReadable();

            MQeTrace.setHandler(trace);
            MQeTrace.setFilter(MQeTrace.GROUP_JMS);
        }

        // Validate the options on the command line
        System.out.println("using context factory '" + icf + "' for the directory");
        if (url == null || url.length() == 0) {
            System.err.println("a url for the directory must be specified");
            displayUsage();
            System.exit(-1);
        } else {
            System.out.println("using directory url '" + url + "'");
        }

        // check the classpath
        boolean missingClasses = false;

        System.out.println("checking classpath");

        // Sun's JMS interfaces
        // This test is superfluous because the program will not load correctly without these classes
        try {
            Class.forName("javax.jms.Message");
            System.out.println("found JMS interface classes");
        } catch (ClassNotFoundException e) {
            System.err.println("Cannot find javax.jms.Message, is jms.jar on the classpath?");
            missingClasses = true;
        }

        // the WebSphere MQ Everyplace JMS classes
        try {
            Class.forName("com.ibm.mqe.jms.MQeMessage");
            System.out.println("found MQe JMS classes");
        } catch (ClassNotFoundException e) {
            System.err.println("Cannot find com.ibm.mqe.jms.MQeMessage, is MQeJMS.jar on the classpath?");
            missingClasses = true;
        }

        // the WebSphere MQ Everyplace base classes
        try {
            Class.forName("com.ibm.mqe.MQe");
            System.out.println("found MQe base classes");
        } catch (ClassNotFoundException e) {
            System.err.println("Cannot find com.ibm.mqe.MQe, is MQeBase.jar on the classpath?");
            missingClasses = true;
        }

        // the JNDI jar file
        try {
            Class.forName("javax.naming.InitialContext");
            System.out.println("found jndi.jar classes");
        } catch (ClassNotFoundException e) {
            System.err.println("Cannot find javax.naming.InitialContext, is jndi.jar on the classpath?");
            missingClasses = true;
        }

        // the jndi provider classes
        try {
            Class.forName(icf);
            System.out.println("found " + icf + " classes");
        } catch (ClassNotFoundException e) {
            System.err.println("Cannot find " + icf + " classes");
            missingClasses = true;
        }

        // some versions of Sun's jndi providers need providerutil.jar
        // but don't check for it - can't guarantee the classnames
        // won't change. Fortunately, the default error when this jar
        // is missing is ok.

        if (missingClasses) {
            System.exit(-1);
        }

        // Extract the classes from jndi

        // first set up the JNDI namespace
        java.util.Hashtable env = new java.util.Hashtable();

        env.put(Context.INITIAL_CONTEXT_FACTORY, icf);
        env.put(Context.PROVIDER_URL, url);
        // LDAP Error 84 workaround follows:
        env.put(Context.REFERRAL, "throw");

        // create the JNDI context
        try {
            ctx = new InitialContext(env);
        } catch (Exception e) {
            System.err.println("Error creating JNDI conect: " + e);
        }

        // lookup the connection factory
        System.out.println("Looking up connection factory in jndi");
        try {
            factory = (QueueConnectionFactory) ctx.lookup(qcfLookup);
        } catch (Exception e) {
            try {
                // first lookup has failed, so try the cn= prefix (in case LDAP is
                // being used)
                factory = (QueueConnectionFactory) ctx.lookup("cn=" + qcfLookup);
            } catch (Exception e2) {
                System.err.println("jndi lookup of '" + qcfLookup + "' failed");
                System.exit(-1);
            }
        }

        // lookup the queue
        System.out.println("Looking up queue in jndi");
        try {
            ioQueue = (Queue) ctx.lookup(qLookup);
        } catch (Exception e) {
            try {
                // first lookup has failed, so try the cn= prefix (in case LDAP is
                // being used)
                ioQueue = (Queue) ctx.lookup("cn=" + qLookup);
            } catch (Exception e2) {
                System.err.println("jndi lookup of '" + qcfLookup + "' failed");
                System.exit(-1);
            }
        }

        // create the remaining JMS classes and send and receive the message
        try {
            System.out.println("Creating connection");
            connection = factory.createQueueConnection();
            // Receive calls will block if connection is not started
            connection.start();

            // Create a connectionMetaData to get provider details
            ConnectionMetaData CMD = connection.getMetaData();
            String pn = CMD.getJMSProviderName();
            String pv = CMD.getProviderVersion();

            System.out.println("From the connection data, JMS provider is " + pn + " Version " + pv);

            // Create session
            boolean transacted = false;

            System.out.println("Creating session");
            session = connection.createQueueSession(transacted, Session.AUTO_ACKNOWLEDGE);

            // Create sender
            System.out.println("Creating sender");
            QueueSender queueSender = session.createSender(ioQueue);

            // Create receiver
            System.out.println("Creating receiver");
            QueueReceiver queueReceiver = session.createReceiver(ioQueue);

            // Create a message
            System.out.println("Creating message");
            TextMessage outMessage = session.createTextMessage(outString);

            // Send it
            System.out.println("Sending message");
            queueSender.send(outMessage);

            // Get it back again. Time out after 1s if unable to find it
            System.out.println("Receiving message");
            Message inMessage = queueReceiver.receive(1000);

            if (inMessage == null) {
                System.err.println("Failed to receive the message");
                // Throwing an exception indicates that the test failed and
                // takes us into the clean routine
                throw new JMSException("Failed to receive the message");
            }

            // print the message to the screen
            System.out.println("");
            System.out.println(inMessage.toString());

            // Check that the reply matches what we sent
            if (inMessage instanceof TextMessage) {
                System.out.println("");
                System.out.println("Retrieved message is a TextMessage; now checking " +
                    "for equality with the sent message");

                String replyString = ((TextMessage) inMessage).getText();

                if (replyString.equals(outString)) {
                    System.out.println("Messages are equal. Great!");
                } else {
                    System.err.println("ERROR! Messages differ!");
                    System.err.println("Original=" + outString);
                    System.err.println("Reply=" + replyString);
                }
            } else {
                System.err.println("The retrieved message was not a TextMessage");
                // Throwing an exception indicates that the test failed and
                // takes us into the clean routine
                throw new JMSException("The retrieved message was not a TextMessage");
            }

            // Close everything down.
            System.out.println("Closing connection");
            connection.close();
            connection = null;

            // If we get here, hopefully everything worked.
            System.out.println("connection closed");
        } catch (Exception e) {
            System.err.println("Exception caught: " + e);

            // check for embedded exceptions in JMSException
            if (e instanceof JMSException) {
                Exception le = ((JMSException) e).getLinkedException();

                if (le != null) {
                    System.err.println("linked exception: " + le);
                }
            }
        }

        System.out.println("IVT finished");
    }

    //------------------------------------------------------------------------
    // displayUsage - displays usage information to stdout
    //------------------------------------------------------------------------
    private static void displayUsage() {
        System.out.println("usage: java MQeJMSIVT_JNDI [-t] -url <providerURL> [-icf <initialContextFactory>]");
    }

}

