/*------------------------------------------------------------------------------*/

/*                 WebSphere MQ Everyplace for Multiplatforms                   */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5724-C77                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999, 2002                                           */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.jms;


// Import of Sun's JMS interface
import javax.jms.*;

// Imports for JNDI
import javax.naming.*;
import javax.naming.directory.*;

// Used for JNDI initialisation
import java.util.Hashtable;

// The following import would not normally be required in a JMS application,
// but is included here to allow us to bypass the JNDI lookup stage
import com.ibm.mqe.jms.MQeQueueConnectionFactory;

// Used for Trace
import com.ibm.mqe.MQeTrace;
import com.ibm.mqe.MQeTraceHandler;
import com.ibm.mqe.trace.*;


/*
 *  Point to Point Sample02.
 *
 *  Demonstrates:
 *    how to use a MessageListener for asynchronous message delivery.
 *    how to use message selectors to filter the input stream.
 *
 * The program creates two QueueReceivers on the same queue, each with
 * a MessageListener.  One QueueReceiver has a message selector for the
 * user defined property colour=red, the other has a selector for
 * colour=blue. A QueueSender is used to send four messages to the
 * queue, two with colour=red, two with colour=blue. These should be
 * received and displayed by the appropriate MessageListener instance.
 *
 *  This sample uses the same administered objects as the installation
 *  verification test program. These are:
 *
 *   (i) an MQeQueueConnectionFactory bound under the name "ivtQCF",
 *       configured with a queue manager initialisation (ini) file.
 *
 *  (ii) an MQeJMSQueue bound under the name "ivtQ", representing a local
 *       queue on the queue manager.
 *
 *  This sample will fail if these objects cannot be retrieved from a JNDI
 *  service provider, unless -nojndi is specified (see below). They can be
 *  created using the CreateJNDIEntry program.
 *
 *  Usage:
 *
 *      java PTPSample02 -nojndi -i <ini file name> [ -q <queue name> ] [ -t ]
 *  or  java PTPSample02 -url <providerURL> [ -icf <initialContextFactory> ] [ -t ]
 *
 *  where:
 *
 *      -nojndi                    - specify this to disable JNDI lookup of
 *                                   the two required administered objects
 *
 *      -i <ini file name>         - the name of the initialisation (.ini) file
 *                                   for a local MQe queue manager. This is used
 *                                   when the -nojndi flag is specified.
 *
 *      -q <queue name>            - the name of the MQe queue to use. This must
 *                                   be local to the MQe queue manager, configured
 *                                   by the ini file (-i). If this is not specified
 *                                   the default is SYSTEM.DEFAULT.LOCAL.QUEUE. This
 *                                   is used when the -nojndi flag is specified.
 *
 *      -t                         - enabled tracing
 *
 *      -url providerURL           - specify the URL of the JNDI initial
 *                                   context This parameter is obligatory
 *                                   unless the -nojndi switch is supplied,
 *                                   in which case it is ignored.
 *
 *      -icf initialContextFactory - specify the initialContextFactory for
 *                                   JNDI This defaults to the file system:
 *                                   com.sun.jndi.fscontext.RefFSContextFactory.
 *                                   This is used when the -nojndi flag is not
 *                                   specified.
 *
 */

public class PTPSample02 {
    public static short  version[] = { 2, 0, 0, 10 }; 

    // These are used when the -nojndi flag is given.
    public static String iniFileName = null;
    public static String queueName = "SYSTEM.DEFAULT.LOCAL.QUEUE";

    // The following can be modified if you want to experiment with
    // using other administered objects with different content.
    public static final String  qcfLookup = "ivtQCF";
    public static final String  qLookup = "ivtQ";

    // edit these to match your environment
    public static String icf = "com.sun.jndi.fscontext.RefFSContextFactory";
    public static String url = "file:d:/MQe/jms/MQsamples/jndi/";

    /** entry point when called from command line. */
    public static void main(String[] args) {
        Queue                   ioQueue = null;
        QueueSession            session = null;
        QueueConnection         connection = null;
        QueueConnectionFactory  factory = null;
        Context                 ctx = null;

        boolean            useJNDI = true;
        boolean            enableTrace = false;
        String             icf = "com.sun.jndi.fscontext.RefFSContextFactory";
        String             url = null;

        System.out.println("PTPSample02 - simple example of using asynchronous message delivery.");

        // Process each of the command-line arguments in turn
        for (int i = 0; i < args.length; i++) {
            String arg = args[i].toLowerCase();

            if (arg.equals("-nojndi")) {
                // If the nojndi flag has been set then the program will not
                // use JNDI to obtain the administered objects, but will
                // create them at runtime
                useJNDI = false;
            } else if (arg.equals("-i")) {
                // Obtain the name of the queue manager initialisation (ini) file
                // if we are not using JNDI
                if (i + 1 < args.length)
                    iniFileName = args[++i];
                else
                    System.out.println("Ignoring -i flag; no value supplied");
            } else if (arg.equals("-q")) {
                // Obtain the name of the MQe queue to use
                // if we are not using JNDI
                if (i + 1 < args.length)
                    queueName = args[++i];
                else
                    System.out.println("Ignoring -q flag; no value supplied");
            } else if (arg.equals("-t")) {
                // Enable program tracing
                enableTrace = true;
            } else if (arg.equals("-url")) {
                // Obtain the URL for JNDI operations
                if (i + 1 < args.length)
                    url = args[++i];
                else
                    System.out.println("Ignoring -url flag; " +
                        "no value supplied");
            } else if (arg.equals("-icf")) {
                // Obtain the name of the class used for creating JNDI's
                // initialContext
                if (i + 1 < args.length)
                    icf = args[++i];
                else
                    System.out.println("Ignoring -icf flag; " +
                        "no value supplied");
            } else
                // Report and discard any unknown command-line options
                System.out.println("Ignoring unknown flag: " + arg);
        }

        // check the parameters
        if (useJNDI && url == null) {
            System.out.println("Must specify a url if using JNDI");
            System.exit(-1);
        }

        if (!useJNDI && iniFileName == null) {
            System.out.println("Must specify an initialisation file (-i) with -nojndi");
            System.exit(-1);
        }

        // Enable trace, if this is required
        if (enableTrace) {
            MQeTraceHandler trace = (MQeTraceHandler) new MQeTraceToReadable();

            MQeTrace.setHandler(trace);
            MQeTrace.setFilter(MQeTrace.GROUP_JMS);
        }

        // For simplicity this sample uses a single try block to catch all
        // Exceptions. In a real application you would probably use several
        // try blocks, allowing more specific error reporting and more options
        // for recovery.
        try {
            // Get the connection factory
            if (useJNDI) {
                // obtain the initial context
                System.out.println("Getting administered objects from JNDI");
                ctx = initJNDI(icf, url);
                // get the connection factory from JNDI
                factory = getConnectionFactoryFromJNDI(ctx, qcfLookup);
            } else {
                // create a connection factory directly
                // Note that this requires a reference to the underlying
                // class that implements QueueConnectionFactory
                System.out.println("Creating a QueueConnectionFactory");
                factory = new MQeQueueConnectionFactory();
                ((MQeQueueConnectionFactory) factory).setIniFileName(iniFileName);
            }

            // Create a connection
            System.out.println("creating connection");
            connection = factory.createQueueConnection();

            // Create a session
            System.out.println("creating session");
            session = connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);

            // Get the Queue (the specification of where to send our message)
            if (useJNDI) {
                // get the Queue from JNDI
                ioQueue = getQueueFromJNDI(ctx, qLookup);
            } else {
                // Use the session to create the queue object, supplying
                // the required MQSeries-specific parameter
                ioQueue = session.createQueue(queueName);
            }

            // Create QueueReceiver with a message selector for 'red' messages
            System.out.println("creating queue receiver");
            QueueReceiver receiver1 = session.createReceiver(ioQueue, "colour='red'");

            // register a listener
            System.out.println("registering message listener 1 for red messages");
            PTPSample02MListener listener1 = new PTPSample02MListener(1);

            receiver1.setMessageListener(listener1);

            // Create another QueueReceiver for 'blue' messages
            System.out.println("creating queue receiver");
            QueueReceiver receiver2 = session.createReceiver(ioQueue, "colour='blue'");

            // register a listener
            System.out.println("registering message listener 2 for blue messages");
            PTPSample02MListener listener2 = new PTPSample02MListener(2);

            receiver2.setMessageListener(listener2);

            // Create a QueueSender
            System.out.println("creating a queue sender");
            QueueSender queueSender = session.createSender(ioQueue);

            queueSender.setDeliveryMode(DeliveryMode.NON_PERSISTENT);

            // Create a message
            TextMessage outMessage = session.createTextMessage();

            for (int i = 0; i < 4; i++) {
                if (i % 2 == 0) {
                    System.out.println("Sending red message " + i + " to the queue");
                    outMessage.setText("this is message " + i);
                    outMessage.setStringProperty("colour", "red");
                } else {
                    System.out.println("Sending blue message " + i + " to the queue");
                    outMessage.setText("this is message " + i);
                    outMessage.setStringProperty("colour", "blue");
                }
                queueSender.send(outMessage);
            }

            // start connection
            System.out.println("starting connection");
            connection.start();

            Thread.sleep(5000);       // give the recievers time to print messages

        } catch (JMSException je) {
            System.err.println("caught " + je);
            Exception e = je.getLinkedException();

            if (e != null) {
                System.err.println("subexception: " + e);
            }

        } catch (Exception e) {
            System.err.println("caught " + e);

        } finally {
            if (connection != null) {
                try {
                    System.out.println("closing connection");
                    connection.close();
                } catch (JMSException e) {
                    System.err.println("connection.close() threw " + e);
                }
            }

        }

        System.out.println("Finished ===============================\n");
    }

    //=========================================================================
    /** create an InitialDirContext for retrieving JNDI objects. */
    static InitialDirContext initJNDI(String icf, String url)
        throws JMSException, Exception {
        // the context to be returned from this routine
        InitialDirContext ctx = null;

        // hashtable for the JNDI initialisation properties
        Hashtable environment = new Hashtable();

        if (url == null) {
            // A URL must be supplied, so throw an exception if none is available
            throw new Exception("You must specify the -url providerURL parameter");
        }

        // Firstly, store the name of the 'initial context factory'
        environment.put(Context.INITIAL_CONTEXT_FACTORY, icf);

        // Secondly, specify the location of the JNDI service provider
        environment.put(Context.PROVIDER_URL, url);

        // Finally, ensure that referrals are thrown. This is required
        // as a workaround to a problem encountered with certain comb-
        // inations of JNDI service provider and JNDI classes
        environment.put(Context.REFERRAL, "throw");

        try {
            // Attempt to connect to the JNDI service provider, supply-
            // ing the properties previously supplied
            ctx = new InitialDirContext(environment);

        } catch (Exception e) {
            // return an exception to the caller
            JMSException je = new JMSException("Cannot create JNDI InitialContext!");

            je.setLinkedException(e);
            throw je;
        }

        return ctx;
    }

    //=========================================================================
    /** Get the named QueueConnectionFactory object from the JNDI namespace.
     */
    static QueueConnectionFactory getConnectionFactoryFromJNDI(Context ctx,
        String name)
        throws JMSException {
        // the factory to be returned
        QueueConnectionFactory factory = null;

        System.out.println("Retrieving a QueueConnectionFactory from JNDI");

        try {
            // Attempt to retrieve a QueueConnectionFactory from the
            // JNDI namespace
            factory = (QueueConnectionFactory) ctx.lookup(name);

        } catch (Exception e) {
            // If this has failed, try the operation again, this
            // time prefixing the lookup key with "cn=". Such a
            // prefix is required when using an LDAP provider
            try {
                factory = (QueueConnectionFactory) ctx.lookup("cn=" + name);
            } catch (Exception e2) {
                // After a second failure pass an exception to the caller
                JMSException je =
                    new JMSException("Unable to retrieve the QCF object from JNDI");

                je.setLinkedException(e2);
                throw je;
            }
        }

        return factory;
    }

    //=========================================================================
    /** Get the named Queue object from the JNDI namespace. */
    static Queue getQueueFromJNDI(Context ctx, String name)
        throws JMSException {
        // The Queue to be returned
        Queue ioQueue = null;

        System.out.println("Retrieving a Queue from JNDI");
        try {
            // Attempt to retrieve a Queue from the JNDI namespace
            ioQueue = (Queue) ctx.lookup(name);
        } catch (Exception e) {
            try {
                // If this fails, attempt again with the "cn=" prefix
                ioQueue = (Queue) ctx.lookup("cn=" + name);

            } catch (Exception e2) {
                // After a second failure pass an exception back to the caller
                JMSException je = new JMSException("Unable to retrieve the Queue '" +
                        name + "' from JNDI");

                je.setLinkedException(e2);
                throw je;
            }
        }

        return ioQueue;
    }
}


/*
 * This class provides the message listener.
 */
class PTPSample02MListener implements MessageListener {
    // an id to allow us to distinguish between different instances
    int id;

    PTPSample02MListener(int id) {
        this.id = id;
    }

    /** This method provides the implementation of the MessageListener
     interface. */
    public void onMessage(Message message) {
        String body;

        try {
            System.out.println("in onMessage of listener " + id);

            if (message instanceof TextMessage) {
                // display the message contents
                body = ((TextMessage) message).getText();
                if (body != null) {
                    System.out.println("message contained '" + body + "'");
                } else {
                    System.out.println("error: message contained no body");
                }
                // display the message 'colour' property
                String colour = message.getStringProperty("colour");

                if (colour == null) {
                    System.out.println("the message didn't have a colour property!");
                } else {
                    System.out.println("colour=" + colour);
                }
            } else {
                System.out.println("error: message was not a TextMessage as expected");
                System.out.println(message);
            }

        } catch (JMSException je) {
            System.out.println("onMessage caught " + je);
            Exception e = je.getLinkedException();

            if (e != null) System.out.println("linked exception: " + e);

        } catch (Exception e) {
            System.out.println("onMessage caught " + e);
        }
    }
}

