package examples.midp.exampleapp.msgpump;


/*----------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                  */

/*                                                                            */

/* Licensed Materials - Property of IBM                                       */

/*                                                                            */

/* 5765-E63                                                                   */

/*                                                                            */

/* (C) Copyright IBM Corp. 1999                                               */

/*----------------------------------------------------------------------------*/

import java.util.Calendar;

import examples.midp.exampleapp.messageservice.MQeMessageService;
import examples.midp.exampleapp.messageservice.MQeMessageServiceParameters;
import examples.midp.exampleapp.messageservice.MessageConsumer;

import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeAttribute;
import com.ibm.mqe.MQeCompressor;
import com.ibm.mqe.MQeCryptor;
import com.ibm.mqe.MQeException;
import com.ibm.mqe.MQeKey;


/**
 * This is the basic outline of a message pump model.
 * The model is the part of the system that does everything whereas the controller is
 * the part that decides what needs to be done.
 *
 * <p>The model contains an MQeMessageService, which handles the queue manager, queues and
 * other parts of MQe. It also contains a link to its controller, which is used if a decision
 * needs to be made or if the user eed to be prompted for something.
 *
 * <p>The model part of the system will not be dependant on whether it is using MIDP, Foundation,
 * J2SE or any other versions of java as it should be independant of these things; the
 * controller will handle all java-version specific things.
 * For this reason, when a message is sent and includes a timestamp, since MIDP does not
 * contain the functions necessary to easily obtain a timestamp, this has been done manually
 * within the model - hence the getTimestamp() and padNumber() functions.
 */
abstract public class MsgPumpModel implements Runnable, MessageConsumer {

    public static short  version[] = { 2, 0, 0, 10 };  // MQe version
  
    // The controller of this model
    private MsgPumpController controller;

    // The message service used to deal with the queue manager, queues etc
    private MQeMessageService msgService;

    // Rules file used to define the queue manager's behaviour
    private String qmgrRules = "examples.midp.exampleapp.messageservice.QueueManagerRules";

    // The attribute of a message (e.g. the cryptor / compressor to use)
    // This is used to save processing, since creating a new MQeAttribute with the same
    // attributes every time you want to send a message is a waste
    private MQeAttribute messageAttribute;

    // Name of the local queue manager
    private String localQMname;

    // Default names for the server QM and request queue.
    /** The default name of the server queue manager.*/
    public static final String DEFAULT_SERVER_QM_NAME = "QM_PumpServer";

    /** The default name of the server queue being pumped to. */
    public static final String DEFAULT_SERVER_Q_NAME = "REQUESTQUEUE";

    // Security related things :
    //   statics for the various security levels.
    //   the default security level.
    //   the key used for message based security.
    public static final int NO_SECURITY = 0;
    public static final int MESSAGE_SECURITY = 1;
    public static final int QUEUE_SECURITY = 2;

    /** The default security value.*/
    public static final int DEFAULT_SECURITY = NO_SECURITY;

    /** The key used for encrypting messages with message based security. */
    public static final String SECURITY_KEY = "A very insecure key";

    //   the current security level
    private int securityType = DEFAULT_SECURITY;

    /** The pump's cryptor. */
    public final String cryptor = "com.ibm.mqe.attributes.MQeDESCryptor";

    /** The pump's compressor. */
    public final String compressor = null;

    /** Shows whether the queues are synchronous or not .*/
    public final boolean sync = true;

    /**
     *   Returns whether the security type being used matches the given type.
     *
     * @param security The security type
     * @return boolean - if the security matches
     */
    public boolean isSecurity(int security) {
        return security == securityType;
    }

    /**
     *   Used to initialize a message pump model.
     *
     * @param controller Link to this model's controller
     * @param localQMname Name of local queue manager
     * @return void
     */
    public void initialize(MsgPumpController controller, String localQMname) throws Exception {
        // Remember who your controller is
        this.controller = controller;

        // Note this comes *after* setting the controller
        processStatus("Initializing model for " + localQMname);

        // Remember what your queue manager is called
        this.localQMname = localQMname;

        // Set parameters in message service
        MQeMessageServiceParameters mparms = new MQeMessageServiceParameters();

        mparms.setQMname(localQMname);

        // If queue based security is required, specify a private registry and set the PIN
        if (securityType == QUEUE_SECURITY) {
            mparms.setRegistryTypePrivate();
        }

        getMsgService().setStartupParameters(mparms);
        getMsgService().setMessageConsumer(this);

        // Attempt to start the message service
        // If this fails with an 'MQe.Except_QMgr_NotConfigured' exception
        // then the QM has not yet been created (i.e. there is no registry
        // or queue information located on the disk.)
        // This means that an initial configuration will have to be performed
        // first to set all of this up.
        try {
            getMsgService().open();
        } catch (MQeException e) {
            if (e.code() == MQe.Except_QMgr_NotConfigured) {
                performInitialConfiguration();
            } else {
                throw e;
            }
        }
    }

    /**
     *   Returns the model's message service ( creating a new one if necessary ).
     *
     * @return MQeMessageService - the message service
     */
    protected MQeMessageService getMsgService() {
        if (msgService == null) {
            processStatus("[Model] Creating message service...");
            msgService = new MQeMessageService();
        }
        return msgService;
    }

    /**
     *   Any final actions necessary to finish stopping the model.
     *
     * @return void
     */
    public void finish() throws Exception {
        if (msgService != null) {
            msgService.close();
            msgService = null;
        }

        synchronized (this) {

            /* Just in case anyone is waiting on us */
            notifyAll();
        }
    }

    /**
     *   Sets a property of the pump.
     *
     * @param property The property
     * @param value The value to set it to
     * @return void
     */
    protected void setProperty(String property, String value) {
        if (property.equals("SecurityType")) {
            if (value != null) {
                securityType = (Integer.valueOf(value)).intValue();
            }
        }
    }

    /**
     *   Used to process a message before it is displayed.
     *   <br>Note: this is not used for MQe messages, but simple status or info
     *         messages such as 'Initializing Client'.
     *
     * @param statusMessage The message to be displayed
     * @return void
     */
    public void processStatus(String statusMessage) {
        if (controller != null) {
            controller.processMsg(statusMessage);
        }
    }

    /**
     *   Returns a nicely formatted datetimestamp.
     *   <p>Note - 'nicely' means US format 'hh:mm mm/dd/yy'.
     *
     *   <p>Should only be used to generate a timestamp for a message to be sent.
     *
     * @return String - a datetimestamp
     */
    protected String getTimestamp() {
        Calendar cal = Calendar.getInstance();

        return padNumber(2, cal.get(cal.HOUR))
            + ":"
            + padNumber(2, cal.get(cal.MINUTE))
            + ":"
            + padNumber(2, cal.get(cal.SECOND))
            + " "
            + padNumber(2, cal.get(cal.MONTH) + 1)
            + "/"
            + padNumber(2, cal.get(cal.DAY_OF_MONTH))
            + "/"
            + padNumber(2, cal.get(cal.YEAR));
    }

    /**
     *   Given a number and a size, this function will return the original
     *   number padded with zeroes to fill in the spaces ( as a String ).
     *   <br>i.e. padNumber ( 3, 5 ) -> 005  ( note the order ).
     *
     * @param size The size to pad number out to
     * @param number The number to pad
     * @return String
     */
    protected String padNumber(int size, int number) {
        String out = String.valueOf(number);

        while (out.length() < size) {
            out = "0" + out;
        }
        return out;
    }

    /**
     *   Returns the model's controller.
     *
     * @return MsgPumpController
     */
    protected MsgPumpController getController() {
        return controller;
    }

    /**
     *   Sets the model's controller.
     * @return void
     */
    protected void setController(MsgPumpController controller) {
        this.controller = controller;
    }

    /**
     *   Gets the local queue manager name.
     *
     * @return String
     */
    public String getLocalQMname() {
        return localQMname;
    }

    /**
     *   Gets the rules used to configure the queue manager.
     *
     * @return String
     */
    public String getQMrules() {
        return qmgrRules;
    }

    /**
     *   Returns the messsage attribute or, if none exists, creates a new one.
     *
     * @return MQeAttribute - the message attribute
     */
    public MQeAttribute getMessageAttribute() throws Exception {
        if (messageAttribute == null) {
            createMessageAttribute();
        }
        return messageAttribute;
    }

    /**
     *   Creates a new message attribute to use.
     *
     * @return void
     */
    public void createMessageAttribute() throws Exception {
        MQeKey key = new MQeKey();

        key.setLocalKey(SECURITY_KEY);

        MQeCryptor cryptor = null;
        MQeCompressor compressor = null;

        if (this.cryptor != null) {
            cryptor = (MQeCryptor) MQe.loadObject(this.cryptor);
        }

        if (this.compressor != null) {
            compressor = (MQeCompressor) MQe.loadObject(this.compressor);
        }
        messageAttribute = new MQeAttribute(null, cryptor, compressor);
        messageAttribute.setKey(key);
    }

    /**
     *   Used to initially set up the queue manager, queues and anything
     *   else required by the model ( listeners etc ).
     *   This is only run if there are no registry file located on the system.
     *
     * @return void
     */
    abstract protected void performInitialConfiguration() throws Exception;

    /**
     *   Called when something wants the model to stop.
     *   Returns whether the caller should wait on the model until it is done or not.
     *
     * @return boolean
     */
    abstract public void stop();

    /**
     *   Pauses the model.
     *
     * @return void
     */
    abstract public void pause();

    /**
     *   Resumes the model.
     *
     * @return void
     */
    abstract public void resume();
}
