package examples.midp.exampleapp.msgpump;


/*----------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                  */

/*                                                                            */

/* Licensed Materials - Property of IBM                                       */

/*                                                                            */

/* 5765-E63                                                                   */

/*                                                                            */

/* (C) Copyright IBM Corp. 1999                                               */

/*----------------------------------------------------------------------------*/


import java.io.File;


/**
 * This class acts as the message pump controller for clients on the J2SE
 * java platform. It contains a main() function and so can be run from the
 * commandline.
 *
 * <p>The main uses of the controller are to get a name for the queue manager
 * and control the pump to suit the user's requests.
 *
 * <p>An input thread is used to collect user input. This is handled by an InputThread.
 *
 * <p>The following commandline arguements are accepted:
 * <br>--security [0 - 2] : Allows you to set the security level of the server.
 *                          0 = no security.
 *                          1 = message based security.
 *                          2 = queue based security.
 * <br>--serverip [xxx.xxx.xxx.xxx] : The ip of the server (localhost is default)
 * <br>--serverport [n] : The port to send messages to (default 8082)
 * <br>--servername [name] : The name of the server to send to
 * <br>--serverqueuename [queue] : The name of the queue to send to
 *
 * <p>e.g. java mqeexampleapp.msgpump.NormalClient --security 1 --serverip 10.0.0.101
 * will send to 10.0.0.101 using message based security
 */
public class NormalClient implements MsgPumpController {

    public static short  version[] = { 2, 0, 0, 10 };  // MQe version  

    // The pump model associated with the controller
    private static MsgPumpModel msgPump = null;

    // The thread used for the model
    private static Thread thread = null;

    // the input thread used to capture user input
    private static InputThread input;

    // Messages used to prompt the user


    /** Initial thing that is displayed to the user after initialization is complete.*/
    public static final String starttext = "Type 'help' for help on available commands";

    /** String used to prompt the user for input. */
    public static final String prompttext = "Client:\\>";

    /**
     *     The function called when the class is run.
     *     This initializes and starts a new controller.
     */
    public static void main(String[] args) {
        try {
            NormalClient controller = new NormalClient();

            if (args.length != 0) {
                controller.setPumpProperties(args);
            }

            controller.initialize();
            controller.start();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     *     Called when the class is started with commandline arguments.
     *     This function will set the appropriate properties in the pump
     *
     * @param args The commandline arguments
     * @return void
     */
    protected void setPumpProperties(String[] args) throws Exception {
        while (args.length != 0) {
            System.out.println(args.length);
            if (args[0].equals("--security") && args.length > 1) {
                getModel().setProperty("Security", args[1]);
                String[] temp = new String[args.length - 2];

                System.arraycopy(args, 2, temp, 0, args.length - 2);
                args = temp;
            } else if (args[0].equals("--serverip") && args.length > 1) {
                getModel().setProperty("ServerIP", args[1]);
                String[] temp = new String[args.length - 2];

                System.arraycopy(args, 2, temp, 0, args.length - 2);
                args = temp;
            } else if (args[0].equals("--serverport") && args.length > 1) {
                getModel().setProperty("Pump_ServerPort", args[1]);
                String[] temp = new String[args.length - 2];

                System.arraycopy(args, 2, temp, 0, args.length - 2);
                args = temp;
            } else if (args[0].equals("--servername") && args.length > 1) {
                getModel().setProperty("ServerQMname", args[1]);
                String[] temp = new String[args.length - 2];

                System.arraycopy(args, 2, temp, 0, args.length - 2);
                args = temp;
            } else if (args[0].equals("--serverqueuename") && args.length > 1) {
                getModel().setProperty("ServerQname", args[1]);
                String[] temp = new String[args.length - 2];

                System.arraycopy(args, 2, temp, 0, args.length - 2);
                args = temp;
            } else {
                throw new Exception("Invalid commandline arguments");
            }
        }
    }

    /**
     *   Purges the queue manager from the system.
     *   On J2SE, this is done by wiping the whole tree structure.
     *
     * @return void
     */
    public void deleteQueueManager() {
        // first ensure the queue manager is closed
        try {
            getModel().getMsgService().close();
        } catch (Exception e) {}
        // Now delete all of its directories
        File dir = new File(getModel().getMsgService().DEFAULT_BASEDIRNAME);

        deleteDir(dir);
    }

    /**
     *     Equivilent to dos deltree or linux rm -r .
     *     <br>Deletes a directory and all subdirectories and files.
     *
     * @param path the directory to delete
     * @return void
     */
    protected void deleteDir(File path) {
        File[] files = path.listFiles();

        for (int i = 0; i < files.length; ++i) {
            if (files[i].isDirectory()) {
                // Recurse if it is a directory
                deleteDir(files[i]);
            }
            try {
                // delete each entry
                files[i].delete();
            } catch (Exception e) { // don't worry errors
            }
        }
        // now delete the base dir
        path.delete();
    }

    /**
     *   Starts the input thread.
     *
     * @return void
     */
    protected void startInput() {
        input = getNewInputThread();
        Thread inThread = new Thread(input);

        inThread.start();
    }

    /**
     *     Returns the model being used by the controller.
     *
     * @return MsgPumpModel - the model
     */
    public MsgPumpModel getModel() {
        if (msgPump == null) {
            msgPump = getNewModel();
        }
        return msgPump;
    }

    /**
     *     Returns a new inputthread.
     * @return InputThread - a new inputthread
     */
    protected InputThread getNewInputThread() {
        return new InputThread(this, starttext, prompttext);
    }

    /**
     *     Processes a status or error message.
     *     In this simple case, this involves merely displaying it
     *
     * @param msg The mesage to display
     * @return void
     */
    public void processMsg(String msg) {
        displayMsg(msg);
    }

    /**
     *     The QM name is generated by using the computer's hostname.
     *     If all else fails, a default is used.
     * @see MsgPumpController#getQMname()
     */
    public String getQMname() {
        try {
            java.net.InetAddress ia = java.net.InetAddress.getLocalHost();
            String host = ia.getHostName();
            int i = host.indexOf('.');

            if (i > 1) {

                /* returned sparkly.hursley.ibm.com */
                return "QM_" + host.substring(0, i - 1);
            } else {
                return "QM_" + host;
            }
        } catch (Exception e) {

            /* If all else fails, return the default QM name */
            displayMsg("*Client Controller* could not get Local Host");
            return "QM_DefaultClient";
        }
    }

    /**
     *     Starts the controller.
     *
     *     <p>This involves starting the model in a new thread.
     *
     * @return void
     */
    public void start() throws Exception {
        processMsg("[Controller] Starting model...");
        startInput();
        Thread thread = new Thread(getModel());

        thread.start();
    }

    /**
     *   Sets the prompt and start texts before initializing.
     *
     * @see MsgPumpController#initialize()
     */
    public void initialize() throws Exception {
        String localQMname = getQMname();

        processMsg("[Controller] Initializing " + localQMname);
        getModel().initialize(this, localQMname);
    }

    /**
     *    Deals with user input.
     *
     * @param input The user input
     * @return void
     */
    public void processInput(String input) {
        if (input.equals("help")) {
            displayMsg("The following commands can be used on this client:");
            displayMsg("pause:         pause the pump");
            displayMsg("resume:        resume pumping");
            displayMsg("setwaittime n: change wait time between messages (in milliseconds)");
            displayMsg("status:        gets the status of the client");
            displayMsg("quit:          stops the pump and quits");
        } else if (input.equals("quit")) {
            getModel().stop();
        } else if (input.equals("pause")) {
            getModel().pause();
        } else if (input.equals("resume")) {
            getModel().resume();
        } else if (input.startsWith("setwaittime ")) {

            /* Retrieve the number from 'setwaittime 1000' for example */
            String i = input.substring(12);

            try {
                int n = (Integer.valueOf(i)).intValue();

                ((MsgPumpModelClient) getModel()).setWaitTime(n);
            } catch (NumberFormatException e) {

                /* Something went wrong - probably wasnt a number */
                displayMsg("Invalid use of setwaittime command");
                displayMsg("e.g. setwaittime 3000");
            }
        } else if (input.equals("status")) {
            displayMsg("");
            displayMsg("-----===== Status Report =====-----");
            displayMsg("QM Name:        " + getModel().getLocalQMname());
            displayMsg("Pump rate:      " + ((MsgPumpModelClient) getModel()).getWaitTime() + "ms");
            displayMsg("Server Address: " + ((MsgPumpModelClient) getModel()).getServerAddress());
            displayMsg("Server QM Name: " + ((MsgPumpModelClient) getModel()).getServerQMname());
            displayMsg("Server Q Name:  " + ((MsgPumpModelClient) getModel()).getServerQname());
            displayMsg("");
        } else {
            displayMsg("Unknown command '" + input + "'");
        }
    }

    /**
     * @see MsgPumpController#displayMsg(String).
     */
    public void displayMsg(String msg) {
        System.out.println(msg);
    }

    /**
     * Returns a new model.
     */
    public MsgPumpModel getNewModel() {
        return new MsgPumpModelClient();
    }

    /**
     * Gets the msgPump.
     * @return Returns a MsgPumpModel
     */
    public MsgPumpModel getMsgPump() {
        return msgPump;
    }
}
