package examples.midp.exampleapp.textapp;



/*----------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                  */

/*                                                                            */

/* Licensed Materials - Property of IBM                                       */

/*                                                                            */

/* 5765-E63                                                                   */

/*                                                                            */

/* (C) Copyright IBM Corp. 1999                                               */

/*----------------------------------------------------------------------------*/


import examples.midp.exampleapp.messageservice.MQeMessageService;
import examples.midp.exampleapp.messageservice.MQeMessageServiceParameters;
import examples.midp.exampleapp.messageservice.MessageConsumer;


import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeException;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeMsgObject;
import com.ibm.mqe.mqemqmessage.MQeMQMsgObject;



/**
 * The aim of the text app is to have 3 queue managers:
 *
 *  <br> The Gateway server      - a server that will be sent messages.
 *  <br> The Registration server - a server that new clients have to register with
 *                                 before they can send messages to the gateway.
 *  <br> The Client              - a client that send messages to the gateway.
 *
 * <p>The aim of this example is to show how a registration server can be used to
 * create the necessary resources for a new client. This approach gives several
 * advantages to larger networks of MQe devices.
 *
 * <p>This class implements a basic model for a text app. Functions are available
 * to create listeners, define queues and sends messages for both clients and servers.
 *
 * <p>The message service is used to connunicate with the underlying MQe structure to
 * simplify the design of the code.
 */
abstract public class AppModel implements MessageConsumer {

    public static short  version[] = { 2, 0, 0, 10 };  // MQe version
  

    // Default queue manager names for servers and clients
    /** Default name for the registration QM.*/
    public static final String DEFAULT_REGISTRATION_QM = "QM_AppRegistration";

    /** Default name for the gateway QM. */
    public static final String DEFAULT_GATEWAY_QM = "QM_AppGateway";

    /** Default name for the client QM.*/
    public static final String DEFAULT_CLIENT_QM = "QM_AppClient";


    // Default listener ports for servers
    /** Default port for listeners. */
    public static final int DEFAULT_LISTENER_PORT = 8082;

    /** Default port for listeners on the registration QM.
     * <p> This is to allow the gateway and registration QMs on the same machine.
     */
    public static final int DEFAULT_REGISTRATION_LISTENER_PORT = 8083;


    // Default locations for servers.
    /** Default location for the gatewar server. */
    public static final String DEFAULT_GATEWAY = "Network://127.0.0.1:" + DEFAULT_LISTENER_PORT;

    /** Default location for the registration server. */
    public static final String DEFAULT_REGISTRATION =
        "Network://9.20.25.160:" + DEFAULT_REGISTRATION_LISTENER_PORT;


    /* Information on where the gateway and registration servers are located and
     * how they are to be accessed.
     * <p>Command and Options are used when creating a channel to the server to specify
     * options and parameters for the initialization.
     */
    public static final String GatewayCommand = null;
    public static final String GatewayOptions = null;
    public static final String RegistrationCommand = null;
    public static final String RegistrationOptions = null;


    // Default names for queues
    /** Default name for a request queue. */
    public static final String DEFAULT_REQUEST_Q = "REQUESTQUEUE";

    /** Default name for a reply queue.*/
    public static final String DEFAULT_REPLY_Q = "REPLYQUEUE";

    /** Default name for the registration queue. */
    public static final String DEFAULT_REGISTRATION_Q = "RegistrationQueue";

    /** Default name for the registration reply queue. */
    public static final String DEFAULT_REGISTRATION_REPLY_Q = "RegistrationReplyQueue";

    /** Default name for a home server queue.*/
    public static final String DEFAULT_HOMESERVER_Q = "SFQ1";


    // Rules file that will be used to define how trigger transmission should occur etc
    /** Default rules file to use. */
    public static final String DEFAULT_RULES = "examples.midp.exampleapp.messageservice.QueueManagerRules";


    /*
     * List of queues and connections used by the model.
     * These are chosen by chooseRemoteQueues(), chooseLocalQueues() etc
     * Example lists are laid out as follows:
     *
     * # localQueues
     *   new String[] { RequestQ, "async", ReplyQ, "sync" };
     *
     *   creates an asynchronous RequestQ and a synchronous ReplyQ
     *
     * # remoteQueues
     *   new String[] { DefaultGatewayQM, RequestQ, "async",
     *                  DefaultRegistrationQM, RegistrationQ, "sync"};
     *
     *   creates an asynchronous remote queue to RequestQ on DefaultGateway
     *   and a synchronous remote queue to RegistrationQ on DefaultRegistrationQM
     *
     * Queues and connections are created by initializeLocalQueues/Connections and
     * local queues are monitored by monitorLocalQueues()
     *
     * etc
     */
    private String localQueues[];
    private String remoteQueues[];
    private String connections[];


    // The name of the queue manager and its listener (unused by clients) as well as its
    // message service (used as a frontend to MQe) and its controller
    private String localQMname;
    private String listener;
    private MQeMessageService msgService;
    private AppController controller;


    /**
     *   Starts the model running (initialize should have been run first).
     *   This involves monitoring all local queues.
     *
     * @return void
     */
    public void run() throws Exception {
        monitorLocalQueues();
    }


    /**
     *   Initializes the model and all necessary components.
     *   Needs to be given the name of the queue manager to initialize.
     *
     * @param controller The model's controller
     * @param localQMname The name for local queue manager
     * @return void
     */
    public void initialize(AppController controller, String localQMname) throws Exception {
        processStatus("Initializing " + localQMname);


        this.controller = controller;
        this.localQMname = localQMname;


        // Bit of paranoia - the there's a message service lurking about for no reason,
        // close it before starting the new one
        if (msgService != null) {
            msgService.close();
        }
        msgService = new MQeMessageService();


        // Pick the queues and connections you want
        localQueues = chooseLocalQueues();
        remoteQueues = chooseRemoteQueues();
        connections = chooseConnections();


        try {
            // Create parameters for the new queue manager
            // set the name of the QM and listener port
            // tell the message service that we're going to be the message consumer and
            // start the queue manager going
            MQeMessageServiceParameters mparms = new MQeMessageServiceParameters();

            mparms.setQMname(localQMname);
            mparms.setListenerPort(getListenerPort());
            msgService.setStartupParameters(mparms);
            msgService.setMessageConsumer(this);
            msgService.open();
        } catch (MQeException e) {
            // if the queue manager does not exist create it together
            // and perform any configuration required
            if (e.code() == MQe.Except_QMgr_NotConfigured) {
                initialConfiguration();
            } else {
                throw e;
            }
        }
    }


    /**
     *   Called if the queue manager has not been started up previously.
     *   This will create all the necessary queues, connections, listeners etc for the
     *   queue manager to function. This will be saved in the registry for the next time
     *   the queue manager is started.
     *
     * @return void
     */
    protected void initialConfiguration() throws Exception {
        // Create and start the queue manager
        // Create all the queues and connection required
        msgService.createQueueManager();
        msgService.open();


        initializeConnections();
        initializeLocalQueues();
        initializeRemoteQueues();
        initializeOtherQueues();


        MQeFields qmParms = new MQeFields();

        qmParms.putAscii(com.ibm.mqe.administration.MQeQueueManagerAdminMsg.QMgr_Rules, DEFAULT_RULES);
        msgService.updateQueueManager(localQMname, qmParms);


        processStatus("Queue Manager has been updated");
    }


    /**
     *   Generates a name for the listener from the queue manager and creates it.
     *   This is done by taking the name of the queue manager and attaching '_Listener'
     *   to the end.
     *
     * @return void
     */
    protected void createListener() throws Exception {
        listener = localQMname + "_Listener";
        msgService.createListener(listener);
    }


    /** Return a list of connections used by the model. */
    protected String[] chooseConnections() {
        return null;
    }

    /** Return a list of local queues used by the model. */
    protected String[] chooseLocalQueues() {
        return new String[] { DEFAULT_REQUEST_Q, "async", DEFAULT_REPLY_Q, "async" };
    }

    /** Return a list of remote queues used by the model. */
    protected String[] chooseRemoteQueues() {
        return null;
    }


    /**
     *   This creates all the connections specified in connections[].
     *
     * @return void
     */
    protected void initializeConnections() throws Exception {
        if (connections == null) {
            processStatus("No connections have been created");
        } else {
            for (int i = 0; i < connections.length; i += 4) {
                msgService.createConnection(
                    localQMname,
                    connections[i],
                    connections[i + 1],
                    connections[i + 2],
                    connections[i + 3]);
            }
        }
    }


    /**
     *   This creates all the local queues specified in localQueues[].
     *
     * @return void
     */
    protected void initializeLocalQueues() throws Exception {
        if (localQueues == null) {
            processStatus("No local queues have been created");
        } else {
            for (int i = 0; i < localQueues.length; i += 2) {
                boolean mode = localQueues[i + 1].equals("sync");

                msgService.createLocalQueue(localQMname, localQueues[i], mode);
            }
        }
    }


    /**
     *   This creates all the remote queues specified in remoteQueues[].
     *
     * @return void
     */
    protected void initializeRemoteQueues() throws Exception {
        if (remoteQueues == null) {
            processStatus("No remote queues have been created");
        } else {
            for (int i = 0; i < remoteQueues.length; i += 3) {
                boolean mode = remoteQueues[i + 2].equals("sync");

                msgService.createRemoteQueue(localQMname, remoteQueues[i], remoteQueues[i + 1], mode);
            }
        }
    }


    /**
     *   Will start monitoring of all local queues as specified in localQueues[].
     *
     * @return void
     */
    protected void monitorLocalQueues() throws Exception {
        if (localQueues == null) {
            processStatus("No local queues to monitor");
        } else {
            for (int i = 0; i < localQueues.length; i += 2) {
                msgService.monitorQueue(localQMname, localQueues[i]);
            }
        }
    }


    /**
     *   Clears all local queues as defined by localQueues[].
     *
     * @return void
     */
    protected void clearLocalQueues() {
        if (localQueues == null) {
            processStatus("No local queues to clear");
        } else {
            if (msgService != null) {
                for (int i = 0; i < localQueues.length; ++i) {
                    try {
                        msgService.emptyQueue(localQMname, localQueues[i]);
                    } catch (Exception e) {// These aren't important - ignore them
                    }
                }
            }
        }
    }


    /**
     *   Displays a message using the controller or if no controller is set, prints it to
     *   System.out.
     *
     * @param msg The message to be displayed
     * @return void
     */
    public void processStatus(String msg) {
        if (controller != null) {
            controller.displayStatus(msg);
        } else {
            System.out.println("*null controller*" + msg);
        }
    }


    /**
     *   Used to register a queue manager with the registration server.
     *   Not necessarily only clients that will want to register with reg server
     *   so this is located in the base abstract model rather than the client.
     *
     *   <p>Registering involves simply sending a message to the registration server and waiting
     *   for a reply. The reply will contain a success bool indicating whether the registration
     *   was a success or not.
     *
     * @param QMname The name of the queue manager
     * @param Qname The name of the queue
     * @return void
     */
    public void registerWithCentral(String QMname, String Qname) throws Exception {
        processStatus("Registering queue manager");


        // Create and send the registration request message
        MQeMsgObject regMsg = new MQeMsgObject();

        regMsg.putAscii("QMgrName", localQMname);
        regMsg.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);
        regMsg.putAscii(MQe.Msg_ReplyToQ, QMname);
        regMsg.putAscii(MQe.Msg_ReplyToQMgr, Qname);


        getMsgService().sendMessage(QMname, Qname, regMsg);


        // Create a match field so that we can find the reply message that corresponds to the request.
        MQeFields msgTest = new MQeFields();

        msgTest.putArrayOfByte(MQe.Msg_CorrelID, MQe.asciiToByte(localQMname));


        // Wait for a reply
        MQeMsgObject respMsg =
            getMsgService().waitForReply(QMname, Qname, msgTest);


        if (respMsg != null && respMsg.getBoolean("Success") != true) {
            // If something goes wrong, throw the reason why
            throw new Exception("Failed to register: " + respMsg.getUnicode("Reason"));
        }
        processStatus("Registration successful");
    }


    /**
     *   Stops the model and clears any local queues of messages befor closing the message
     *   service.
     *
     * @return void
     */
    public void stop() throws Exception {
        clearLocalQueues();
        if (msgService != null) {
            msgService.close();
            msgService = null;
        }
    }


    /**
     *   Gets the localQMname.
     *
     * @return String - name of the local queue manager
     */
    public String getLocalQMname() {
        return localQMname;
    }


    /**
     *   Sets the localQMname.
     *
     * @param localQMname The localQMname to set
     * @return void
     */
    public void setLocalQMname(String localQMname) {
        this.localQMname = localQMname;
    }


    /**
     *   Gets the msgService.
     *
     * @return MQeMessageService - the message service
     */
    public MQeMessageService getMsgService() {
        return msgService;
    }


    /**
     *   Sets the msgService.
     *
     * @param msgService The msgService to set
     * @return void
     */
    public void setMsgService(MQeMessageService msgService) {
        this.msgService = msgService;
    }


    /**
     *   Gets the controller.
     *
     * @return AppController - the controller
     */
    public AppController getController() {
        return controller;
    }


    /**
     *   Sends a message to the default queue on the default queue manager.
     *
     * @param message The message to send
     * @return void
     */
    public void sendNewMessage(String message) throws Exception {
        // Create the message, setting the data, the type to request, and the
        // QM and Q to send the reply message, then send the message
        MQeMQMsgObject msg = new MQeMQMsgObject();

        msg.setData(MQe.asciiToByte(message));
        msg.setMessageType(MQe.Msg_Style_Request);
        msg.setReplyToQueueManagerName(getLocalQMname());
        msg.setReplyToQueueName(DEFAULT_REPLY_Q);
        getMsgService().sendMessage(DEFAULT_GATEWAY_QM, DEFAULT_REQUEST_Q, msg);


        processStatus("Message '" + message + "' sent");
    }

    /**
     *   Produces a status report for the queue manager.
     *   This lists The type of the model (client/server), the queues and connections and
     *   listener port.
     *
     * @return String - the status report
     */
    protected String statusReport() {
        String report = "";

        report += "-= Status Report for " + localQMname + " =-\n";
        if (isServer()) {
            report += "\nType: Server\n";
            report += "Listener port: " + getListenerPort() + "\n";
        } else {
            report += "\nType: Client\n";
        }


        report += "\n__Local Queues__\n";


        if (localQueues == null) {
            report += "No local queues\n\n";
        } else {
            for (int i = 0; i < localQueues.length; i += 2) {
                report += "[" + localQueues[i + 1] + "] " + localQueues[i] + "\n";
            }
        }


        report += "\n__Remote Queues__\n";


        if (remoteQueues == null) {
            report += "No remote queues\n";
        } else {
            for (int i = 0; i < remoteQueues.length; i += 3) {
                report += "[" + remoteQueues[i + 2] + "] " + remoteQueues[i + 1] + " (" + remoteQueues[i] + ")\n";
            }
        }


        report += "\n__Connections__\n";


        if (connections == null) {
            report += "No connections\n";
        } else {
            for (int i = 0; i < connections.length; i += 4) {
                report += connections[i] + " [" + connections[i + 1] + "]\n";
            }
        }


        return report;
    }


    /**
     *   Used to check whether the model is a client or server.
     *   This determines whether a listener should be created and whether the model should
     *   register with the central server.
     *
     *   <p>This design has problem with expandability if, in the future, a server that
     *   registers with the central server is required.
     *
     * @return boolean - true if server, false if client
     */
    abstract protected boolean isServer();


    /**
     *   This creates all less common queues that a queue manager may need
     *   ( home server queues, store and forward queues etc).
     *
     * @return void
     */
    abstract protected void initializeOtherQueues() throws Exception;


    /**
     *   Returns the port that the queue manager listens on.
     *   (unimportant to clients, as they don't have listeners).
     *
     * @return int - port to listen on
     */
    abstract protected int getListenerPort();
}
