package examples.midp.exampleapp.textapp;



/*----------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                  */

/*                                                                            */

/* Licensed Materials - Property of IBM                                       */

/*                                                                            */

/* 5765-E63                                                                   */

/*                                                                            */

/* (C) Copyright IBM Corp. 1999                                               */

/*----------------------------------------------------------------------------*/


import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.InetAddress;



/**
 * The class acts as the J2SE controller for the text app client and should not
 * be used on MIDP devices.
 *
 * <p>Once the model has been initialized and registered with the reg server,
 * the client will prompt the user for input.
 * If the user enters one of the keywords used to perform a function such as 'help'
 * or 'status' then the relevant command will be performed, otherwise, the text
 * will be sent as a message to the gateway server.
 */
public class Client implements AppController {

    public static short  version[] = { 2, 0, 0, 10 };  // MQe version


    /** The first thing to be displayed to the user after initialization */
    public static final String initialPromptMessage = "Use the command 'help' for help";


    // the model that will be used by the controller
    private AppModel model;


    /**
     *   This function is invoked when this class is run from the commandline.
     *   A new controller is created and started to manage the model.
     */
    public static void main(String[] args) {
        Client controller = new Client();

        try {
            controller.initialize();
            controller.run();
        } catch (Exception e) {
            controller.displayStatus("App failed");
            e.printStackTrace();
        }
    }


    /**
     *   This initializes the controller, which generally involves just initializing the model.
     *
     * @return void
     */
    protected void initialize() throws Exception {
        getModel().initialize(this, getQMname());
    }


    /**
     *   Returns the controller's model.
     *   If no model exists, a new one is created and returned
     *
     * @return AppModel - the model
     */
    protected AppModel getModel() {
        if (model == null) {
            model = getNewModel();
        }
        return model;
    }


    /**
     *   Stops the controller and all constituent parts.
     *
     * @return void
     */
    public void stop() throws Exception {
        model.stop();
    }


    /**
     *   Called to start the controller running.
     *   This will typically be called after the controller has been initialized
     *
     * @return void
     */
    public void run() throws Exception {
        getModel().run();
        displayStatus(initialPromptMessage);
        processInput(getInput());
    }


    /**
     *   Displays the help screen to the user.
     *
     * @return void
     */
    public void displayHelp() {
        displayStatus("-=Commands=-");
        displayStatus("help :  Displays this dialog");
        displayStatus("status: Displays a status report");
        displayStatus("quit :  Quits the app");
    }


    /**
     *   This class handles user input.
     *   If the user enters anything that doesnt match a command, it is
     *   sent to the gateway as a message.
     *
     * @param input What the user input
     * @return void
     */
    public void processInput(String input) {
        if (input.equals("help")) {
            displayHelp();
        } else if (input.equals("status")) {
            displayStatus(model.statusReport());
        } else if (input.equals("quit")) {
            try {
                stop();
            } catch (Exception e) {}
            System.exit(0);
        } else {
            // unknown command - send as mesasge to gateway
            try {
                ((AppModelClient) getModel()).sendNewMessage(input);
            } catch (Exception e) {
                displayStatus("Failed to send message: " + e);
            }
        }
        processInput(getInput());
    }


    /**
     *   Obtains input from the user.
     *
     * @return String - user input
     */
    protected String getInput() {
        return promptFor("-->", null);
    }


    /**
     *   returns a new model.
     *   This is used so that classes that extend this class can simply override
     *   this function rather than getModel().
     *
     * @return AppModel
     */
    protected AppModel getNewModel() {
        return new AppModelClient();
    }


    /**
     * @see AppController#getQMname()
     */
    public String getQMname() {
        try {
            java.net.InetAddress ia = java.net.InetAddress.getLocalHost();
            String host = ia.getHostName();
            int i = host.indexOf('.');

            if (i > 1) {

                /* returned sparkly.hursley.ibm.com for example */
                return "QM_" + host.substring(0, i - 1) + "_App";
            } else {
                return "QM_" + host + "_App";
            }
        } catch (Exception e) {

            /* If all else fails, return the default QM name */
            displayStatus("Could not get Local Host - using default name");
            return AppModel.DEFAULT_CLIENT_QM;
        }
    }


    /**
     * @see AppController#displayStatus(String)
     */
    public void displayStatus(String msg) {
        System.out.println(msg);
    }


    /**
     *   Used to prompt the user for something.
     *
     * @param prompt What to prompt the user for
     * @param stringDefault The default to display
     * @return String - the reply
     */
    protected String promptFor(String prompt, String stringDefault) {
        displayStatus(prompt);


        BufferedReader rdr = new BufferedReader(new InputStreamReader(System.in));
        String input = null;

        try {
            input = rdr.readLine();
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
        return input;
    }
}
