package examples.midp.exampleapp.textapp;


/*----------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                  */

/*                                                                            */

/* Licensed Materials - Property of IBM                                       */

/*                                                                            */

/* 5765-E63                                                                   */

/*                                                                            */

/* (C) Copyright IBM Corp. 1999                                               */

/*----------------------------------------------------------------------------*/


import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.rms.RecordStore;


/**
 * This is the MIDP client controller for the text app. It is designed to work as
 * a MIDlet and cannot be run from the commandline (use Client.java for that).
 *
 * <p>The client is required to register with the registration server before it can
 * send messages to the gateway. This is done by the registerWithCentral() function
 * located in the AppModel class.
 *
 * <p>The MIDP controller works as follows:
 *
 * <p>If the registry of a previous queue manager is still present in the record store,
 * load the queue manager's details from that and start running without bothering
 * to register (since the client must have registered before for the registry to
 * be there).
 * <br>If no signs of a previous QM are visible, the user is prompted for a name for the
 * new queue manager. Once they have chosen a name, the client registers with the
 * reg server.
 *
 * <p>The following jad properties are available:
 *
 *      <pre>
 *      App_Registration_ServerIP
 *        The IP address of the registration server
 *
 *      App_Gateway_ServerIP
 *        The IP address of the gateway server
 *      </pre>
 */
public class MidpClient extends MIDlet implements AppController, CommandListener {

    public static short  version[] = { 2, 0, 0, 10 };  // MQe version

    // Commands / buttons used by the view. These can go on any screen
    private static final Command ok = new Command("Ok", Command.OK, 0);
    private static final Command options = new Command("Options", Command.SCREEN, 0);
    private static final Command status = new Command("Status", Command.SCREEN, 0);
    private static final Command send = new Command("Send", Command.SCREEN, 0);

    // The various states that the textapp can be in
    private static final int INITIALIZING = 0;
    private static final int WAITINGFORQMNAME = 1;
    private static final int WAITINGFORINPUT = 2;
    private static final int SHOWINGSTATUS = 3;

    // the model that will be used by the controller
    private AppModelClient model;

    // Specifies the current state of the app
    private int mode;

    /*
     * Various screens and components used by the view:
     *  prompt:        used to prompt the user for something
     *  statusScreen:  displays a status report to the user
     *  initialScreen: shows startup info to the user so they can view progress
     *  mainScreen:    the main window displayed to the user
     *  message:       allows the user to enter a message to send to the gateway
     *  statusbar:     located on the main screen and used to give info to the user
     */
    private TextField prompt;
    private TextBox statusScreen;
    private TextBox initialScreen;
    private Form mainScreen;
    private TextField message;
    private StringItem statusbar;

    // The display is used to display things on the screen.
    private Display display;

    // The command buttons attached to each screen
    private Command mainScreenButtons[] = { send, status };
    private Command statusScreenButtons[] = { ok };

    /**
     * See javax.microedition.midlet.MIDlet#startApp for more details.
     */
    public void startApp() throws MIDletStateChangeException {
        display = Display.getDisplay(this);
        start();
    }

    /**
     *   Attaches buttons to a given screen.
     *
     * @param screen The screen to attach the buttons to
     * @param commands An array of buttons to attach to the screen
     * @return Displayable - the given windows with the buttons attached
     */
    protected void attachButtons(Displayable screen, Command[] commands) {
        for (int i = 0; i < commands.length; ++i) {
            screen.addCommand(commands[i]);
        }
    }

    protected void setPumpProperties() {
        model.setProperty("RegistrationIP", getAppProperty("App_Registration_ServerIP"));
        model.setProperty("GatewayIP", getAppProperty("App_Gateway_ServerIP"));
    }

    /**
     *   This is the first thing run when the midlet is started.
     *   It will attempt to get the name of a previous queue manger from the
     *   system and if it fails, the user will be prompted to enter one.
     *
     *   <p>When the user hits the OK button, commandAction() is notified and
     *   will continue the initialization.
     *
     *   <p>If a previous queue manager did exist, the name is retrieved from
     *   the record store and initialization is started.
     *
     * @return void
     */
    protected void start() {
        String localQMname = getQMname();

        if (localQMname == null) {
            mode = WAITINGFORQMNAME;
            promptFor("Enter a name for the QM", "QM_MidpAppClient");
        } else {
            initialize(localQMname);
        }
    }

    /**
     *   Initializes the main screen - this involves creating a new form and adding the
     *   2 components:
     *
     *     <br>message   - a text field so the user can enter a message to send.
     *     <br>statusbar - to give status information to the user.
     *
     *   <p>Buttons are also attached to the form.
     *
     * @return void
     */
    protected void initializeMainScreen() {
        mainScreen = new Form("Text App");
        message = new TextField("== Send Message ==", "Test Message", 100, TextField.ANY);
        statusbar = new StringItem("== Status Bar ==", "");
        mainScreen.append(message);
        mainScreen.append(statusbar);
        attachButtons(mainScreen, mainScreenButtons);
    }

    /**
     *   Initializes the view.
     *   This is only called once a name for the QM has been established.
     *
     * @param localQMname The name of the queue manager
     * @return void
     */
    protected void initialize(String localQMname) {
        mode = INITIALIZING;
        initialScreen = new TextBox("Initializing", "", 100, TextField.ANY);
        changeDisplay(initialScreen);
        try {
            model = new AppModelClient();
            setPumpProperties();
            model.initialize(this, localQMname);
            mode = WAITINGFORINPUT;
            initializeMainScreen();
            mainScreen.setCommandListener(this);
            changeDisplay(mainScreen);
            model.run();
        } catch (Exception e) {
            displayStatus("[Failed to initialize app] " + e.getMessage());
            e.printStackTrace();
            deleteQueueManager();
            try {
                destroyApp(true);
            } catch (Exception e2) {}
        }
    }

    /**
     *   Completely removes the queue manager from the system. This involves
     *   indiscriminately purging all record stores that the MIDlet / MIDlet
     *   suite has access to.
     *
     *   <p>This will of course mean that if you have both the textapp and the
     *   message pump in the same MIDlet suite, wiping one queue manager will
     *   also remove the other.
     *
     * @return void
     */
    public void deleteQueueManager() {
        try {
            model.getMsgService().close();
        } catch (Exception e) {}
        // Get list of all recordstores in suite and delete
        String[] qs = javax.microedition.rms.RecordStore.listRecordStores();

        if (qs != null) {
            for (int i = 0; i < qs.length; i++) {
                try {
                    javax.microedition.rms.RecordStore.deleteRecordStore(qs[i]);
                } catch (Exception e) {}
            }
        }
    }

    /**
     *   Stops the controller and all constituent parts - in this case, the model.
     *
     * @return void
     */
    public void stop() throws Exception {
        model.stop();
    }

    /**
     * There's nothing to pause.
     * <p>See javax.microedition.midlet.MIDlet#pauseApp for more details.
     */
    public void pauseApp() {}

    /**
     * See javax.microedition.midlet.MIDlet#destroyApp for more details.
     */
    public void destroyApp(boolean arg0) throws MIDletStateChangeException {
        try {
            stop();
        } catch (Exception e) {
            System.out.println("Failure to shutdown app");
            e.printStackTrace();
        }
    }

    /**
     *   Due to the way MIDP works, the user will hit the ok button on the screen at
     *   which point commandAction will be called and initialization will continue.
     *
     * @param title The title of the prompt
     * @param stringDefault The default string to display
     * @return void
     */
    public void promptFor(String title, String stringDefault) {
        Form promptForm = new Form("Text app");

        prompt = new TextField(title, stringDefault, 100, TextField.ANY);
        promptForm.append(prompt);
        promptForm.addCommand(ok);
        promptForm.setCommandListener(this);
        changeDisplay(promptForm);
    }

    /**
     *   Changes the display to the given screen.
     *
     * @param newDisplay The screen to display
     * @return void
     */
    protected void changeDisplay(Screen newDisplay) {
        display.setCurrent(newDisplay);
    }

    /**
     * See CommandListener#commandAction method for more details.
     */
    public void commandAction(Command c, Displayable d) {
        if (c == status) {
            showStatus();
        }
        if (mode == WAITINGFORQMNAME && c == ok) {
            addNewQM(prompt.getString());
            initialize(prompt.getString());
        } else if (d == mainScreen) {
            if (c == status) {
                showStatus();
            } else if (c == send) {
                try {
                    model.sendNewMessage(message.getString());
                } catch (Exception e) {
                    displayStatus("Failed to send message");
                    e.printStackTrace();
                }
                clearMessage();
            }
        } else if (d == statusScreen) {
            changeDisplay(mainScreen);
        }
    }

    /**
     *   adds a new recordstore to the device containing the name of the queue manager
     *   this can be used of the app is restarted to keep the same qm name.
     *
     * @param name The name of the queue manager
     * @return void
     */
    protected void addNewQM(String name) {
        try {
            RecordStore temp = RecordStore.openRecordStore("appQMname-" + name, true);

            temp.closeRecordStore();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     *   Displays a status report to the user.
     *
     * @return void
     */
    protected void showStatus() {
        mode = SHOWINGSTATUS;
        String status = model.statusReport();

        statusScreen = new TextBox("-= Status =-", status, 1000, TextField.ANY);
        attachButtons(statusScreen, statusScreenButtons);
        statusScreen.setCommandListener(this);
        changeDisplay(statusScreen);
    }

    /**
     *   Clears the messagebox that is used by the user to enter messages.
     *
     * @return void
     */
    protected void clearMessage() {
        message.setString("");
    }

    /**
     *   This will search the record store for signs of a previous queue manager.
     *   If one existed, it will return the name of it, otherwise it will return
     *   null.
     *
     * @see examples.midp.exampleapp.textapp.AppController#getQMname
     */
    public String getQMname() {
        String stores[] = RecordStore.listRecordStores();
        String current;

        if (stores != null) {
            for (int i = 0; i < stores.length; ++i) {
                current = new String(stores[i]);
                if (current.startsWith("appQMname-")) {
                    return current.substring(10);
                }
            }
        }

        // QM name could not be found in the record store
        return null;
    }

    /**
     *   If the controller is initializing, use the initialScreen to display the
     *   message, otherwise use the statusbar. Either way, throw it to stdout too
     *   to make debugging easier.
     *
     * @see examples.midp.exampleapp.textapp.AppController#displayStatus
     */
    public void displayStatus(String msg) {
        if (mode != INITIALIZING) {
            statusbar.setText(msg);
        } else {
            initialScreen.setString(msg);
        }
        System.out.println(msg);
    }
}
