// Version mqe_java/source/examples/mqbridge/administration/commandline/MQeBridgesQuery.java, MQemqbridge, la200, la200-L030324.1 1.8

/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.mqbridge.administration.commandline;


import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;
import examples.mqbridge.queuemanager.*;
import com.ibm.mqe.mqbridge.*; 
import examples.administration.commandline.*;


/**
 * A command-line tool to query all the attributes of a bridges object.
 * 
 * <P><B>Syntax</B></P>
 * <P><TT>java examples.mqbridge.administration.commandline.MQBridgesQuery 
 * &lt;qMgr&gt; 
 * &lt;iniFile&gt;
 * </TT></P>
 * <P><B>Parameters</B></P>
 * <DL>
 * 
 * <DT>qMgr</DT>
 * <DD>The name of the queue manager to which the admin msg should be sent.</DD>
 * 
 * <DT>iniFile</DT>
 * <DD>The fully-qualified path name of the .ini file of the server queue manager.</DD>
 * 
 * </DL>
 * 
 * <P><B>Example</B></P>
 * <P><TT>java ...MQBridgesQuery MyQMgr C:\MQe\QMgrs\MyQMgr\MyQMgr.ini
 * </TT></P>
 * <P>Queries and displays all attributes of the bridges object obtained from the queue manager.</P>
 */
public class MQeBridgesQuery {

    public       static short version[] = { 2, 0, 0, 10 }; 

    /**
     * Syntax of the command.
     */
    public static final String syntax = 
        "Syntax: ...MQBridgesQuery <qMgr> <iniFile>\n"
        + "Where...\n"
        + "qMgr\n"
        + "  The name of the queue manager to which the admin msg should be sent.\n"
        + "iniFile\n"
        + "  The fully-qualified path name of the .ini file of the server queue manager.\n";

    /**
     * Entry point from the command line.
     */
    public static void main(String args[]) {
        int rc = 0;
        MQeBridgesQuery me = new MQeBridgesQuery();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Gathers the parameters and passes them to a method which does the command.
     * @param args
     * @return void
     * @exception java.lang.Exception
     */
    public void activate(String args[]) throws Exception {

        String targetQMgr = null;   // The queue manager to which the admin msg is sent.
        String iniFile = null;      // The ini file used to create the server while we use it.
 
        if (args.length < 2) {
            throw new Exception("Error: Not enough parameters. " + syntax);
        }

        targetQMgr = args[0].trim();
        iniFile = args[1].trim();

        // Access the local MQe queue manager...
        MQeServer server = new MQeServer(iniFile, true // Supress any stdout information.
            );

        // Start up the server...
        server.activate(true);

        // Access the queue manager created in this JVM... the server QM
        MQeQueueManager myQMgr = MQeQueueManager.getDefaultQueueManager();

        // Query the bridges object from the target queue manager...
        queryMQBridges(targetQMgr, myQMgr);

        // Ask the server to shut down peacefully...
        server.activate(false);

        // Close down the MQeClient
        server.close();
    } 

    /**
     * Creates an admin message which will query a bridges object, and send it
     * to a named queue manager.
     * 
     * <P>When it gets a response, a summary of the information obtained will be output
     * to the System.out stream.</P>
     * 
     * @param targetQMgr The name of the MQe queue manager on which the bridge will be created.
     * 
     * @param myQMgr A reference to an active MQe queue manager, used to send mesages from and receive messages using.
     * @return void
     * @exception Exception
     */
    public void queryMQBridges(String 			targetQMgr, MQeQueueManager 	myQMgr
    ) throws Exception {

        System.out.println("Building the admin message to query a bridges resource.");
        MQeMQBridgesAdminMsg msg = new MQeMQBridgesAdminMsg();

        // Set the command in the admin msg to "inquire".
        msg.inquireAll(null);

        System.out.println("Built the admin message.");

        System.out.println("Sending the admin message to " + targetQMgr
            + " using queue manager " + myQMgr.getName());

        // Aim the admin message at the queue manager whose bridges resource we wish to query
        msg.setTargetQMgr(targetQMgr);
        
        // Indicate that we want a reply message.
        msg.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);  
        
        // Indicate the reply message should go to the normal admin reply queue.
        msg.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);      

        // Make sure the response to this request is posted to the same queue manager we're
        // sending the administration message to.
        // Later, we can synchronously get the response.
        msg.putAscii(MQe.Msg_ReplyToQMgr, targetQMgr); 

        // Setup the correl id so we can match the reply to the request
        // In this case, just get a unique correl-id especially for this request.
        msg.putArrayOfByte(MQe.Msg_CorrelID, Long.toHexString(myQMgr.uniqueValue()).getBytes());

        // Put the message to the queue...
        myQMgr.putMessage(targetQMgr	        // Queue manager we're sending to.
            , MQe.Admin_Queue_Name	// Admin queue we're sending to.
            , msg                  // Message we're sending.
            , null                 // No attributes on this message.
            , (long) 0              // We're not sending a confirm.
        );

        System.out.println("Sent the message ok.");

        // Set up the "get-filter" to pick up the correct response.
        MQeFields msgFilter = new MQeFields();

        // Copy the correl-id from the original message... so we can use it as a filter...
        msgFilter.putArrayOfByte(MQe.Msg_CorrelID, msg.getArrayOfByte(MQe.Msg_CorrelID));

        System.out.println("Waiting for a response to the admin message to arrive");

        // Get the reply from the response queue...
        MQeAdminMsg replyMsg = (MQeAdminMsg) myQMgr.waitForMessage(
                targetQMgr            		// The queue manager we're getting the response from.
                , MQe.Admin_Reply_Queue_Name	// The queue we're expecting the response to appear on.
                , msgFilter             		// Filter on which message we want to wait for.
                , null                  		// No attributes.
                , (long) 0               		// We're not going to confirm our get.
                , 10 * 1000						// Give up if no response after 10 seconds.
            );

        // Make sure we got a message back, or fail.
        if (replyMsg == null) {
            System.out.println("No response got back!");
            throw new Exception("No response to the request message was received.");
        } 

        System.out.println("Got a response to the admin message.");

        // Make sure the response was a success...
        if (replyMsg.getRC() == MQeAdminMsg.RC_Fail) {
            System.out.println("Action caused by the admin message failed!");
            throw new MQeException(replyMsg.getReason());           
        }

        System.out.println("Admin message worked ok.");

        // We have sent an InquireAll MQeMQBridgesAdminMsg message to a bridge-enabled
        // queue manager. 
	
        // The reply message has just been recovered from a queue.
		
        // The message received is referenced using the replyMsg variable.
        // replyMsg is of type MQeAdminMsg

        // Extract the answer to the administration request into an MQeFields object,
        // for easy manipulation.
        MQeFields results = replyMsg.getOutputFields();
        // Extract an array holding all the information of the bridge resources which are children 
        // of the bridges resource.
        
        MQeFields[] children = results.getFieldsArray(MQeCharacteristicLabels.MQE_FIELD_LABEL_CHILDREN);

        // Display facets of the bridges resource to System.out
        System.out.println("Bridges resource:");

        // To display the name of the Websphere MQ-bridge, proxy, client connection, 
        // or listener.
        
        String fieldName = results.getUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_NAME);

        System.out.println("The Field_Label_Name is " + fieldName);

        String ruleClass = results.getUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_STARTUP_RULE_CLASS);

        System.out.println("The StartUp_Rule is " + ruleClass);

        // Display the list of all the bridges in the list.
        System.out.println("Children:(" + Integer.toString(children.length) + " of these)");
        for (int childIndex = 0; childIndex < children.length; childIndex += 1) {
            // Pick out the name of this child.
            
            String bridgeName = children[childIndex].getUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_CHILD);

            System.out.println("Bridge defined is called " + bridgeName);
        }
        
        String desc = results.getUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_DESCRIPTION);

        System.out.println("Description is " + desc);
        
        String adminClass = results.getUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_ADMINISTERED_OBJECT_CLASS);

        System.out.println("AdminClass : " + adminClass);

        // To control whether the delete/start action affects child objects.
        
        results.putBoolean(MQeCharacteristicLabels.MQE_FIELD_LABEL_AFFECT_CHILDREN, true);
        
        // To Label the field containing the BridgeName
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_BRIDGE_NAME, "MQBridgeV100");
        
        // Label to hold the Default transformer
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_DEFAULT_TRANSFORMER,
            "com.ibm.mqe.mqbridge.MQeJMSRFHTransformer");
       
        // The label holding the value of the time interval (in units of 1 minute) that dictates the period 
        // of each ?heartbeat????event coming from the MQeHeart class.
        
        results.putInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_HEARTBEAT_INTERVAL, 5);
        
        String name = results.getUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_MQ_Q_MGR_PROXY_NAME);

        System.out.println("The MQ QMgr Proxy Name : " + name);
     
        // Using an IP address...
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_HOST_NAME, "127.0.0.1");
        // Using a host in the default domain
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_HOST_NAME, "lizzie");
        // Using a fully qualified hostname
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_HOST_NAME, "daytona2.hursley.ibm.com");
        // Using the WebSphere MQ  Java Bindings on localhost
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_HOST_NAME, ""); 
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_CLIENT_CONNECTION_NAME, "SYSTEM.DEF.SVRCONN");
        
        results.putInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_PORT, 1414);
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_USER_ID, "mqm");
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_PASSWORD, "chocolate");
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_SECURITY_EXIT, "my.SecurityExit");
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_RECEIVE_EXIT, "my.ReceiveExit");
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_SEND_EXIT, "my.SendExit");
        
        results.putInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_CCSID, 819);
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_MQ_BRIDGE_ADAPTER_CLASS, "com.ibm.mqe.mqbridge.MQeMQAdapter");
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_SYNC_Q_PURGER_RULES_CLASS, 
            "com.ibm.mqe.mqbridge.MQeSyncQueuePurgerRule");
        
        results.putInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_MAX_CONNECTION_IDLE_TIME, 5);
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_SYNC_Q_NAME, "com.ibm.mqe.mqbridge.MQeSyncQueueName");
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_LISTENER_NAME, "MQE.XMITQ");
        
        results.putInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_SYNC_Q_PURGE_INTERVAL, 60);
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_DEAD_LETTER_Q_NAME, "MQE.DLQ");
        
        results.putInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_SECONDS_WAIT_FOR_MSG, 10);
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_UNDELIVERED_MESSAGE_RULE_CLASS,
            "com.ibm.mqe.mqbridge.MQeUndeliveredMessageRule");
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_LISTENER_STATE_STORE_ADAPTER,
            "com.ibm.mqe.adapters.MQeDiskFieldsAdapter");
        
        results.putInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_FLOWS_PER_COMMIT, 100);
        
        results.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_TRANSFORMER, 
            "com.ibm.mqe.mqbridge.MQeJMSRFHTransformer");
        
        // Output the run-state characteristic.
        int runState = results.getInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_RUN_STATE);

        switch (runState) {
        case MQeRunState.RUN_STATE_RUNNING:
            System.out.println("Run state is running.");
            break;

        case MQeRunState.RUN_STATE_STOPPED:
            System.out.println("Run state is stopped.");
            break;

        default:
            System.out.println("Runstate field holds unexpected value !");
        }
    } 
}

