// Version mqe_java/source/examples/mqbridge/administration/commandline/MQeBridgesStop.java, MQemqbridge, la200, la200-L030324.1 1.5

/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.mqbridge.administration.commandline;


import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;
import examples.mqbridge.queuemanager.*;
import com.ibm.mqe.mqbridge.*;
import examples.administration.commandline.*;


/**
 * A command-line tool to stop a bridges resource running.
 * 
 * <P>All children resources of the bridges resource are stopped as a result.</P>
 * 
 * <P><B>Syntax</B></P>
 * <P><TT>java examples.mqbridge.administration.commandline.MQBridgesStop
 * &lt;affectChildren&gt;
 * &lt;qMgr&gt; 
 * &lt;iniFile&gt;
 * </TT></P>
 * <P><B>Parameters</B></P>
 * <DL>
 * 
 * <DT>qMgr</DT>
 * <DD>The name of the queue manager to which the admin msg should be sent.</DD>
 * 
 * <DT>iniFile</DT>
 * <DD>The fully-qualified path name of the .ini file of the server queue manager.</DD>
 * 
 * </DL>
 * 
 * <P><B>Example</B></P>
 * <P><TT>java ...MQBridgesStop MyQMgr C:\MQe\QMgrs\MyQMgr\MyQMgr.ini
 * </TT></P>
 * <P>Stops the bridges resource, and all bridge, MQSeries proxy, client connection and
 * transmission queue listener bridge resources which exist on the MyQMgr, using the MyQMgr to
 * send the administration message.</P>
 */
public class MQeBridgesStop {  

    public       static short version[] = { 2, 0, 0, 10 }; 

    /**
     * Syntax of the command.
     */
    public static final String syntax = 
        "Syntax: ...MQBridgesStop <qMgr> <iniFile>\n"
        + "Where...\n"
        + "qMgr\n"
        + "  The name of the queue manager to which the admin msg should be sent.\n"
        + "iniFile\n"
        + "  The fully-qualified path name of the .ini file of the server queue manager.\n";

    /**
     * Entry point from the command line.
     */
    public static void main(String args[]) {
        int rc = 0;
        MQeBridgesStop me = new MQeBridgesStop();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Gathers the parameters and passes them to a method which does the command.
     * @param args
     * @return void
     * @exception java.lang.Exception
     */
    public void activate(String args[]) throws Exception {

        String targetQMgr = null;   // The queue manager to which the admin msg is sent.
        String iniFile = null;      // The ini file used to create the server while we use it.
 
        if (args.length < 2) {
            throw new Exception("Error: Not enough parameters. " + syntax);
        }

        targetQMgr = args[1].trim();
        iniFile = args[2].trim();

        // Access the local MQe queue manager...
        MQeServer server = new MQeServer(iniFile, true // Supress any stdout information.
            );

        // Start up the server...
        server.activate(true);

        // Access the queue manager created in this JVM... the server QM
        MQeQueueManager myQMgr = MQeQueueManager.getDefaultQueueManager();

        // Stop the bridges resource from the target queue manager...
        stopMQBridges(targetQMgr, myQMgr);

        // Ask the server to shut down peacefully...
        server.activate(false);

        // Close down the MQeClient
        server.close();
    }  
   
    /**
     * Creates an admin message which will stop a bridges resource, and all those resources
     * which are children.
     * 
     * @param targetQMgr The name of the MQe queue manager on which the bridge will be created.
     * @param myQMgr A reference to an active MQe queue manager, used to send mesages from and receive messages using.
     * @return void
     * @exception Exception
     */
    public void stopMQBridges(String 	targetQMgr, MQeQueueManager myQMgr
    ) throws Exception {

        System.out.println("Building the admin message to stop a bridges resource.");
        MQeMQBridgesAdminMsg msg = new MQeMQBridgesAdminMsg();

        // Set the command in the admin msg to "inquire".
        msg.stop();

        System.out.println("Built the admin message.");

        System.out.println("Sending the admin message to " + targetQMgr
            + " using queue manager " + myQMgr.getName());

        // Aim the admin message at the queue manager whose bridges resource we wish to query
        msg.setTargetQMgr(targetQMgr);
        
        // Indicate that we want a reply message.
        msg.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);  
        
        // Indicate the reply message should go to the normal admin reply queue.
        msg.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);      

        // Make sure the response to this request is posted to the same queue manager we're
        // sending the administration message to.
        // Later, we can synchronously get the response.
        msg.putAscii(MQe.Msg_ReplyToQMgr, targetQMgr); 

        // Setup the correl id so we can match the reply to the request
        // In this case, just get a unique correl-id especially for this request.
        msg.putArrayOfByte(MQe.Msg_CorrelID, Long.toHexString(myQMgr.uniqueValue()).getBytes());

        // Put the message to the queue...
        myQMgr.putMessage(targetQMgr	        // Queue manager we're sending to.
            , MQe.Admin_Queue_Name	// Admin queue we're sending to.
            , msg                  // Message we're sending.
            , null                 // No attributes on this message.
            , (long) 0              // We're not sending a confirm.
        );

        System.out.println("Sent the message ok.");

        // Set up the "get-filter" to pick up the correct response.
        MQeFields msgFilter = new MQeFields();

        // Copy the correl-id from the original message... so we can use it as a filter...
        msgFilter.putArrayOfByte(MQe.Msg_CorrelID, msg.getArrayOfByte(MQe.Msg_CorrelID));

        System.out.println("Waiting for a response to the admin message to arrive");

        // Get the reply from the response queue...
        MQeAdminMsg replyMsg = (MQeAdminMsg) myQMgr.waitForMessage(
                targetQMgr            		// The queue manager we're getting the response from.
                , MQe.Admin_Reply_Queue_Name	// The queue we're expecting the response to appear on.
                , msgFilter             		// Filter on which message we want to wait for.
                , null                  		// No attributes.
                , (long) 0               		// We're not going to confirm our get.
                , 10 * 1000						// Give up if no response after 10 seconds.
            );

        // Make sure we got a message back, or fail.
        if (replyMsg == null) {
            System.out.println("No response got back!");
            throw new Exception("No response to the request message was received.");
        } 

        System.out.println("Got a response to the admin message.");

        // Make sure the response was a success...
        if (replyMsg.getRC() == MQeAdminMsg.RC_Fail) {
            System.out.println("Action caused by the admin message failed!");
            throw new MQeException(replyMsg.getReason());           
        }

        System.out.println("Admin message worked ok.");
    }  
} 

