// Version 1.2 mqe_java/source/examples/mqbridge/administration/programming/AdminHelperBridge.java, MQeExamples, la200, la200-L030324.1

/*------------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms                *
 *                                                                              *
 * Licensed Materials - Property of IBM                                         *
 *                                                                              *
 * 5765-E63                                                                     *
 *                                                                              *
 * (C) Copyright IBM Corp. 2002                                                 *
 *------------------------------------------------------------------------------*/
package examples.mqbridge.administration.programming;


import java.util.*;
import com.ibm.mqe.*;
import com.ibm.mqe.mqbridge.*;


public class AdminHelperBridge {
    public static short[] version = { 2, 0, 0, 10 }; 	
    Reporter reporter;
    AdminMessageDispatcher dispatcher;
	
    /**
     * Builds an object able to administer bridge resources.
     * 
     * @param reporter The object to which debug and error information is sent.
     * @param dispatcher An object used to dispatch administration messages.
     */
    public AdminHelperBridge(Reporter reporter, AdminMessageDispatcher dispatcher
    ) {
        this.reporter = reporter;
        this.dispatcher = dispatcher;
    }
	
    /**
     * Queries all facets of the specified bridge.
     * 
     * @param targetMQeQueueManagerName The name of the queue manager to which the admin message 
     * will be directed.
     * @param mqeQueueManagerToUse The active queue manager used to send and receive admin messages.
     * @param bridgeName The name of the bridge resource to query.
     * @return MQeFields The data describing the bridge which has been queried.
     * @throws Exception If something goes wrong.
     */
    public MQeFields queryAll(String targetMQeQueueManagerName, String bridgeName
    ) throws Exception {
        MQeMQBridgeAdminMsg msg = new MQeMQBridgeAdminMsg();

        msg.setName(bridgeName);
        msg.inquireAll(new MQeFields());
        MQeFields queryResults = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);

        return queryResults;												  													  	
    }	
	
    /**
     * Start a bridge resource running, and optionally start its' children resources running too.
     */
    public void start(String targetMQeQueueManagerName, String bridgeName, boolean affectsChildren
    ) throws Exception {
        MQeMQBridgeAdminMsg msg = new MQeMQBridgeAdminMsg();

        msg.setName(bridgeName);
        msg.start(affectsChildren);
        MQeFields results = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);										
    }	
	
    /**
     * Stop a bridge and all its' child resources from running.
     */
    public void stop(String targetMQeQueueManagerName, String bridgeName
    ) throws Exception {
        MQeMQBridgeAdminMsg msg = new MQeMQBridgeAdminMsg();

        msg.setName(bridgeName);
        msg.stop();
        MQeFields results = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);										
    }	
	
    /**
     * Finds out whether a bridge resource is running or not.
     */		
    public boolean isRunning(String targetMQeQueueManagerName, String bridgeName
    ) throws Exception {
        boolean isRunning = false;
        MQeFields queryResults = queryAll(targetMQeQueueManagerName, bridgeName
            );
        int runState = queryResults.getInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_RUN_STATE);

        if (runState == MQeRunState.RUN_STATE_RUNNING) {
            isRunning = true;
        } else {
            isRunning = false;
        }		   												
        return isRunning;												  	
    }
	
    /**
     * Provides a list of child resources controlled by the specified bridge resource.
     * <p>Each member of the list returned is an MQ queue manager proxy resource name.
     */
    public Hashtable listChildren(String targetMQeQueueManagerName, String bridgeName
    ) throws Exception {
        Hashtable results = new Hashtable();
        MQeFields queryResults = queryAll(targetMQeQueueManagerName, bridgeName);		
        MQeFields[] children = queryResults.getFieldsArray(MQeCharacteristicLabels.MQE_FIELD_LABEL_CHILDREN);

        for (int childIndex = 0; childIndex < children.length; childIndex += 1) {	
            String resourceName = children[childIndex].getUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_CHILD);

            results.put(resourceName, resourceName);	
        }
        return results;											     	
    }
		
    /**
     * Checks to see if the bridge exists of not.
     * 
     * @param targetMQeQueueManagerName The name of the queue manager to which admin messages are sent.
     * @param bridgeName The name of the bridge resource to check for.
     * @return boolean True if the bridge does exist. False otherwise.
     * @throws Exception if there is a problem.
     */
    public boolean doesExist(String targetMQeQueueManagerName, String bridgeName
    ) throws Exception {
        boolean doesExist = false;
		
        AdminHelperBridges bridgesAdminHelper = new AdminHelperBridges(reporter, dispatcher);													  	
        Hashtable bridgeList = bridgesAdminHelper.listChildren(targetMQeQueueManagerName);

        if (bridgeList.get(bridgeName) == null) {
            doesExist = false;
        } else {
            doesExist = true;
        }		
        return doesExist;																
    }	
	
    public void create(String targetMQeQueueManagerName, String bridgeName, String resourceDescription
    ) throws Exception {
        reporter.debug("Building the admin message to create a bridge "
            + " queue manager effected=" + targetMQeQueueManagerName
            + " bridge name=" + bridgeName 
            + " resource descrition=" + resourceDescription
        );
            	        
        MQeMQBridgeAdminMsg msg = new MQeMQBridgeAdminMsg();

        // Create a fields object in which we can pre-initialise the bridge...
        MQeFields params = new MQeFields();

        // When we create the bridge, set the default transformer.       
        params.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_DESCRIPTION, resourceDescription);
        
        // Set the command in the admin msg to "create".
        msg.create(params);
        
        // Set the name of the bridge we wish to create into the admin msg...
        msg.setName(bridgeName);

        reporter.debug("Built the admin message.");

        // Send the message.
        MQeFields results = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);	  									 	
    }					  									 	
	
    /**
     * Creates a bridge resource on the specified queue manager.
     * 
     * @param targetMQeQueueManagerName The name of the queue manager on which the bridge will be created.
     * @param bridgeName The name of the bridge resource to create
     * @param defaultTransformerClassName The name of the default transformer.
     * @param resourceDescription The description of the resource used.
     * @param heartbeatIntervalMinutes The number of minutes between "hearbeat" events in the bridge.
     */
    public void create(String targetMQeQueueManagerName, String bridgeName, String defaultTransformerClassName, String resourceDescription, int heartbeatIntervalMinutes
    ) throws Exception {
        reporter.debug("Building the admin message to create a bridge "
            + " queue manager effected=" + targetMQeQueueManagerName
            + " bridge name=" + bridgeName
            + " default transformer=" + defaultTransformerClassName 
            + " resource descrition=" + resourceDescription
            + " heartbeat interval=" + heartbeatIntervalMinutes + " minutes"
        );
            	        
        MQeMQBridgeAdminMsg msg = new MQeMQBridgeAdminMsg();

        // Create a fields object in which we can pre-initialise the bridge...
        MQeFields params = new MQeFields();

        // When we create the bridge, set the default transformer.
        params.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_DEFAULT_TRANSFORMER, defaultTransformerClassName);
        params.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_DESCRIPTION, resourceDescription);
        params.putInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_HEARTBEAT_INTERVAL, heartbeatIntervalMinutes);
        
        // Set the command in the admin msg to "create".
        msg.create(params);
        
        // Set the name of the bridge we wish to create into the admin msg...
        msg.setName(bridgeName);

        reporter.debug("Built the admin message.");

        // Send the message.
        MQeFields results = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);	  									 	
    }					  									 	
	
    /**
     * Deletes a bridge resource from a specified queue manager.
     * 
     * @param targetMQeQueueManagerName The name of the queue manager from which the resource will be removed.
     * @param bridgeName The name of the resource to remove.
     * @throws Exception If there is a failure and Exception is thrown.
     */
    public void delete(String targetMQeQueueManagerName, String bridgeName
    ) throws Exception {
        reporter.debug("Building the admin message to delete a bridge "
            + "queue manager effected=" + targetMQeQueueManagerName
            + "bridge name=" + bridgeName
        );
        MQeMQBridgeAdminMsg msg = new MQeMQBridgeAdminMsg();
        MQeFields params = new MQeFields();

        msg.delete(params);
        msg.setName(bridgeName);
        reporter.debug("built the admin message.");
        MQeFields results = dispatcher.sendWaitMessage(reporter, targetMQeQueueManagerName, msg);	  									 																		
    }				
}
