// Version 1.2 mqe_java/source/examples/mqbridge/administration/programming/AdminHelperDispatcher.java, MQeExamples, la200, la200-L030324.1

/*------------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms                *
 *                                                                              *
 * Licensed Materials - Property of IBM                                         *
 *                                                                              *
 * 5765-E63                                                                     *
 *                                                                              *
 * (C) Copyright IBM Corp. 2002                                                 *
 *------------------------------------------------------------------------------*/
package examples.mqbridge.administration.programming;


import java.util.*;
import com.ibm.mqe.*;
import com.ibm.mqe.mqbridge.*;


public class AdminHelperDispatcher implements AdminMessageDispatcher {
    public static short[] version = { 2, 0, 0, 10 }; 
    	
    private static final int TIMETIMEOUT_WAIT_FOR_ADMIN_REPLY_MILLISECS = 10000;
    private static final long DO_NOT_CONFIRM = 0L;
    private static final MQeAttribute NULL_ATTRIBUTES = null;
    private static final int WAIT_POLL_INTERVAL_MILLSECS = 500;
    private static final MQeFields NULL_FILTER = null;	
	
    MQeQueueManager activeMQeQueueManagerToUse;
	
    public AdminHelperDispatcher(MQeQueueManager activeMQeQueueManagerToUse) {
        this.activeMQeQueueManagerToUse = activeMQeQueueManagerToUse;
    }

    /**
     * To put the specified message to the Admin Q on the specified MQe queue manager
     * and get a response.
     *
     * @param myQMgr A reference to an active MQe queue manager, which will be used send and receive
     *               MQe administration messages.
     *
     * @param targetQMgrName
     *               The name of the queue manager to which the message will be sent.
     *
     * @param msg    The message we will send to the target queue manager, and await a response.
     *
     * @return The MQeFields response that the reply message contains
     * @exception Exception fails it the reply indicates the Admin message
     *                      caused a failure, or if there was a failure posting the message.
     */
    public MQeFields sendWaitMessage(Reporter reporter, String targetQMgrName, MQeAdminMsg  msg
    ) throws Exception {
        reporter.debug("..Sending the admin message to " + targetQMgrName + " using queue manager " + activeMQeQueueManagerToUse.getName());

        // Prime the admin message, for example set the reply to queue...
        MQeFields msgFilter = primeAdminMsg(reporter, targetQMgrName, msg);

        // Put the message and wait for a reply
        MQeAdminMsg respMsg = processAdminMsg(reporter, targetQMgrName, msg, msgFilter);

        // Check if the request worked if not an exception is thrown
        checkAdminReply(reporter, respMsg);

        // If no exception was thrown then admin request was successful.
        reporter.debug("..Admin request was successful.");

        return respMsg.getOutputFields();
    }
    
    /**
     * Check to see if a good reply was received. 
     * 
     * <p>If not detail the error(s) that occured and then throw an exception.
     *
     * @return boolean  true if good
     * @param replyMsg MQeAdminMsg - reply message to check
     */
    public boolean checkAdminReply(Reporter reporter, MQeAdminMsg replyMsg
    ) throws Exception {
        // Was a reply received ?
        if (replyMsg == null) {
            reporter.error("..No response received to the request");
            throw new MQeException("No response message received");
        }

        // If the reply was bad then check why
        if (replyMsg.getRC() != MQeAdminMsg.RC_Success) {
            reporter.error("..Action Failed: " + replyMsg.getReason());

            // If mixed then list each error that occured
            if (replyMsg.getRC() == MQeAdminMsg.RC_Mixed) {
                MQeFields errors = replyMsg.getErrorFields();
                Enumeration en = errors.fields();
            
                // process each error
                while (en.hasMoreElements()) {
                    String value[] = {};
                    String name = (String) en.nextElement();

                    // Field in error may be an array
                    if (errors.dataType(name) == MQeField.TypeArrayElements)
                        value = errors.getAsciiArray(name);
                    else { // in the case when it is not an array
                        // Ignore individual dynamic array elements, the array is handled
                        // as a whole.
                        if (name.indexOf(MQeFields.ArraySeparator) < 0) {
                            value = new String[] { errors.getAscii(name)};
                        }
                    }

                    for (int j = 0; j < value.length; j++) {
                        reporter.error("Field in error: " + name + "  " + value[j]);
                    }
                }
            }
            // Failed - throw exception
            throw new MQeException(replyMsg.getReason());
        }
        return true; 
    }    

    /**
     * Send the admin message, and wait for a repy. 
     * 
     * <p>An exception is thrown if there is an error
     * 
     * @param myQMgr A reference to an active MQe queue manager, which will be
     *                used to send and receive administration messages.
     *
     * @param onQMgr   - the name of the queue manager where the message will be sent.
     * @param msg      - the admin message to send
     * @param filter   - an mqefields object that allows request and reply
     *                    messages to be correlated.
     * @return replyMsg  returns the message received in response to the
     *                    admin request or null if none was returned.
     */
    public MQeAdminMsg processAdminMsg(Reporter reporter, String onQMgr, MQeAdminMsg msg, MQeFields filter
    ) throws Exception {
			
        MQeAdminMsg respMsg = null;
		
        if (!activeMQeQueueManagerToUse.getName().equals(onQMgr)) {
			
            // Put the admin message to the admin queue
            reporter.debug(
                "Put admin message to remote QM"
                + "remote queue manager name=" + onQMgr
                + "remote queue name=" + MQe.Admin_Queue_Name
            );
            activeMQeQueueManagerToUse.putMessage(onQMgr, MQe.Admin_Queue_Name, msg, null, 0);
	
            // Wait a while for the response message		         
            reporter.debug("Admin reply is targetted at a remote queue manager... waiting for a response to the admin message to arrive");

            respMsg = (MQeAdminMsg) activeMQeQueueManagerToUse.waitForMessage(
                        onQMgr, MQe.Admin_Reply_Queue_Name, filter, NULL_ATTRIBUTES, DO_NOT_CONFIRM, TIMETIMEOUT_WAIT_FOR_ADMIN_REPLY_MILLISECS, WAIT_POLL_INTERVAL_MILLSECS                
                    );
            
        } else {
            // Create and prime the listener to receive traffic.
            PrimedTriggerListener listener = new PrimedTriggerListener();

            listener.prime();
			
            try {
			
                // Register the listener so it gets notified when the reply admin message arrives.
                activeMQeQueueManagerToUse.addMessageListener(listener, MQe.Admin_Reply_Queue_Name, filter);
			
                // Put the admin message to the admin queue
                reporter.debug(
                    "Put admin message to local QM"
                    + "local queue manager name=" + onQMgr
                    + "local queue name=" + MQe.Admin_Queue_Name
                );
                activeMQeQueueManagerToUse.putMessage(onQMgr, MQe.Admin_Queue_Name, msg, null, 0);
		
                // Wait a while for the response message	         
                reporter.debug("Admin reply is targetted at a local queue manager... waiting for a response to the admin message to arrive");
		
                // Listen for a reply to arrive.
                boolean replyMessageArived = listener.waitForTrigger(TIMETIMEOUT_WAIT_FOR_ADMIN_REPLY_MILLISECS);

                if (replyMessageArived) {
                    // A message arrived. Go get it.
                    respMsg = (MQeAdminMsg) activeMQeQueueManagerToUse.getMessage(onQMgr, MQe.Admin_Reply_Queue_Name, filter, null, 0L);				
                }
            } finally {
                // De-register the listener
                activeMQeQueueManagerToUse.removeMessageListener(listener, MQe.Admin_Reply_Queue_Name, filter);				
            }			
        }			

        return respMsg;
    }
        
    /**
     * Primes an admin message. 
     * 
     * <p>Sets the admin message with items such as
     * who to reply to along with a unique identifier that can be used to
     * check for a matching response.
     *
     * @param onQMgr   - the name of the queue manager where the message will be sent.
     * @param msg      - the admin message to prime
     * @return MQeFields - a unique identifier for this request
     */
    public MQeFields primeAdminMsg(Reporter reporter, String onQMgr, MQeAdminMsg msg)
        throws Exception {

        /*
         * Set the target queue manager that will process this message
         */
        msg.setTargetQMgr(onQMgr);

        /*
         * Ask for a reply message
         */
        msg.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);
        msg.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
        msg.putAscii(MQe.Msg_ReplyToQMgr, onQMgr);

        /*
         *  Setup the correl id so we can match the reply to the request
         */
        msg.putArrayOfByte(
            MQe.Msg_CorrelID,
            Long.toHexString(MQe.uniqueValue()).getBytes());

        /*
         * Ensure matching response message is retrieved
         */
        MQeFields msgTest = new MQeFields();

        msgTest.putArrayOfByte(
            MQe.Msg_CorrelID,
            msg.getArrayOfByte(MQe.Msg_CorrelID));

        /*
         * Return the unique filter for this message
         */
        return msgTest;
    }
    
    /**
     * Sends a message.
     */
    public boolean send(Reporter reporter, String targetQueueManagerName, String targetQueueName, MQeMsgObject msg) {
        boolean isOk = true;

        try {
            activeMQeQueueManagerToUse.putMessage(targetQueueManagerName, targetQueueName, msg, NULL_ATTRIBUTES, DO_NOT_CONFIRM);
        } catch (Exception e) {
            reporter.error(e);
            isOk = false;
        }
        return isOk;
    }
    
    /**
     * Gets a message.
     */
    public MQeMsgObject get(Reporter reporter, String targetQueueManagerName, String targetQueueName) {
        MQeMsgObject result = null;

        try {
            result = activeMQeQueueManagerToUse.getMessage(targetQueueManagerName, targetQueueName, NULL_FILTER, NULL_ATTRIBUTES, DO_NOT_CONFIRM);
        } catch (Exception e) {
            reporter.error(e);
        }
        return result;
    }
}
