/*------------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                    */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.mqbridge.rules;


import examples.rules.*;
import com.ibm.mqe.*;
import com.ibm.mq.*;
import java.text.*;
import java.util.*;


/******************************************************************************
 *
 * This is an example rules class for the MQ-Bridge sync queue purger.
 *
 * <p>To allow assured message delivery, the sync queue contains one record
 * per MQe message that is in the state of having been sent to MQ but still
 * locked ready for deletion at the MQe queue manager. In the event of a crash,
 * users could manually remove these locked messages but leave the sync queue
 * records alone, which means that over time (on an unstable system) sync
 * queue records can build up, resulting in poor bridge performance.</p>
 *
 * <p>This is an example class which dictates what action to take upon old
 * records that are have been left on the sync queue.
 * Whenever purging of the sync queue takes place, the bridge calls the
 * permit() method with each unconfirmed message that is currently on the sync
 * queue. This rules class then decides if this is an old message that is no
 * longer relevant, and thus if it can be safely purged by the bridge.</p>
 *
 * <p> This class allows any messages that are over ten
 * minutes old to be deleted. </p>
 *
 ******************************************************************************/
public class DestructiveMQSyncQueuePurgerRule extends MQeRule {

    public        static short    version[] = { 2, 0, 0, 10 }; 

    /**
     * The threshold for deleting (in minutes)
     */
    private static final long SECONDS_BEFORE_DELETE = 600;  // 10 minutes
    
    /**
     * Decides whether a given message can be deleted.
     * This implementation of the rule deletes any
     * messages that are over ten minutes old.
     * @param MQMessage the sync queue record in question
     * @param Long the age of the message in seconds.
     * @param String the name of the sync queue being purged. 
     * @return Boolean true if the message can be deleted.
     */
    public Object permit(Object messageO, Object ageO, Object syncQNameO
    ) throws Exception {

        // Check that all the parameters are of the correct type...
        if ((!(messageO instanceof MQMessage)) 
            || (!(ageO instanceof Long))
            || (!(syncQNameO instanceof String))) {
            throw new MQeException(Except_Rule, "Disallowed by rule", "Parameters have incorrect types"
                );
        }    

        boolean retVal;

        // Create strongly typed versions of the parameters...
        MQMessage message = (MQMessage) messageO;
        long age = ((Long) ageO).longValue();
        String syncQName = (String) syncQNameO;
        
        // If the message is older than ten minutes then display a warning before asking the
        // purging application to it.
        if (age > SECONDS_BEFORE_DELETE) {

            // Determine when the message was put to the sync queue.
            Date gcDate = message.putDateTime.getTime();
            SimpleDateFormat formatter = new SimpleDateFormat("hh:mm:ss 'on' dd MMMM, yyyy.");
            String dateString = formatter.format(gcDate);
 
            // Display a warning.
	   
            MQeTrace.trace(this, (short) 0, MQeTrace.GROUP_INFO, syncQName, dateString);
            // Delete the message, please.
            retVal = true;
        } else {

            // The message is not old enough. Leave it there for the time being.
            retVal = false;
        }
        
        return new Boolean(retVal);
    }
}
