/*------------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                    */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.mqbridge.rules;


import com.ibm.mqe.mqbridge.*;
import java.util.*;
import com.ibm.mqe.*;
import com.ibm.mqe.registry.*;


/********************************************************************************
 *
 * To be a model rules class.
 *
 * <p>This rule class is specified on an mqbridge administered object's
 * registry entry.
 *
 * <p>When the administered object is loaded, this rule class is consulted 
 * to see whether the administered object should be started, or left in the 
 * stopped state, or whether the administered object's children should be 
 * started or not.
 *
 ********************************************************************************/
public class MQeStartupRule extends MQeRule {

    public        static short    version[] = { 2, 0, 0, 10 }; 

    /**
     * A possible value returned from the permit() method.
     * Says "leave this administered object in its current state"
     */
    public static final int START_NOTHING = 0;

    /**
     * A possible value returned from the permit() method.
     * Says "start all my parents, and my parent's parents, then 
     * start me.
     */
    public static final int START_PARENTS_AND_ME = 1;

    /**
     * A possible value returned from the permit() method.
     * Says "start all my parents, and my parent's parents, then
     * start me, then start each of my children in turn.
     */
    public static final int START_PARENTS_AND_ME_AND_CHILDREN = 2;

    /*
     * No constructor, so MQeStartupRule() is provided by default.
     */
  
    /**
     * Query whether an administered object should be started
     * just after it has been loaded.
     * 
     * <p>The specified administered object has just been loaded into the
     * mqbridge system. The bridge wants to find out whether this
     * administered object needs to be moved into the "running" state, or
     * left in the "stopped" state.
     * 
     * <p>If this rule decides that this administered object should
     * be started, then the mqbridge will make sure that all the
     * parents of this administered object are also started.
     * 
     * <p>If this rule decides that this administered object should
     * be started, then it must also decide whether the children
     * of this administered object should also be started.
     * 
     * @param administeredObjectNameO
     *                  A String which has been cast
     *                  to an Object type. It holds the fully-qualified name of the
     *                  entry in the registry which identifies the administered object
     *                  to be started up. You can use it to look up values for
     *                  this administered object in the registry, to decide whether
     *                  this administered object should be started or otherwise.
     *
     * @param administeredObjectTypeO
     *                  A String which has been cast
     *                  to an Object type. It holds the "sort" of administered object
     *                  which has just been loaded. This can also be used to uniquely
     *                  identify an entry in the registry, whose values can be
     *                  queried to decide whether this object should be started or not.
     *
     * @param registryO An MQeRegistry which has been cast to an
     *                  Object type. It refers to an open, ready-to-use MQeRegistry
     *                  type which can be used to look-up the passed administered
     *                  object, so it's registry values can be queried. This can
     *                  be used to help decide whether this administered object
     *                  should be started or not.
     *
     * @return an Integer object. This value holds one of the START_*
     *         constant values defined by this class.
     *         <p>Valid return values are:
     *         <ul>
     *         <li>START_NOTHING
     *         <p>Says "leave this administered object in it's current state"
     *         
     *         <li>START_PARENTS_AND_ME
     *         <p>Says "start all my parents, and my parents' parents, then
     *         start me.
     *         
     *         <li>START_PARENTS_AND_ME_AND_CHILDREN
     *         <p>Says "start all my parents, and my parents' parents, then
     *         start me, then start each of my children in turn.
     *         </ul>
     *         
     *         <p>If any other value is returned, or a non-Integer is returned
     *         or the class cannot be loaded for some reason, or it throws
     *         an exception, then this has the same effect as returning
     *         the START_NOTHING constant value.
     *
     * @exception Exception Fails with an exception if something goes wrong
     * when deciding on the return value, or if the parameters are the
     * wrong type. An exception will have the same effect as returning 
     * the START_NOTHING value.
     *
     */
    public Object permit(Object administeredObjectNameO    // String
        , Object administeredObjectTypeO    // String
        , Object registryO                  // MQeRegistry
    ) throws Exception {
        if ((!(administeredObjectNameO instanceof String)) 
            || (!(administeredObjectTypeO instanceof String))
            || (!(registryO instanceof MQeRegistry))
        ) {
            throw new MQeException(Except_Rule, "Disallowed by rule", "Parameters have incorrect types"
                );
        }    

        // Create strongly typed versions of the parameters...
        String administeredObjectName = (String) administeredObjectNameO;
        String administeredObjectType = (String) administeredObjectTypeO;
        MQeRegistry registry = (MQeRegistry) registryO;
                                             
        // Call the strongly-typed version of the permit call...
        Integer result = permit(administeredObjectName, administeredObjectType, registry
            );

        return result; 
    }

    /*************************************************************************
     *
     * The same as the permit() method call, but the parameters are strongly
     * typed.
     *
     * <p>Although this rule class could use information contained in the 
     * registry to decide whether to start or otherwise the named 
     * administered object, this model class just returns the 
     * START_PARENTS_AND_ME value. This is a sensible default.
     *
     *************************************************************************/
    protected Integer permit(String administeredObjectName, String administeredObjectType, MQeRegistry registry
    ) {
        Integer result = new Integer(START_PARENTS_AND_ME);

        return result;
    }

}
