/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.mqbridge.transformers;


import com.ibm.mqe.mqbridge.*; 
import com.ibm.mqe.*; 
import com.ibm.mq.*;
import java.util.*;


/*****************************************************************************
 *
 * A simple MQ to MQe transformer class.
 *
 * <p>This transformer does not actually understand the format of any messages
 * it is passed. 
 *
 * <p>It has an ordered list of smaller transformers. 
 * When a message needs to be transformed, this class will work through its
 * list of transformers one by one, presenting the message to each
 * transformer. The results of the first transformer to successfully return
 * a converted message are returned to the user of this class.
 *
 * <p>This style of transformer could be used in conjunctions with a 
 * collection of smaller transformers, where each "small" transformer
 * understands a limited number of message formats.
 *
 * <p>This class keeps its list of transformers in a static ordered list (array)
 * but it could easily have read the list from a file when the activate method
 * is called, or obtained a list through some other method (possibly using the
 * user-defined parameters passed on the activate method to do so).
 *
 *****************************************************************************/
public class MQeListTransformer implements MQeTransformerInterface {

    public        static short    version[] = { 2, 0, 0, 10 }; 

    /**
     * The ordered list of names of transformers.
     */ 
    protected String[] transformerClassName = {

            // Add the class name, or the class alias name of your 
            // transformer classes here, in the order you wish them
            // to be presented with a message.
            // 
            // For example:
            // "myPackage.myTransformerClass1",
            // "myPackage.myTransformerClass2",
            // "myPackage.myTransformerClass3",
            // "myPackage.myTransformerClass4",

            // The base transformer class will accept any messages
            // and convert it. You may wish to remove this class
            // from your list, to highlight problems if your set
            // of transformers are passed a message they cannot
            // understand/handle.
            "com.ibm.mqe.mqbridge.MQeBaseTransformer"
        };

    /**
     * A list of transformers which are loaded and ready for use.
     */
    protected MQeTransformerInterface[] transformerInstance;

    /**
     * Tells the class implementing this interface any parameters
     * that may have been supplied with the transformer definition.
     *
     * <p>This example class could have used the parameters to build
     * its list of transformers, but to keep this example short, 
     * we ignore any parameters.
     *
     * @param params StringTokenizer containing the transformer
     * definition parameters, as specified by colon-separated
     * parameters in the configuration, supplied by the administrator.
     * @return void
     * @exception java.lang.Exception
     */
    public void activate(StringTokenizer userParameters 
    ) throws Exception {

        // How many transformers do we have ?
        int transformers = transformerClassName.length;

        // Every time we have our activate method call, we 
        // can assume our underlying parameters have changed,
        // so we should re-initialise everything.
        transformerInstance = new MQeTransformerInterface[transformers];

        // Only use the parameters if they are non-null.
        if (userParameters != null) {
            while (userParameters.hasMoreTokens()) {
                String parameter = userParameters.nextToken();
                // We could use the parameter here if we wanted.
                // But this example class does not need to use the
                // parameter list, so we ignore them.
            }
        }

        // Now we have our list of transformer class names, 
        // we should attempt to load each transformer...
        for (int i = 0; i < transformers; i += 1) {

            // Load each transformer...
            try {
                transformerInstance[i] = (MQeTransformerInterface) 
                        MQe.loadObject(transformerClassName[i]);
            } catch (Exception e) {
                // Failed to load transformer...
                System.out.println("MQeListTransformer: failed to load transformer "
                    + transformerClassName[i]
                );
                e.printStackTrace();
                throw e;
            }

            // Activate each transformer...
            try {
                // Note: The configuration parameters are NOT passed on to the 
                // smaller-scoped transformers in this example.
                // We could have passed them on...
                transformerInstance[i].activate(null);
            } catch (Exception e) {
                // Failure to activate a transformer...
                System.out.println("MQeListTransformer: failed to activate transformer "
                    + transformerClassName[i]
                );
                e.printStackTrace();
                throw e;
            }
        }

        // We now have an initialised array of transformers which are all
        // waiting to be used...
    }

    /**
     * Converts an MQ format message into an MQe format message.
     * @param msg The WMQ message object we are to transform.
     * @param remoteQMgrName The Destination Remote Queue Manager Name
     * (from the MQ Remote Queue Definition)
     * @param remoteQName The Destination Remote Queue Name (from the
     * MQ Remote Queue Definition).
     * @return The equivalent MQe format message.
     * @exception java.lang.Exception
     */
    public MQeMsgObject transform(MQMessage msg, String remoteQMgrName, String remoteQName
    ) throws Exception {

        // Create a storage space for our result.
        MQeMsgObject result = null;

        // Go through each transformer asking each in turn to try to 
        // convert the message...
        for (int i = 0;                       // Start with the first transformer.
            ((i < transformerClassName.length) // Work through until the end of the list
                || (result != null));         // Or until we have a result
            i += 1                           // Work through the list from front to back.
        ) {
            try {

                // Ask this transformer if it can convert the message...
                result = transformerInstance[i].transform(msg, remoteQMgrName, remoteQName
                        );

                if (result == null) {
                    // The transformer returned null, so that means
                    // it also did not understand the message.
                    throw new Exception("Transformer returned null !"
                            + "class=" + transformerClassName[i]
                        );
                }
            } catch (Exception e) {// The transformer could not transform the message...
                // move on to see if the next transformer in our list understands
                // this message...
                // Supress the exception.
            }
        }

        if (result == null) {
            // After all our efforts, we were still unable to convert the message
            // into its alternate format. Fail with an exception.
            throw new Exception("MQeListTransformer could not transform a message");
        }

        return result;
    }
        
    /**
     * Converts an MQe format message into an MQ format message.
     * @param The MQe message object we are to transform.
     * @param queue A reference to the MQe queue the put the message.
     * (e.g. For determining default expiry time etc...)
     * @param options A reference to an Integer object in which any required
     * MQPutMessageOptions.options values should be placed. 
     * @return The equivalent MQ format message.
     * @exception java.lang.Exception
     */
    public MQMessage transform(MQeMsgObject msg, MQeMQBridgeQueue queue, MQPutMessageOptions options
    ) throws Exception {

        // Create a storage space for our result.
        MQMessage result = null;

        // Go through each transformer asking each in turn to try to 
        // convert the message...
        for (int i = 0;                       // Start with the first transformer.
            ((i < transformerClassName.length) // Work through until the end of the list
                || (result != null));         // Or until we have a result
            i += 1                           // Work through the list from front to back.
        ) {
            try {
                // Ask this transformer if it can convert the message...
                result = transformerInstance[i].transform(msg, queue, options
                        );

                if (result == null) {
                    // The transformer returned null, so that means
                    // it also did not understand the message.
                    throw new Exception("Transformer returned null !"
                            + "class=" + transformerClassName[i]
                        );
                }
            } catch (Exception e) {// The transformer could not transform the message...
                // move on to see if the next transformer in our list understands
                // this message...
                // Suppress the exception.
            }
        }

        if (result == null) {
            // After all our efforts, we were still unable to convert the message
            // into its alternate format. Fail with an exception.
            throw new Exception("MQeListTransformer could not transform a message");
        }

        return result;
    }
}

