/*------------------------------------------------------------------------------*/
/*                    WebSphere MQ Everyplace for Multiplatforms                */
/*                                                                              */
/* Licensed Materials - Property of IBM                                         */
/*                                                                              */
/* 5765-E63                                                                     */
/*                                                                              */
/* (C) Copyright IBM Corp. 2002                                                 */
/*------------------------------------------------------------------------------*/
/*******************************************************************************
*                                                                              *
* Native methods for Unix Authentication (see Native.JavaUnix class)           *
* Sun JDK Version 1.1 using JNI                                                *
*                                                                              *
*******************************************************************************/

#include <jni.h>
#include <syslog.h>
#include <string.h>
#include <pwd.h>
#include <sys/types.h>

#ifdef USE_SHADOW
#include <shadow.h>
#endif

extern char *crypt(const char *, const char *);

#include "examples_nativecode_JavaUnix.h"


/* define the MQe (Windows NT) event types */
#define   MQe_Log_Success             0x00
#define   MQe_Log_Error               0x01
#define   MQe_Log_Warning             0x02
#define   MQe_Log_Information         0x04
#define   MQe_Log_Audit_Success       0x08
#define   MQe_Log_Audit_Failure       0x10


/* Only Solaris like globals in shared libs */
#ifdef __SOLARIS__
char *eventSource = NULL;
#endif


/*******************************************************************************
 * Helper Functions
 ******************************************************************************/

/**
 * convert bytes to Hex string
 */
char hexChars[] = {'0', '1', '2', '3', '4', '5', '6', '7',
                   '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
const char * bytesToChar(const char *byteStr, int len)
{
  int i, pos = 0;
  char *out;

  out = (char *) malloc(3*len);
  for(i=0; i < len; i++) {
    if (i > 0)
      out[pos++] = ' ';
    out[pos++] = hexChars[(byteStr[i]>>4) & 0x0f];
    out[pos++] = hexChars[byteStr[i] & 0x0f];
  }
  out[pos]='\0';
  return out;
}


/*******************************************************************************
* Logon a userid and return the Logon Handle                                   *
*******************************************************************************/
JNIEXPORT jboolean JNICALL Java_examples_nativecode_JavaUnix_logonUser
(JNIEnv *env, jobject obj, jstring juserID, jstring jpassword)
{
  const char *userid, *passwd;
  char *epasswd;
  struct passwd *pw;
#ifdef USE_SHADOW
  struct spwd *spwd;
  struct spwd *getspnam();
#endif

  /* convert the Java string arguments to C strings                          */
  userid = (*env)->GetStringUTFChars(env, juserID, NULL);
  passwd = (*env)->GetStringUTFChars(env, jpassword, NULL);


  if ( (pw = getpwnam(userid)) == NULL ) {
    (*env)->ReleaseStringUTFChars(env, juserID, userid);
    (*env)->ReleaseStringUTFChars(env, juserID, userid);
    return JNI_FALSE;
  }

#ifdef USE_SHADOW
  spwd = getspnam(userid);
  if (spwd)
    pw->pw_passwd = spwd->sp_pwdp;
#endif

  (*env)->ReleaseStringUTFChars(env, juserID, userid);

  /*
   * if no passwd is given, what to do ????
   */
  if ( pw->pw_passwd == '\0' ) {
    (*env)->ReleaseStringUTFChars(env, juserID, userid);
    return JNI_TRUE;
  }

  /*
   * If the encrypted password begins with a "!", the account
   * is locked and the user cannot login, even if they have
   * been "pre-authenticated."
   */
  if (pw->pw_passwd[0] == '!' || pw->pw_passwd[0] == '*') {
    (*env)->ReleaseStringUTFChars(env, jpassword, passwd);
    return JNI_FALSE;
  }

  epasswd = crypt(passwd, pw->pw_passwd);
  (*env)->ReleaseStringUTFChars(env, jpassword, passwd);
  if ( strcmp(epasswd, pw->pw_passwd) )
    return JNI_FALSE;
  else
    return JNI_TRUE;
}


/*******************************************************************************
* Logoff (Revert) and close the Logon Handle                                   *
*******************************************************************************/
JNIEXPORT void JNICALL Java_examples_nativecode_JavaUnix_logoffUser
(JNIEnv *env, jobject obj)
{
  /* nothing to do for Unix */

  return;
}

/*******************************************************************************
* RegisterEventSource                                                          *
*******************************************************************************/
JNIEXPORT void JNICALL Java_examples_nativecode_JavaUnix_registerEventSource
(JNIEnv *env, jobject obj, jstring jsourceName)
{
  const char *source;

  /* only used under Solaris */
#ifdef __SOLARIS__
  if (jsourceName != NULL)
    {
      source = (*env)->GetStringUTFChars(env, jsourceName, NULL);
      eventSource = (char *) malloc(strlen(source+1));
      strcpy(eventSource, source);
      eventSource[strlen(source)] = '\0';
      openlog(eventSource, 0, LOG_USER); /* register the source      */
      (*env)->ReleaseStringUTFChars(env, jsourceName, source);
    }
#endif
}

/*******************************************************************************
 * ReportEventSource                                                           *
 ******************************************************************************/
JNIEXPORT jboolean JNICALL Java_examples_nativecode_JavaUnix_reportEvent
(JNIEnv *env, jobject obj, jint    jeventType,
 jint    jeventID, jstring jdata, jbyteArray jbytes)
{
  const char *logDataStr = NULL;
  char *logStr, eventIDStr[20];
  jbyte *jb = NULL;
  jsize numBytes;
  jint  UNIXeventType;

  if (jdata != NULL)                          /* any string insert ?          */
    {
      /* does this handle string arrays correct ???? */
      logDataStr = (*env)->GetStringUTFChars(env, jdata, NULL);
    }
  if (jbytes != NULL)                        /* any byte data                */
    {
      jb = (*env)->GetByteArrayElements(env, jbytes, NULL);
      numBytes = (*env)->GetArrayLength(env, jbytes); /* number of bytes      */
      logDataStr = bytesToChar((const char *)jb, numBytes);
    }
  numBytes = strlen(logDataStr);

  /* insert jeventID in log string */
  sprintf(eventIDStr, "ID:%d; ", jeventID);
  if ( (logStr = (char *) malloc(numBytes + strlen(eventIDStr))) == NULL ) {
    if (jdata != NULL)                         /* if we allocated a C string   */
      (*env)->ReleaseStringUTFChars(env, jdata, logDataStr); /* we must free it*/
    if (jbytes != NULL)                        /* if we allocated a C array    */
      (*env)->ReleaseByteArrayElements(env, jbytes, jb, JNI_ABORT); /* free it*/
      return JNI_FALSE;
  }

  /* convert MQe event type to UNIX event type */
  switch (jeventType)
    {
  case MQe_Log_Error:
      UNIXeventType = LOG_ERR;
      break;
  case MQe_Log_Warning:
      UNIXeventType = LOG_WARNING;
      break;
  default:
      UNIXeventType = LOG_INFO;
    }

  strcpy(logStr, eventIDStr);
  strcat(logStr, logDataStr);
  syslog(UNIXeventType,           /* event type to log            */
         logStr);                 /* event log data               */

  if (jdata != NULL)                         /* if we allocated a C string   */
    (*env)->ReleaseStringUTFChars(env, jdata, logDataStr); /* we must free it*/
  if (jbytes != NULL)                        /* if we allocated a C array    */
    (*env)->ReleaseByteArrayElements(env, jbytes, jb, JNI_ABORT); /* free it*/

  return JNI_TRUE;
}

/*******************************************************************************
 * DeregisterEventSource                                                       *
 *******************************************************************************/
JNIEXPORT jboolean JNICALL Java_examples_nativecode_JavaUnix_deregisterEventSource
(JNIEnv *env, jobject obj)
{
  closelog();
  /* only used under Solaris */
#ifdef __SOLARIS__
  if (eventSource != NULL)
    free(eventSource);
#endif
  return JNI_TRUE;
}


