package examples.osgi.client;


// Version 1.3 mqe_java/source/examples/osgi/client/MQeClientApp.java, MQeOSGi, la200, la200-L030324.1
/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
 
import java.lang.*;
import com.ibm.mqe.*;
import com.ibm.mqe.adapters.*;
import com.ibm.mqe.registry.*;
import com.ibm.mqe.communications.*;
import com.ibm.mqe.administration.*;
import java.io.File;
import java.util.*;
import examples.osgi.admin.*;


/***
 * Example MQe Application. This checks to see if an MQeQueueManager is
 * already running in the JVM, if so it assumes it is running in the same
 * runtime as the server application and so uses that QueueManager.
 * If no QueueManager is detected then a new one is defined and started
 * in memory and a connection definition and remote queue definition are
 * setup to the server. Code is then run in a new thread which sends
 * a single message to the server.  
 * No checks are made to ensure the message is received.  
 * If we created a new QueueManager for the Client then it is stopped and
 * deleted when it is stopped by calling shutdownMQeClient().
 * 
 *  This Example uses a helper class:
 *   
 * <pre>
 *  . MQeAdmin - used to configure MQe Components. 
 *   	
 * </pre>
 *  This class contains the code to create and send a message.
 *  It is written this way to separate the MQe configuration code from Application
 *  code.
 * 
 *  This class is expected to be used in a particular way with the 5 main methods being
 *  called in a defined order as detailed below:
 * <pre>
 *  . MQeClientApp(String qmName, String regLocation)
 * 		This constructor must be used, supplying the QM name and registry location
 * 		to be used.
 *  . setServerAddressAndPort
 * 		sets the port and address where the MQeServerApp is running
 *  . setupMQeClient()
 * 		This defines and starts a QM in memory with default queues.
 *  . runJMSTest()
 * 		This creates a new MQeClient and starts it in a new thread.
 *  . shutdownMQeClient()
 * 		This shutsdown and deletes the QM.
 * </pre>
 *  A main method is also supplied to enable testing of this class outside of the 
 *  OSGI framework.
 * 
 * The MQeClientApp has been written so that it can be either run in the same
 * JVM as the MQeServerApp in which case it will share the underlying 
 * MQeQueueManager OR it can be run in its own JVM, perhaps even on a seperate
 * machine, in this case the MQeClientApp starts has its own QueueManager. In 
 * either case the MQeServerApp should be started first.
 * 
 *  By default the MQeTcpipHttpAdapter is used for comms and the MQeMemoryFieldsAdapter
 *  is used for registry storage. 
 * 
 *  The port and address where the MQeServerApp is running can be configured 
 *  by using the setReceiversAddressAndPort method.
 * 
 * 
 *
 **/


public class MQeClientApp implements Runnable {
	
    public static short[] version = { 2, 0, 0, 10 }; 

    private MQeAdmin mqeAdmin;
    private String myQmName = "myQM";
    private String myReg = "c:\\myQM\\";
    private boolean createdQM = false;
    private Thread testThread;
    private String targetQMName = "MQeServer";

    /**
     * Constructor to create a new MQeClientApp.
     * This sets the QueueManger name and Registry location
     * to be used by the MQeQueuemanager. It also 
     * creates a new MQeAdmin object to be used to setup MQe.
     */	
    public MQeClientApp(String qmName, String regLocation) {
		
        myQmName = qmName;
        myReg = regLocation;
        mqeAdmin = new MQeAdmin(myQmName, myReg);		
	
    }
	
    /**
     * Run Method
     * Creates and sends the message.
     * 
     */

    public void run() {
        String msgData = "This is a test message";
        String id = "1";
        //get a reference to the current QM
        MQeQueueManager qm = MQeQueueManager.getDefaultQueueManager();
				
        try {
            //Build a message 
            System.out.println("..Create a message with id: " + id + " and data:" + msgData);
            MQeMsgObject msgObj1 = new MQeMsgObject();

            msgObj1.putAscii("MsgData", msgData);
            msgObj1.putArrayOfByte(MQe.Msg_MsgID, MQe.asciiToByte(id));

            //put the message
            System.out.println("..Put the message to QM/queue: " + targetQMName + "/" + qm.System_Default_Queue_Name);
            qm.putMessage(targetQMName, qm.System_Default_Queue_Name, msgObj1, null, 0);		
        } catch (Exception e) {
            System.out.println("ERROR - " + e);
        }
	
    }

    /**
     * Method used to set the recievers address and port. If this method
     * is not used then then default is 8085 on 127.0.0.1.
     */
    public void setServerAddressAndPort(String targetQMIPAddress, String targetQMPort) {
        mqeAdmin.setServerAddressAndPort(targetQMIPAddress, targetQMPort);
	
    }

    /**
     * Simple method that just checks if there is already a QM in this JVM.
     * It is used by the app to check a QM is not running before starting a 
     * new one. Returns true if a QM already exists.
     */
    private boolean qmExists() {
        MQeQueueManager qm = MQeQueueManager.getDefaultQueueManager();
	
        if (!(qm == null)) {
            //a QM exists 
            return true;
			
        }
        return false;
    }

    /**
     * Checks to see if a QM is already running. If not it uses the MQeAdmin
     * to define a new QM, start it then create a connection defintion
     * to the servers QM. It then adds a remote queue defintion to the 
     * servers local queue.
     * If a QM is already running then it assumes the server has already been
     * started in the same JVM so uses that QM.
     */
    public void setupMQeClient() {
		
        //check to see if a QM is already running, if so we are to use it as we assume the user
        //wants us to run on the same QM as the server.
        if (!qmExists()) {		
		
            mqeAdmin.defineQM();
            mqeAdmin.startQM();
            mqeAdmin.createConnection();
            mqeAdmin.addRemoteQueueDefinition();
            createdQM = true;
        }
        //QM already exists, we don't need to do anything. Client will just 
        //send to localQM
			
		
		
	
    }

    /**
     * This starts the new MQeClientApp object in its own thread.
     * This uses the run method to send a single message to the reciever.
     * 
     */
    public void runTest() {
        //Kick off a new thread to do this as otherwise the acivators start method will not end until this
        //test has finished.
        testThread = new Thread(this);
        testThread.start();
    }
	
    /**
     * This stops the sending thread.
     * If we started our own QM in setupMQeClient() then we 
     *  shutsdown and deletes the QM. 
     */
    public void shutdownMQeClient() {
		
        //check the JMS test thread has stopped and if not stop it.
        if (testThread != null) {
            Thread temp = testThread;

            testThread = null;
            try {
                temp.join();
            } catch (InterruptedException e) {}
        }
		
        if (createdQM) {
            mqeAdmin.stopQM();
            mqeAdmin.deleteQM();
		
        }
        //Else Do nothing as the reciever will shutdown the QM
		
    }
	
    /**
     * This main method can be used to test the example outside of the OSGI
     * framework.  In the framework the same job is done by the Activators
     * start and stop methods.
     */

    public static void main(String[] args) {
		
        MQeClientApp dem = new MQeClientApp("MQeClient", "c:\\MQeClient");

        dem.setServerAddressAndPort("127.0.0.1", "8085");
        dem.setupMQeClient();
        dem.runTest();
        //wait 5 secs so msg has a chance to send.
        try {
            Thread.sleep(5000);
        } catch (Exception e) {
            System.out.println("ERROR - " + e);
        }
        dem.shutdownMQeClient();
		
    }

}
