package examples.queuemanager;


// Version mqe_java/source/examples/queuemanager/MQeServer.java, MQeExamples, la200, la200-L030626 1.16
/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/

import java.io.File;
import java.util.Hashtable;

import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeQueueManager;


/**
 * <p>To provide a simple example MQe server.
 *
 * <p>Components configured on a server at startup are :
 * <pre>
 *      ... Aliases
 *      ... Registry
 *      ... QueueManager
 *      ... Preload
 * </pre>
 * <p>Syntax :
 * <pre>  java examples.queuemanager.MQeServer  filename
 *      where filename is the name of the ini file containing the startup parameters.
 * <p>ExamplesMQeServer.ini file is the default file.
 * </pre>
 * <p><b>Note:</b>Only one server per JVM  at a time.
 * <p> This extends the {@link com.ibm.mqe.MQe} class.
 *
 * <p>------------------------------------------------------------------------------
 * <pre>
 * Example configuration data fields :
 *
 *
 *   [QueueManager]
 *      (ascii)Name=ServerQMgr
 *
 *   [Registry]
 *      (ascii)LocalRegType=FileRegistry
 *      (ascii)DirName=.\ServerQMgr\Registry\
 *      (ascii)Adapter=com.ibm.mqe.adapters.MQeDiskFieldsAdapter
 * </pre>
 */
public class MQeServer extends MQe {
    public static short version[] = { 2, 0, 0, 10 };  

    public static MQeQueueManager queueManager;
    public static Hashtable loadTable;
    public static boolean initialised = false;
    public static MQeFields sections = null;

    /**
     * If you want to start MQe tracing, then set this flag to true.
     */
    private static final boolean START_MQE_TRACE = false;

    /**
     * Null constructor.
     */
    public MQeServer() {
        super();
    }

    /**
     * Constructor with configuration parameter.
     * 
     * @param thisConfig Configuration options.
     * @exception java.lang.Exception
     */
    public MQeServer(MQeFields thisConfig) throws Exception {
        super();
        init(thisConfig);
    }

    /**
     * Starts an MQeServer.
     *
     * @param iniFileName A String containing the ini file name containing startup parms.
     * @exception java.lang.Exception
     */
    public MQeServer(String iniFileName) throws Exception {
        this(iniFileName, false);
    }

    /**
     * Start an MQeServer optionally displaying things on stdout as we go.
     * 
     * @param iniFileName A String containing the ini file name containing startup parms.
     * @param supressStdout A Boolean. if True, the ini file contents are not displayed on the
     * Std Output. Displayed, if a value of False is passed in this parameter.
     * @exception java.lang.Exception
     */
    public MQeServer(String iniFileName, boolean suppressStdout) throws Exception {
        super();
        // Load the ini file
        init(MQeQueueManagerUtils.loadConfigFile(iniFileName), suppressStdout);
    }

    /**
     * Activate and de-activate the server.
     * 
     * @param start A boolean, True to activate and False to deactivate.
     * @return void
     * @exception java.lang.Exception
     */
    public void activate(boolean Start) throws Exception {
        // activate ?
        if (Start) {
            // if already not active ..
            if (!initialised) {
                // assign any class aliases
                MQeQueueManagerUtils.processAlias(sections);
                // check for any pre-loaded classes
                loadTable = MQeQueueManagerUtils.processPreLoad(sections);
                // only once
                initialised = true;
            }
            
            // setup and activate the queue manager
            queueManager = MQeQueueManagerUtils.processQueueManager(sections);
            // setup and activate the listener for incomming connections
            MQeQueueManagerUtils.processListenerSections(sections, queueManager);
        } else {
            //... stop the server
            //  Close the queue manager
            if (queueManager != null) {
                queueManager.close();
            }
            
            // release object
            queueManager = null;
            initialised = false;
        }
    }

    /**
     * Stops an MQeServer.
     * 
     * @return void
     * @exception java.lang.Exception
     */
    public void close() throws Exception {
        // de-activate the server
        activate(false);

        // Stop trace if we started it.
        if (START_MQE_TRACE) {
            MQeQueueManagerUtils.traceOff();
        }
    }

    /**
     * First stage setup.
     * 
     * @param parms MQeFields object.
     * @return void
     * @exception java.lang.Exception
     */
    public void init(MQeFields parms) throws Exception {
        init(parms, false);
    }

    /**
     * First stage setup.
     * 
     * @param parms MQeFields object.
     * @param supressStdout A Boolean. if True, the ini file contents are not displayed on the
     * Std Output. Displayed, if a value of False is passed in this parameter.
     * @exception java.lang.Exception
     */
    public void init(MQeFields parms, boolean suppressStdout) throws Exception {
        // Only one server at a time
        if (initialised) {
            throw new Exception("Server already running");
        }
        // Remeber startup parms
        sections = parms;
        // set any alias names
        MQeQueueManagerUtils.processAlias(sections);

        // Set any java system properties which have been specified in the .ini file.
        MQeQueueManagerUtils.processSystemProperties(sections);

        if (START_MQE_TRACE) {
            MQeQueueManagerUtils.traceOn();
        }

        // Display the startup parameters
        if (!suppressStdout) {
            System.out.println(sections.dumpToString("#1\t=\t#2\r\n"));
        }
    }

    /**
     * Invoked from the command line.
     */
    public static void main(String[] args) {
        // incase of error
        try { // default file
            String FileName = "." + File.separator + "ExamplesMQeServer.ini";

            if (args.length == 0) {
                System.out.println("Usage : java examples.queuemanager.MQeServer '.ini filename' ");
                System.out.println("Must specify the <.ini> filename.");
            } else {
                FileName = args[0];
                
                // Start the server
                new MQeServer(FileName).activate(true);
            }
        } //  error occured
        catch (Exception e) {
            e.printStackTrace();
        }
    }
}
