package examples.queuemanager;


// Version mqe_java/source/examples/queuemanager/MQeServlet.java, MQeExamples, la200, la200-L030731 1.17
/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
import java.io.*;
import java.util.*;
import javax.servlet.*;

import javax.servlet.http.*;

import com.ibm.mqe.*;
import com.ibm.mqe.communications.*;
import examples.queuemanager.*;


/**
 * <p>To provide a simple MQe servlet.
 *
 * <p>Components configured on a servlet at startup are :
 * <pre>
 *      ... Listener (only TimeInterval parameter)
 *      ... Registry
 *      ... QueueManager
 * </pre>
 * <p>This class extends HttpServlet, the servlet base class.
 * <p>------------------------------------------------------------------------------
 * <pre>
 * Example configuration data fields :
 *
 *   [Listener]
 *   (int)TimeInterval=300000
 *   (int)MaxChannels=0
 *   (ascii)Name=ServletListener
 *   
 *   [QueueManager]
 *   (ascii)Name=ServerQMgr
 *   
 *   [Registry]
 *   (ascii)LocalRegType=FileRegistry
 *   (ascii)DirName=c:\la000\ServerQMgr\Registry\
 *   (ascii)Adapter=com.ibm.mqe.adapters.MQeDiskFieldsAdapter
 *   
 *
 *   See ExampleMQeServer.ini for an example startup paramters file
 * </pre>
 */
public class MQeServlet extends HttpServlet {
    public static short version[] = { 2, 0, 0, 10 }; 

    // Defaults used if parameter not passed to servlet
    public String defaultStartupInifile = "./ExampleMQeServer.ini";
    public String defaultQueueStore = "./Queues";

    protected MQe mqe = new MQe();
    protected MQeServletWrapper wrapper = new MQeServletWrapper(); // Channel mgr object
    protected MQeQueueManager queueManager = null; // Queue manager object

    /** 
     * The environment in which this servlet is run may instantiate more than one instance
     * of the servlet.  It is vital the first instance of the servlet instantiates the queue
     * manager, and only the first servlet instance.  Conversley no servlet should close the
     * queue manager until all servlets are closed, so this should be the responsibility of
     * the last servlet instance.  In order to achieve this we need to synchronize around
     * a static lock and keep a count of how many references there are to the queue manager.
     */
    private static Object mqeLock = new Object();
    private static int numberOfReferences = 0;

    /**
     * called when a request is made to stop the servlet.
     * 
     * @return void
     */
    public void destroy() {

        // make sure the references are decremented, and if necessary the
        // queue manager closed is all done within a synchronized block so
        // we don't close the queue manager as another thread tries to open it
        synchronized (mqeLock) {
            // Decrement the number of references to the queue manager
            numberOfReferences--;

            if (0 == numberOfReferences) {
                try {
                    // if there are no servlets using the queue manager
                    // close it down
                    queueManager.closeImmediate();

                } catch (Exception ex) {}
            }
        }

        // stop the wrapper thread.
        wrapper.runnableRequestStop();
        queueManager = null;

        MQeTrace.trace(this, (short) 0, MQeTrace.GROUP_INFO);

        // Close any active trace
        MQeQueueManagerUtils.traceOff();
        mqe = null;

        // log the event, if necessary
        // log("MQe servlet stopped");

    }

    /**
     * Handle POST.
     * 
     * @param request HttpServletRequest object
     * @param response HttpServletResponse object
     * @return void
     * @exception java.io.IOException
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws IOException {
        byte[] httpInData = null;

        // any request to process ?
        if (request == null)
            throw new IOException("Invalid request");

        try {
            // get the output stream
            ServletOutputStream httpOut = response.getOutputStream();

            // get the input stream
            ServletInputStream httpIn = request.getInputStream();

            // get the length of data, -1 is returned if no content length,
            // this can happen if a chunked message has been sent and the 
            // server does not put the content length in when it has unchunked
            // the request.
            int max_length_of_data = request.getContentLength(); // data length

            httpInData = read(httpIn, max_length_of_data);
            
            // process the request
            byte[] httpOutData = wrapper.processPost(httpInData);

            // not all application servers set the content length so we shall
            // do so explicitly
            response.setContentLength(httpOutData.length);

            // Pass back the response
            httpOut.write(httpOutData);
        } catch (Exception e) {
            // pass it on ...
            throw new IOException("Request failed" + e);
        }
    }

    /**
     * Servlet Initialisation.
     * 
     * @param sc ServletConfig object
     * @return void
     * @exception ServletException
     */
    public void init(ServletConfig sc) throws ServletException {

        super.init(sc);
        synchronized (mqeLock) {
            // first of all check to see if there is a default queue manager around.
            // if there is we shall return that, if not we need to start one and if
            // that fails we need to configure one
            queueManager = MQeQueueManager.getDefaultQueueManager();

            if (null == queueManager) {

                try {

                    // get the initialization file that should hold the base
                    // information we need The alternative is to pass all the info in
                    // using parameters
                    
                    String startupIni;

                    // The getInitParameter is supplied by the javax.servlet.GenericServlet
                    // class and will retrieve information from parameters defined in your
                    // server to be passed to this servlet
                    if ((startupIni = getInitParameter("Startup")) == null) {
                        startupIni = defaultStartupInifile;
                    }

                    /**
                     * load the information from the initialization file into a
                     * MQeFields object
                     */
                    MQeFields sections = MQeQueueManagerUtils.loadConfigFile(startupIni);

                    /**
                     * get the location to be used for a queue store
                     */
                    String queueStore;

                    if (null == (queueStore = getInitParameter("queueStore"))) {
                        queueStore = defaultQueueStore;
                    }
                    try {
                        // this call will fail if the queue manager has no Registry
                        // configured
                        queueManager = MQeQueueManagerUtils.processQueueManager(sections);
                    } catch (MQeException e) {
                        if (e.Except_QMgr_NotConfigured == e.code()) {
                            // no queue manager configured so configure one 
                            MQeQueueManagerConfigure qmConfig =
                                new MQeQueueManagerConfigure(sections, "MsgLog:" + queueStore);

                            qmConfig.defineQueueManager();
                            qmConfig.defineDefaultSystemQueue();
                            qmConfig.defineDefaultDeadLetterQueue();
                            qmConfig.defineDefaultAdminReplyQueue();
                            qmConfig.defineDefaultAdminQueue();
                            qmConfig.setChannelTimeout(0L);
                            qmConfig.close();
                        }
                    }

                    // log the event, if necessary 
                    // log("MQe servlet parameters:\r\n" + sections.dumpToString("#1\t=\t#2\r\n"));

                    // Uncomment the following line to start trace before the queue manager is started
                    //      MQeQueueManagerUtils.traceOn("MQeServlet Trace", null);

                    // Servlet initialisation complet
                    numberOfReferences++;
                    log("MQe servlet started");
                    wrapper.startWrapper();

                } catch (Exception e) {
                    // log("MQe servlet startup failed: " + e);

                    throw new ServletException(e.toString());
                }
            }
        }

    }

    /**
     * Read the input stream.
     * 
     * @param in InputStream, the input stream
     * @param bytes An integer, number of bytes to read.
     * @return byte[] containing the message
     * @exception java.io.IOException The exception description.
     */
    public byte[] read(InputStream in, int bytes) throws IOException {

        byte[] data = null;
        // Bytes read so far
        int i = 0;

        if (-1 == bytes) {
            // there was no content length so we will have to read the input
            // stream a byte at a time until we get to the EOF
            ByteArrayOutputStream temp = new ByteArrayOutputStream();
            int intChar;

            while (true) {
                intChar = in.read();
                if (intChar == -1)
                    break;
                temp.write(intChar);
            }
            data = temp.toByteArray();
        } else {
            // we know how long the data is so we can read it in direct to a
            // byte array
            data = new byte[bytes];
            // Carry on getting bytes until we have the required number
            while (i < bytes) {
                // Bytes read on this read
                int numBytes = 0;

                // read next batch of bytes
                numBytes = in.read(data, i, bytes - i);

                // Check for possible end of file
                if (numBytes < 0) {
                    throw new EOFException();
                }
                // Increment bytes read count
                i = i + numBytes;
            }
        }

        return data;

    }

}
