// Version 1.11 mqe_java/source/examples/rules/ExampleQueueManagerRules.java, MQeExamples, la200, la200-L030324.1

/* -----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms                *
 *                                                                              *
 * Licensed Materials - Property of IBM                                         *
 *                                                                              *
 * 5765-E63                                                                     *
 *                                                                              *
 * (C) Copyright IBM Corp. 1999                                                 *
 * -----------------------------------------------------------------------------*/
package examples.rules;


import com.ibm.mqe.*;


/*     
 * Example queue manager rules class.                                      
 * 
 * <p>This starts a background thread to trigger transmission of messages        
 * at a specific interval. (15 seconds).
 * <p>The background thread will take about 1 second to exit when the queue
 * manager closes down.
 * <p>Extends MQeQueueManagerRule,the base MQe Queue Manater Rule class. 
 */
public class ExampleQueueManagerRules extends MQeQueueManagerRule
    implements Runnable {

    // Which version of MQe was this example code taken from ?                                       	
    public static short version[] = { 2, 0, 0, 10 }; 
    
    // default interval between triggers is 15 seconds                          
    private static final long MILLISECS_BETWEEN_TRIGGER_TRANSMITS = 15000;
    
    // interval between which we check whether the queue manager is closing down.
    private static final long MILLISECS_BETWEEN_CLOSE_CHECKS = 1000;
    
    // Max wait of ten seconds to kill off the background thread when 
    // the queue manager is closing down.
    private static final long MAX_WAIT_FOR_BACKGROUND_THREAD_MILLISECONDS = 10000;
    
    // Reference to the control block used to communicate with the background thread
    // which does a sleep-trigger-sleep-trigger loop.
    // Note that freeing such blocks for garbage collection will not stop the thread
    // to which it refers.
    private Thread th = null;
    
    // Flag which is set when shutdown of the background thread is required.
    // Volatile because the thread using the flag and the thread setting it to true
    // are different threads, and it is important that the flag is not held in 
    // CPU registers, or one thread will see a different value to the other.
    private volatile boolean toldToStop = false;

    /*
     * Called when the Queue manager is activated. 
     * @return void
     * @exception java.lang.Exception
     */
    public void queueManagerActivate() throws Exception {
        super.queueManagerActivate();
        
        // background thread which triggers transmission                        
        th = new Thread(this, "TriggerThread");

        // Make sure the thread doesn't stop the moment we tell it to start.
        toldToStop = false;
        
        // start timer thread  
        th.start();                     
                                 
    }

    /*
     * Called when a Queue manager Close is called.
     * @return void
     * @exception java.lang.Exception
     */
    public void queueManagerClose() throws Exception {
        super.queueManagerClose();
        
        // Tell the background thread to stop, as the queue manager is closing now.
        toldToStop = true;
        
        // Now wait for the background thread, if it's not already stopped.
        if (th != null) {
            try { 
                // Only wait for a certain time before giving up and timing out.
                th.join(MAX_WAIT_FOR_BACKGROUND_THREAD_MILLISECONDS);
     
                // Free up the thread control block for garbage collection.          	
                th = null;
            } catch (InterruptedException e) {// Don't propogate the exception. 
                // Assume that the thread will stop shortly anyway.
            }
        }                  
    }

    /*
     * Background thread run method.                                            
     * Triggers transmission every interval until thread is stopped.           
     */
    public void run() {
		
        // Do a sleep-trigger-sleep-trigger loop until the queue manager closes or we get
        // an exception.
        while (!toldToStop) {
			
            try {           		
        		
                // Count down until we've waited enough
                // We do a tight loop with a smaller granularity because 
                // otherwise we would stop a queue manager from closing quickly
                long timeToWait = MILLISECS_BETWEEN_TRIGGER_TRANSMITS;

                while (timeToWait > 0 && !toldToStop) {
        		
                    // sleep for specified interval                                 
                    Thread.sleep(MILLISECS_BETWEEN_CLOSE_CHECKS);
            		
                    // We've waited for some time. Account for this in the overall wait.
                    timeToWait -= MILLISECS_BETWEEN_CLOSE_CHECKS;
                }
            	
                if (!toldToStop) {
                    // trigger transmission on QMgr (which is rule owner)       
                    ((MQeQueueManager) owner).triggerTransmission();
                }
            	
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
}
