// Version mqe_java/source/examples/security/createSecureQueue.java, MQeSecurity, la200, la200-L030324.1 1.2

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5724-C77                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/* -----------------------------------------------------------------------------*/

package examples.security;


import java.io.File;
import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.MQePrivateClient;


/** createSecureQueue.java
 * <p>This creates a secure queue on an existing queue manager. The queue is
 * created with an authenticator, cryptor, compressor and attribute rule.
 * The queue manager must have a private registry, so that the queue can be
 * given a cryptor.
 *
 * <p>The program requires two command line parameters.
 *
 * <p>The first parameter is a configuration file for the queue manager. This
 * is used to start the queue manager as a client.
 *
 * <p>The second parameter is the PIN for the queue manager's private
 * registry.
 *
 **/

public class createSecureQueue {
    public static short  version[] = { 2, 0, 0, 10 };  // MQe version

    // the name of the queue
    String qName = "protQueue";

    // define the attributes we want the queue to have. These are defined by
    // their class names.
    String cryptorType = "com.ibm.mqe.attributes.MQeDESCryptor";
    String compressorType = "com.ibm.mqe.attributes.MQeGZIPCompressor";
    String authenticatorType = "examples.attributes.NTAuthenticator";
    String attributeRule = "com.ibm.mqe.MQeAttributeRule";

    //local variables
    MQePrivateClient client;
    MQeQueueManager  clientQM;
    String           clientQMName;
    MQeQueueAdminMsg msg;

    /**
     * open the queue manager as a client
     *
     * @param configFile the configuration (.ini) file for the queue manager
     * @param qmPIN      the PIN for the queue manager's registry
     * @exception  java.lang.Exception  propagated from invoked methods
     **/
    void openQM(String configFile, String qmPIN) throws Exception {
        // start the queue manager as a client
        client = new MQePrivateClient(configFile, qmPIN, null, null);

        //save the queue manager and its name
        clientQM = client.queueManager;
        clientQMName = clientQM.getName();
    }

    /**
     * create the admin message to add the queue attributes
     *
     * @exception  java.lang.Exception  propagated from invoked methods
     **/
    void createAdminMsg() throws Exception {
        // the file descriptor
        String FileDesc = "MsgLog:.";

        // create an Admin msg to add the queue
        msg = new MQeQueueAdminMsg();
        msg.setTargetQMgr(clientQMName);
        msg.setName(clientQMName, qName);
        msg.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);
        msg.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
        msg.putAscii(MQe.Msg_ReplyToQMgr, clientQMName);
        msg.putArrayOfByte(MQe.Msg_CorrelID, Long.toHexString(clientQM.uniqueValue()).getBytes());

        // define parameter values for the queue
        MQeFields parms = new MQeFields();

        parms.putUnicode(msg.Queue_Description, "DES protected queue");
        parms.putAscii(msg.Queue_FileDesc, FileDesc);

        // this is where we specify the queue attributes
        parms.putAscii(msg.Queue_Cryptor, cryptorType);
        parms.putAscii(msg.Queue_Compressor, compressorType);
        parms.putAscii(msg.Queue_Authenticator, authenticatorType);
        parms.putAscii(msg.Queue_AttrRule, attributeRule);

        //add the parameters to the message
        msg.create(parms);
    }

    /**
     * send the admin message to the client queue manager
     *
     * @exception  java.lang.Exception  propagated from invoked methods
     **/
    void sendAdminMsg() throws Exception {
        // send the Admin msg
        System.out.println("putting Admin Msg to QM/queue:" + clientQMName + "/" + MQe.Admin_Queue_Name);
        clientQM.putMessage(clientQMName, MQe.Admin_Queue_Name, msg, null, 0);
    }

    /**
     * wait for a reply message and process it to determine success or failure
     *
     * @exception  java.lang.Exception  propagated from invoked methods
     **/
    void processReply() throws Exception {
        // use the CorrelID to create a filter for the reply message
        MQeFields replyFilter = new MQeFields();

        replyFilter.putArrayOfByte(MQe.Msg_CorrelID, msg.getArrayOfByte(MQe.Msg_CorrelID));

        // get the Admin Reply msg
        MQeMsgObject reply = clientQM.waitForMessage(clientQMName,
                MQe.Admin_Reply_Queue_Name,
                replyFilter,
                null,
                0,
                3000);

        if (reply instanceof MQeAdminMsg) {
            MQeAdminMsg adminReply = (MQeAdminMsg) reply;

            System.out.println("Admin Reply Msg received");
            if (adminReply.getRC() == MQeAdminMsg.RC_Success)
                System.out.println("Queue added OK");
            else
                System.out.println("create Queue failed:" + adminReply.getReason());
        } else
            System.out.println("reply message is not an admin message");
    }

    /**
     * close the queue manager
     *
     * @exception  java.lang.Exception  propagated from invoked method
     **/
    void close() throws Exception {
        clientQM.close();
    }

    /**
     * main method.
     *
     * @param  args  The command line arguments. The first is a configuration
     *               (.ini) file for the queue manager, the second is the PIN
     *               for the queue manager's private registry.
     *
     **/
    public static void main(String[] args) {
        createSecureQueue secQueue = new createSecureQueue();

        // check the command line arguments
        if (args.length < 2)
            System.err.println("usage: createSecureQueue configFile qmPIN");
        else {
            try {
                secQueue.openQM(args[0], args[1]);
                secQueue.createAdminMsg();
                secQueue.sendAdminMsg();
                secQueue.processReply();
                secQueue.close();
            } catch (Exception e) {
                System.out.println("Exception caught:" + e);
            }
        }
    }

}

